@@ -2,11 +2,6 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifdef UNSAFE_BUFFERS_BUILD
-// TODO(crbug.com/40284755): Remove this and spanify to fix the errors.
-#pragma allow_unsafe_buffers
-#endif
-
 #include "base/debug/stack_trace.h"
 
 #include <windows.h>
@@ -22,7 +17,6 @@
 #include "base/files/file_path.h"
 #include "base/logging.h"
 #include "base/memory/singleton.h"
-#include "base/ranges/algorithm.h"
 #include "base/strings/strcat_win.h"
 #include "base/strings/string_util.h"
 #include "base/synchronization/lock.h"
@@ -117,16 +111,17 @@ long WINAPI StackDumpExceptionFilter(EXCEPTION_POINTERS* info) {
   std::cerr << "\n";
 
   debug::StackTrace(info).Print();
-  if (g_previous_filter)
+  if (g_previous_filter) {
     return g_previous_filter(info);
+  }
   return EXCEPTION_CONTINUE_SEARCH;
 }
 
 FilePath GetExePath() {
-  wchar_t system_buffer[MAX_PATH];
-  GetModuleFileName(NULL, system_buffer, MAX_PATH);
-  system_buffer[MAX_PATH - 1] = L'\0';
-  return FilePath(system_buffer);
+  std::array<wchar_t, MAX_PATH> system_buffer;
+  GetModuleFileName(NULL, system_buffer.data(), system_buffer.size());
+  system_buffer.back() = L'\0';
+  return FilePath(system_buffer.data());
 }
 
 constexpr size_t kSymInitializeRetryCount = 3;
@@ -137,12 +132,14 @@ constexpr size_t kSymInitializeRetryCount = 3;
 // See crbug.com/1339753
 bool SymInitializeWrapper(HANDLE handle, BOOL invade_process) {
   for (size_t i = 0; i < kSymInitializeRetryCount; ++i) {
-    if (SymInitialize(handle, nullptr, invade_process))
+    if (SymInitialize(handle, nullptr, invade_process)) {
       return true;
+    }
 
     g_init_error = GetLastError();
-    if (g_init_error != g_status_info_length_mismatch)
+    if (g_init_error != g_status_info_length_mismatch) {
       return false;
+    }
   }
   DLOG(ERROR) << "SymInitialize failed repeatedly.";
   return false;
@@ -150,13 +147,15 @@ bool SymInitializeWrapper(HANDLE handle, BOOL invade_process) {
 
 bool SymInitializeCurrentProc() {
   const HANDLE current_process = GetCurrentProcess();
-  if (SymInitializeWrapper(current_process, TRUE))
+  if (SymInitializeWrapper(current_process, TRUE)) {
     return true;
+  }
 
   // g_init_error is updated by SymInitializeWrapper.
   // No need to do "g_init_error = GetLastError()" here.
-  if (g_init_error != ERROR_INVALID_PARAMETER)
+  if (g_init_error != ERROR_INVALID_PARAMETER) {
     return false;
+  }
 
   // SymInitialize() can fail with ERROR_INVALID_PARAMETER when something has
   // already called SymInitialize() in this process. For example, when absl
@@ -164,8 +163,9 @@ bool SymInitializeCurrentProc() {
   // almost immediately after startup. In such a case, try to reinit to see if
   // that succeeds.
   SymCleanup(current_process);
-  if (SymInitializeWrapper(current_process, TRUE))
+  if (SymInitializeWrapper(current_process, TRUE)) {
     return true;
+  }
 
   return false;
 }
@@ -180,9 +180,7 @@ bool InitializeSymbols() {
   g_initialized_symbols = true;
   // Defer symbol load until they're needed, use undecorated names, and get line
   // numbers.
-  SymSetOptions(SYMOPT_DEFERRED_LOADS |
-                SYMOPT_UNDNAME |
-                SYMOPT_LOAD_LINES);
+  SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_UNDNAME | SYMOPT_LOAD_LINES);
   if (!SymInitializeCurrentProc()) {
     // When it fails, we should not call debugbreak since it kills the current
     // process (prevents future tests from running or kills the browser
@@ -239,8 +237,7 @@ class SymbolContext {
   static SymbolContext* GetInstance() {
     // We use a leaky singleton because code may call this during process
     // termination.
-    return
-      Singleton<SymbolContext, LeakySingletonTraits<SymbolContext> >::get();
+    return Singleton<SymbolContext, LeakySingletonTraits<SymbolContext>>::get();
   }
 
   SymbolContext(const SymbolContext&) = delete;
@@ -266,11 +263,9 @@ class SymbolContext {
 
       // Code adapted from MSDN example:
       // http://msdn.microsoft.com/en-us/library/ms680578(VS.85).aspx
-      ULONG64 buffer[
-        (sizeof(SYMBOL_INFO) +
-          kMaxNameLength * sizeof(wchar_t) +
-          sizeof(ULONG64) - 1) /
-        sizeof(ULONG64)];
+      ULONG64 buffer[(sizeof(SYMBOL_INFO) + kMaxNameLength * sizeof(wchar_t) +
+                      sizeof(ULONG64) - 1) /
+                     sizeof(ULONG64)];
       memset(buffer, 0, sizeof(buffer));
 
       // Initialize symbol information retrieval structures.
@@ -278,8 +273,8 @@ class SymbolContext {
       PSYMBOL_INFO symbol = reinterpret_cast<PSYMBOL_INFO>(&buffer[0]);
       symbol->SizeOfStruct = sizeof(SYMBOL_INFO);
       symbol->MaxNameLen = kMaxNameLength - 1;
-      BOOL has_symbol = SymFromAddr(GetCurrentProcess(), frame,
-                                    &sym_displacement, symbol);
+      BOOL has_symbol =
+          SymFromAddr(GetCurrentProcess(), frame, &sym_displacement, symbol);
 
       // Attempt to retrieve line number information.
       DWORD line_displacement = 0;
@@ -307,9 +302,7 @@ class SymbolContext {
  private:
   friend struct DefaultSingletonTraits<SymbolContext>;
 
-  SymbolContext() {
-    InitializeSymbols();
-  }
+  SymbolContext() { InitializeSymbols(); }
 
   Lock lock_;
 };
@@ -344,7 +337,7 @@ StackTrace::StackTrace(const CONTEXT* context) {
 void StackTrace::InitTrace(const CONTEXT* context_record) {
   if (ShouldSuppressOutput()) {
     CHECK_EQ(count_, 0U);
-    base::ranges::fill(trace_, nullptr);
+    std::ranges::fill(trace_, nullptr);
     return;
   }
 
@@ -389,7 +382,7 @@ void StackTrace::InitTrace(const CONTEXT* context_record) {
     trace_[count_++] = reinterpret_cast<void*>(stack_frame.AddrPC.Offset);
   }
 
-  base::ranges::fill(span(trace_).last(trace_.size() - count_), nullptr);
+  std::ranges::fill(span(trace_).last(trace_.size() - count_), nullptr);
 }
 
 // static
