@@ -251,7 +251,7 @@ TEST_P(FileDescriptorWatcherTest, WatchReadableOneByte) {
   WriteByte();
   RunLoop run_loop;
   EXPECT_CALL(mock_, ReadableCallback())
-      .WillOnce(testing::Invoke([this, &run_loop]() {
+      .WillOnce(testing::Invoke([this, &run_loop] {
         ReadByte();
         run_loop.Quit();
       }));
@@ -271,8 +271,8 @@ TEST_P(FileDescriptorWatcherTest, WatchReadableTwoBytes) {
   WriteByte();
   RunLoop run_loop;
   EXPECT_CALL(mock_, ReadableCallback())
-      .WillOnce(testing::Invoke([this]() { ReadByte(); }))
-      .WillOnce(testing::Invoke([this, &run_loop]() {
+      .WillOnce(testing::Invoke([this] { ReadByte(); }))
+      .WillOnce(testing::Invoke([this, &run_loop] {
         ReadByte();
         run_loop.Quit();
       }));
@@ -292,11 +292,11 @@ TEST_P(FileDescriptorWatcherTest, WatchReadableByteWrittenFromCallback) {
   WriteByte();
   RunLoop run_loop;
   EXPECT_CALL(mock_, ReadableCallback())
-      .WillOnce(testing::Invoke([this]() {
+      .WillOnce(testing::Invoke([this] {
         ReadByte();
         WriteByte();
       }))
-      .WillOnce(testing::Invoke([this, &run_loop]() {
+      .WillOnce(testing::Invoke([this, &run_loop] {
         ReadByte();
         run_loop.Quit();
       }));
@@ -315,7 +315,7 @@ TEST_P(FileDescriptorWatcherTest, DeleteControllerFromCallback) {
   WriteByte();
   RunLoop run_loop;
   EXPECT_CALL(mock_, ReadableCallback())
-      .WillOnce(testing::Invoke([&run_loop, &controller]() {
+      .WillOnce(testing::Invoke([&run_loop, &controller] {
         controller = nullptr;
         run_loop.Quit();
       }));
@@ -385,14 +385,14 @@ TEST_P(FileDescriptorWatcherTest,
 
   RunLoop run_loop;
   EXPECT_CALL(mock_, ReadableCallback())
-      .WillOnce(testing::Invoke([this, &controller]() {
+      .WillOnce(testing::Invoke([this, &controller] {
         ReadByte();
         CloseWriteFd2();
         controller.reset();
       }));
   EXPECT_CALL(mock_, ReadableCallback2())
-      .WillOnce(testing::Invoke([this]() { ReadByte2(); }))
-      .WillOnce(testing::Invoke([this, &controller2, &run_loop]() {
+      .WillOnce(testing::Invoke([this] { ReadByte2(); }))
+      .WillOnce(testing::Invoke([this, &controller2, &run_loop] {
         ReadByte2();
         controller2.reset();
         run_loop.Quit();
