@@ -8,6 +8,7 @@
 #include <optional>
 
 #include "base/base_export.h"
+#include "base/compiler_specific.h"
 #include "base/files/file_path.h"
 
 namespace base {
@@ -30,7 +31,14 @@ class BASE_EXPORT SafeBaseName {
 
   // Same as above, but takes a StringPieceType for convenience.
   static std::optional<SafeBaseName> Create(FilePath::StringPieceType);
-  const FilePath& path() const { return path_; }
+  const FilePath& path() const LIFETIME_BOUND { return path_; }
+
+  // Convenience functions.
+  const std::string AsUTF8Unsafe() const { return path_.AsUTF8Unsafe(); }
+  const FilePath::StringType& value() const LIFETIME_BOUND {
+    return path_.value();
+  }
+  [[nodiscard]] bool empty() const { return path_.empty(); }
 
   bool operator==(const SafeBaseName& that) const;
 
