@@ -9,11 +9,11 @@
 
 #include "base/memory/platform_shared_memory_mapper.h"
 
+#include <sys/mman.h>
+
 #include "base/logging.h"
 #include "base/numerics/safe_conversions.h"
 
-#include <sys/mman.h>
-
 namespace base {
 
 std::optional<span<uint8_t>> PlatformSharedMemoryMapper::Map(
@@ -33,12 +33,13 @@ std::optional<span<uint8_t>> PlatformSharedMemoryMapper::Map(
     return std::nullopt;
   }
 
-  return make_span(static_cast<uint8_t*>(address), size);
+  return span(static_cast<uint8_t*>(address), size);
 }
 
 void PlatformSharedMemoryMapper::Unmap(span<uint8_t> mapping) {
-  if (munmap(mapping.data(), mapping.size()) < 0)
+  if (munmap(mapping.data(), mapping.size()) < 0) {
     DPLOG(ERROR) << "munmap";
+  }
 }
 
 }  // namespace base
