@@ -1,11 +1,13 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/memory/platform_shared_memory_region.h"
 
-#include "base/logging.h"
-#include "base/memory/shared_memory.h"
+#include <algorithm>
+#include <tuple>
+
+#include "base/check.h"
 #include "base/memory/shared_memory_mapping.h"
 #include "base/process/process_metrics.h"
 #include "base/system/sys_info.h"
@@ -14,22 +16,25 @@
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-#if defined(OS_MACOSX) && !defined(OS_IOS)
-#include <mach/mach_vm.h>
+#if BUILDFLAG(IS_APPLE)
+#include <mach/vm_map.h>
 #include <sys/mman.h>
-#elif defined(OS_POSIX) && !defined(OS_IOS)
+#elif BUILDFLAG(IS_POSIX)
 #include <sys/mman.h>
+
 #include "base/debug/proc_maps_linux.h"
-#elif defined(OS_WIN)
+#elif BUILDFLAG(IS_WIN)
 #include <windows.h>
-#elif defined(OS_FUCHSIA)
+
+#include "base/logging.h"
+#elif BUILDFLAG(IS_FUCHSIA)
 #include <lib/zx/object.h>
 #include <lib/zx/process.h>
+
 #include "base/fuchsia/fuchsia_logging.h"
 #endif
 
-namespace base {
-namespace subtle {
+namespace base::subtle {
 
 const size_t kRegionSize = 1024;
 
@@ -72,6 +77,19 @@ TEST_F(PlatformSharedMemoryRegionTest, CreateTooLargeRegionIsInvalid) {
   EXPECT_FALSE(region2.IsValid());
 }
 
+// Tests that creating a region of maximum possible value returns an invalid
+// region.
+TEST_F(PlatformSharedMemoryRegionTest, CreateMaxSizeRegionIsInvalid) {
+  size_t max_region_size = std::numeric_limits<size_t>::max();
+  PlatformSharedMemoryRegion region =
+      PlatformSharedMemoryRegion::CreateWritable(max_region_size);
+  EXPECT_FALSE(region.IsValid());
+
+  PlatformSharedMemoryRegion region2 =
+      PlatformSharedMemoryRegion::CreateUnsafe(max_region_size);
+  EXPECT_FALSE(region2.IsValid());
+}
+
 // Tests that regions consistently report their size as the size requested at
 // creation time even if their allocation size is larger due to platform
 // constraints.
@@ -124,7 +142,7 @@ TEST_F(PlatformSharedMemoryRegionTest, InvalidAfterPass) {
   PlatformSharedMemoryRegion region =
       PlatformSharedMemoryRegion::CreateWritable(kRegionSize);
   ASSERT_TRUE(region.IsValid());
-  ignore_result(region.PassPlatformHandle());
+  std::ignore = region.PassPlatformHandle();
   EXPECT_FALSE(region.IsValid());
 }
 
@@ -134,7 +152,7 @@ TEST_F(PlatformSharedMemoryRegionTest, InvalidAfterMove) {
       PlatformSharedMemoryRegion::CreateWritable(kRegionSize);
   ASSERT_TRUE(region.IsValid());
   PlatformSharedMemoryRegion moved_region = std::move(region);
-  EXPECT_FALSE(region.IsValid());
+  EXPECT_FALSE(region.IsValid());  // NOLINT(bugprone-use-after-move)
   EXPECT_TRUE(moved_region.IsValid());
 }
 
@@ -196,8 +214,7 @@ TEST_F(PlatformSharedMemoryRegionTest, MapAtWithOverflowTest) {
   EXPECT_FALSE(mapping.IsValid());
 }
 
-#if defined(OS_POSIX) && !defined(OS_ANDROID) && \
-    (!defined(OS_MACOSX) || defined(OS_IOS))
+#if BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_APPLE)
 // Tests that the second handle is closed after a conversion to read-only on
 // POSIX.
 TEST_F(PlatformSharedMemoryRegionTest,
@@ -223,31 +240,34 @@ TEST_F(PlatformSharedMemoryRegionTest, ConvertToUnsafeInvalidatesSecondHandle) {
 #endif
 
 void CheckReadOnlyMapProtection(void* addr) {
-#if defined(OS_MACOSX) && !defined(OS_IOS)
+#if BUILDFLAG(IS_APPLE)
   vm_region_basic_info_64 basic_info;
-  mach_vm_size_t dummy_size = 0;
-  void* temp_addr = addr;
-  MachVMRegionResult result = GetBasicInfo(
-      mach_task_self(), &dummy_size,
-      reinterpret_cast<mach_vm_address_t*>(&temp_addr), &basic_info);
-  ASSERT_EQ(result, MachVMRegionResult::Success);
+  vm_size_t dummy_size = 0;
+  mach_msg_type_number_t info_count = VM_REGION_BASIC_INFO_COUNT_64;
+  mach_port_t object_name;
+  kern_return_t kr = vm_region_64(
+      mach_task_self(), reinterpret_cast<vm_address_t*>(&addr), &dummy_size,
+      VM_REGION_BASIC_INFO_64, reinterpret_cast<vm_region_info_t>(&basic_info),
+      &info_count, &object_name);
+  mach_port_deallocate(mach_task_self(), object_name);
+
+  ASSERT_EQ(kr, KERN_SUCCESS);
   EXPECT_EQ(basic_info.protection & VM_PROT_ALL, VM_PROT_READ);
   EXPECT_EQ(basic_info.max_protection & VM_PROT_ALL, VM_PROT_READ);
-#elif defined(OS_POSIX) && !defined(OS_IOS)
+#elif BUILDFLAG(IS_POSIX)
   std::string proc_maps;
   ASSERT_TRUE(base::debug::ReadProcMaps(&proc_maps));
   std::vector<base::debug::MappedMemoryRegion> regions;
   ASSERT_TRUE(base::debug::ParseProcMaps(proc_maps, &regions));
-  auto it =
-      std::find_if(regions.begin(), regions.end(),
-                   [addr](const base::debug::MappedMemoryRegion& region) {
-                     return region.start == reinterpret_cast<uintptr_t>(addr);
-                   });
+  auto it = std::ranges::find_if(
+      regions, [addr](const base::debug::MappedMemoryRegion& region) {
+        return region.start == reinterpret_cast<uintptr_t>(addr);
+      });
   ASSERT_TRUE(it != regions.end());
   // PROT_READ may imply PROT_EXEC on some architectures, so just check that
   // permissions don't contain PROT_WRITE bit.
   EXPECT_FALSE(it->permissions & base::debug::MappedMemoryRegion::WRITE);
-#elif defined(OS_WIN)
+#elif BUILDFLAG(IS_WIN)
   MEMORY_BASIC_INFORMATION memory_info;
   size_t result = VirtualQueryEx(GetCurrentProcess(), addr, &memory_info,
                                  sizeof(memory_info));
@@ -257,7 +277,7 @@ void CheckReadOnlyMapProtection(void* addr) {
                                  logging::GetLastSystemErrorCode());
   EXPECT_EQ(memory_info.AllocationProtect, static_cast<DWORD>(PAGE_READONLY));
   EXPECT_EQ(memory_info.Protect, static_cast<DWORD>(PAGE_READONLY));
-#elif defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_FUCHSIA)
 // TODO(alexilin): We cannot call zx_object_get_info ZX_INFO_PROCESS_MAPS in
 // this process. Consider to create an auxiliary process that will read the
 // test process maps.
@@ -265,16 +285,16 @@ void CheckReadOnlyMapProtection(void* addr) {
 }
 
 bool TryToRestoreWritablePermissions(void* addr, size_t len) {
-#if defined(OS_POSIX) && !defined(OS_IOS)
+#if BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_IOS)
   int result = mprotect(addr, len, PROT_READ | PROT_WRITE);
   return result != -1;
-#elif defined(OS_WIN)
+#elif BUILDFLAG(IS_WIN)
   DWORD old_protection;
   return VirtualProtect(addr, len, PAGE_READWRITE, &old_protection);
-#elif defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_FUCHSIA)
   zx_status_t status =
-      zx::vmar::root_self()->protect(reinterpret_cast<uintptr_t>(addr), len,
-                                     ZX_VM_PERM_READ | ZX_VM_PERM_WRITE);
+      zx::vmar::root_self()->protect(ZX_VM_PERM_READ | ZX_VM_PERM_WRITE,
+                                     reinterpret_cast<uintptr_t>(addr), len);
   return status == ZX_OK;
 #else
   return false;
@@ -289,11 +309,17 @@ TEST_F(PlatformSharedMemoryRegionTest, MappingProtectionSetCorrectly) {
   ASSERT_TRUE(region.ConvertToReadOnly());
   WritableSharedMemoryMapping ro_mapping = MapForTesting(&region);
   ASSERT_TRUE(ro_mapping.IsValid());
-  CheckReadOnlyMapProtection(ro_mapping.memory());
+  CheckReadOnlyMapProtection(ro_mapping.data());
 
-  EXPECT_FALSE(TryToRestoreWritablePermissions(ro_mapping.memory(),
-                                               ro_mapping.mapped_size()));
-  CheckReadOnlyMapProtection(ro_mapping.memory());
+  // SAFETY: There's no public way to get a span of the full mapped memory size.
+  // The `mapped_size()` is larger then `size()` but is the actual size of the
+  // shared memory backing.
+  auto full_map_mem =
+      UNSAFE_BUFFERS(span(ro_mapping.data(), ro_mapping.mapped_size()));
+  EXPECT_FALSE(TryToRestoreWritablePermissions(full_map_mem.data(),
+                                               full_map_mem.size()));
+
+  CheckReadOnlyMapProtection(ro_mapping.data());
 }
 
 // Tests that platform handle permissions are checked correctly.
@@ -417,66 +443,4 @@ TEST_F(PlatformSharedMemoryRegionTest, UnsafeRegionConvertToUnsafeDeathTest) {
   EXPECT_DEATH_IF_SUPPORTED(region.ConvertToUnsafe(), kErrorRegex);
 }
 
-// Check that taking from a SharedMemoryHandle works.
-TEST_F(PlatformSharedMemoryRegionTest, TakeFromSharedMemoryHandle) {
-  SharedMemory shm;
-  auto region = PlatformSharedMemoryRegion::TakeFromSharedMemoryHandle(
-      shm.TakeHandle(), PlatformSharedMemoryRegion::Mode::kUnsafe);
-  ASSERT_FALSE(region.IsValid());
-
-  shm.CreateAndMapAnonymous(10);
-  region = PlatformSharedMemoryRegion::TakeFromSharedMemoryHandle(
-      shm.TakeHandle(), PlatformSharedMemoryRegion::Mode::kUnsafe);
-  ASSERT_TRUE(region.IsValid());
-
-#if !(defined(OS_MACOSX) && !defined(OS_IOS))
-  // Note that it's not possible on all platforms for TakeFromSharedMemoryHandle
-  // to conveniently check if the SharedMemoryHandle is readonly or
-  // not. Therefore it is actually possible to get an kUnsafe
-  // PlatformSharedMemoryRegion from a readonly handle on some platforms.
-  SharedMemoryCreateOptions options;
-  options.size = 10;
-  options.share_read_only = true;
-  shm.Create(options);
-  EXPECT_DEATH_IF_SUPPORTED(
-      PlatformSharedMemoryRegion::TakeFromSharedMemoryHandle(
-          shm.GetReadOnlyHandle(), PlatformSharedMemoryRegion::Mode::kUnsafe),
-      "");
-#endif  // !(defined(OS_MACOSX) && !defined(OS_IOS))
-}
-
-// Check that taking from a readonly SharedMemoryHandle works.
-TEST_F(PlatformSharedMemoryRegionTest, TakeFromReadOnlySharedMemoryHandle) {
-  SharedMemory shm;
-  // Note that getting a read-only handle from an unmapped SharedMemory will
-  // fail, so the invalid region case cannot be tested.
-  SharedMemoryCreateOptions options;
-  options.size = 10;
-  options.share_read_only = true;
-  shm.Create(options);
-  auto readonly_handle = shm.GetReadOnlyHandle();
-#if defined(OS_ANDROID)
-  readonly_handle.SetRegionReadOnly();
-#endif
-  auto region = PlatformSharedMemoryRegion::TakeFromSharedMemoryHandle(
-      readonly_handle, PlatformSharedMemoryRegion::Mode::kReadOnly);
-  ASSERT_TRUE(region.IsValid());
-}
-
-// Check that taking from a SharedMemoryHandle in writable mode fails.
-TEST_F(PlatformSharedMemoryRegionTest, WritableTakeFromSharedMemoryHandle) {
-  SharedMemory shm;
-  EXPECT_DEATH_IF_SUPPORTED(
-      PlatformSharedMemoryRegion::TakeFromSharedMemoryHandle(
-          shm.TakeHandle(), PlatformSharedMemoryRegion::Mode::kWritable),
-      "");
-
-  shm.CreateAndMapAnonymous(10);
-  EXPECT_DEATH_IF_SUPPORTED(
-      PlatformSharedMemoryRegion::TakeFromSharedMemoryHandle(
-          shm.TakeHandle(), PlatformSharedMemoryRegion::Mode::kWritable),
-      "");
-}
-
-}  // namespace subtle
-}  // namespace base
+}  // namespace base::subtle
