@@ -17,12 +17,14 @@ MessagePumpIOSForIO::FdWatchController::~FdWatchController() {
 }
 
 bool MessagePumpIOSForIO::FdWatchController::StopWatchingFileDescriptor() {
-  if (fdref_ == NULL)
+  if (fdref_ == NULL) {
     return true;
+  }
 
   CFFileDescriptorDisableCallBacks(fdref_.get(), callback_types_);
-  if (pump_)
+  if (pump_) {
     pump_->RemoveRunLoopSource(fd_source_);
+  }
   fd_source_.reset();
   fdref_.reset();
   callback_types_ = 0;
@@ -58,11 +60,9 @@ void MessagePumpIOSForIO::FdWatchController::OnFileCanWriteWithoutBlocking(
   watcher_->OnFileCanWriteWithoutBlocking(fd);
 }
 
-MessagePumpIOSForIO::MessagePumpIOSForIO() : weak_factory_(this) {
-}
+MessagePumpIOSForIO::MessagePumpIOSForIO() = default;
 
-MessagePumpIOSForIO::~MessagePumpIOSForIO() {
-}
+MessagePumpIOSForIO::~MessagePumpIOSForIO() = default;
 
 bool MessagePumpIOSForIO::WatchFileDescriptor(int fd,
                                               bool persistent,
@@ -96,7 +96,6 @@ bool MessagePumpIOSForIO::WatchFileDescriptor(int fd,
                                &source_context));
     if (scoped_fdref == NULL) {
       NOTREACHED() << "CFFileDescriptorCreate failed";
-      return false;
     }
 
     CFFileDescriptorEnableCallBacks(scoped_fdref, callback_types);
@@ -107,7 +106,6 @@ bool MessagePumpIOSForIO::WatchFileDescriptor(int fd,
                                             0));
     if (scoped_fd_source == NULL) {
       NOTREACHED() << "CFFileDescriptorCreateRunLoopSource failed";
-      return false;
     }
     CFRunLoopAddSource(run_loop(), scoped_fd_source, kCFRunLoopCommonModes);
 
@@ -121,11 +119,9 @@ bool MessagePumpIOSForIO::WatchFileDescriptor(int fd,
       NOTREACHED() << "FDs don't match: "
                    << CFFileDescriptorGetNativeDescriptor(fdref)
                    << " != " << fd;
-      return false;
     }
     if (persistent != controller->is_persistent_) {
       NOTREACHED() << "persistent doesn't match";
-      return false;
     }
 
     // Combine old/new event masks.
@@ -168,8 +164,9 @@ void MessagePumpIOSForIO::HandleFdIOEvent(CFFileDescriptorRef fdref,
     scoped_do_work_item = pump->delegate()->BeginWorkItem();
   }
 
-  if (callback_types & kCFFileDescriptorWriteCallBack)
+  if (callback_types & kCFFileDescriptorWriteCallBack) {
     controller->OnFileCanWriteWithoutBlocking(fd, pump);
+  }
 
   // Perform the read callback only if the file descriptor has not been
   // invalidated in the write callback. As |FdWatchController| invalidates
