@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,42 +7,79 @@
 #ifndef BASE_PROCESS_LAUNCH_H_
 #define BASE_PROCESS_LAUNCH_H_
 
+#include <limits.h>
 #include <stddef.h>
 
 #include <string>
+#include <string_view>
 #include <utility>
 #include <vector>
 
 #include "base/base_export.h"
+#include "base/command_line.h"
 #include "base/environment.h"
-#include "base/macros.h"
+#include "base/files/file_path.h"
+#include "base/memory/raw_ptr.h"
 #include "base/process/process.h"
 #include "base/process/process_handle.h"
-#include "base/strings/string_piece.h"
+#include "base/threading/thread_restrictions.h"
+#include "build/blink_buildflags.h"
 #include "build/build_config.h"
 
-#if defined(OS_WIN)
-#include <windows.h>
-#elif defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_WIN)
+#include "base/win/windows_types.h"
+#elif BUILDFLAG(IS_FUCHSIA)
 #include <lib/fdio/spawn.h>
 #include <zircon/types.h>
 #endif
 
-#if defined(OS_POSIX) || defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 #include "base/posix/file_descriptor_shuffle.h"
 #endif
 
-#if defined(OS_MACOSX) && !defined(OS_IOS)
-#include "base/mac/mach_port_rendezvous.h"
+#if BUILDFLAG(IS_ANDROID)
+#include "base/android/binder.h"
+#endif
+
+#if BUILDFLAG(IS_MAC)
+#include "base/mac/process_requirement.h"
 #endif
 
 namespace base {
 
-class CommandLine;
+#if BUILDFLAG(IS_POSIX)
+// Some code (e.g. the sandbox) relies on PTHREAD_STACK_MIN
+// being async-signal-safe, which is no longer guaranteed by POSIX
+// (it may call sysconf, which is not safe).
+// To work around this, use a hardcoded value unless it's already
+// defined as a constant.
 
-#if defined(OS_WIN)
+// These constants are borrowed from glibc’s (arch)/bits/pthread_stack_min.h.
+#if defined(ARCH_CPU_ARM64)
+#define PTHREAD_STACK_MIN_CONST \
+  (__builtin_constant_p(PTHREAD_STACK_MIN) ? PTHREAD_STACK_MIN : 131072)
+#else
+#define PTHREAD_STACK_MIN_CONST \
+  (__builtin_constant_p(PTHREAD_STACK_MIN) ? PTHREAD_STACK_MIN : 16384)
+#endif  // defined(ARCH_CPU_ARM64)
+
+static_assert(__builtin_constant_p(PTHREAD_STACK_MIN_CONST),
+              "must be constant");
+
+// Make sure our hardcoded value is large enough to accommodate the
+// actual minimum stack size. This function will run a one-time CHECK
+// and so should be called at some point, preferably during startup.
+BASE_EXPORT void CheckPThreadStackMinIsSafe();
+#endif  // BUILDFLAG(IS_POSIX)
+
+#if BUILDFLAG(IS_APPLE)
+class MachRendezvousPort;
+using MachPortsForRendezvous = std::map<uint32_t, MachRendezvousPort>;
+#endif
+
+#if BUILDFLAG(IS_WIN)
 typedef std::vector<HANDLE> HandlesToInheritVector;
-#elif defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_FUCHSIA)
 struct PathToTransfer {
   base::FilePath path;
   zx_handle_t handle;
@@ -53,30 +90,31 @@ struct HandleToTransfer {
 };
 typedef std::vector<HandleToTransfer> HandlesToTransferVector;
 typedef std::vector<std::pair<int, int>> FileHandleMappingVector;
-#elif defined(OS_POSIX)
+#elif BUILDFLAG(IS_POSIX)
 typedef std::vector<std::pair<int, int>> FileHandleMappingVector;
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 // Options for launching a subprocess that are passed to LaunchProcess().
 // The default constructor constructs the object with default options.
 struct BASE_EXPORT LaunchOptions {
-#if (defined(OS_POSIX) || defined(OS_FUCHSIA)) && !defined(OS_MACOSX)
+#if (BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)) && !BUILDFLAG(IS_APPLE)
   // Delegate to be run in between fork and exec in the subprocess (see
   // pre_exec_delegate below)
   class BASE_EXPORT PreExecDelegate {
    public:
     PreExecDelegate() = default;
+
+    PreExecDelegate(const PreExecDelegate&) = delete;
+    PreExecDelegate& operator=(const PreExecDelegate&) = delete;
+
     virtual ~PreExecDelegate() = default;
 
     // Since this is to be run between fork and exec, and fork may have happened
     // while multiple threads were running, this function needs to be async
     // safe.
     virtual void RunAsyncSafe() = 0;
-
-   private:
-    DISALLOW_COPY_AND_ASSIGN(PreExecDelegate);
   };
-#endif  // defined(OS_POSIX)
+#endif  // BUILDFLAG(IS_POSIX)
 
   LaunchOptions();
   LaunchOptions(const LaunchOptions&);
@@ -88,9 +126,16 @@ struct BASE_EXPORT LaunchOptions {
   // If not empty, change to this directory before executing the new process.
   base::FilePath current_directory;
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   bool start_hidden = false;
 
+  // Process will be started using ShellExecuteEx instead of CreateProcess so
+  // that it is elevated. LaunchProcess with this flag will have different
+  // behaviour due to ShellExecuteEx. Some common operations like OpenProcess
+  // will fail. Currently the only other supported LaunchOptions are
+  // |start_hidden| and |wait|.
+  bool elevated = false;
+
   // Sets STARTF_FORCEOFFFEEDBACK so that the feedback cursor is forced off
   // while the process is starting.
   bool feedback_cursor_off = false;
@@ -100,7 +145,7 @@ struct BASE_EXPORT LaunchOptions {
   // for a good overview of Windows handle inheritance.
   //
   // Implementation note: it might be nice to implement in terms of
-  // base::Optional<>, but then the natural default state (vector not present)
+  // std::optional<>, but then the natural default state (vector not present)
   // would be "all inheritable handles" while we want "no inheritance."
   enum class Inherit {
     // Only those handles in |handles_to_inherit| vector are inherited. If the
@@ -164,13 +209,28 @@ struct BASE_EXPORT LaunchOptions {
   // If set to true, permission to bring windows to the foreground is passed to
   // the launched process if the current process has such permission.
   bool grant_foreground_privilege = false;
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+
+  // If set to true, sets a process mitigation flag to disable Hardware-enforced
+  // Stack Protection for the process.
+  // This overrides /cetcompat if set on the executable. See:
+  // https://docs.microsoft.com/en-us/cpp/build/reference/cetcompat?view=msvc-160
+  // If not supported by Windows, has no effect. This flag weakens security by
+  // turning off ROP protection.
+  bool disable_cetcompat = false;
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   // Remap file descriptors according to the mapping of src_fd->dest_fd to
   // propagate FDs into the child process.
   FileHandleMappingVector fds_to_remap;
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
-#if defined(OS_WIN) || defined(OS_POSIX) || defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_ANDROID)
+  // Set of strong IBinder references to be passed to the child process. These
+  // make their way to ChildProcessServiceDelegate.onConnectionSetup (Java)
+  // within the new child process.
+  std::vector<android::BinderRef> binders;
+#endif
+
+#if BUILDFLAG(IS_WIN) || BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   // Set/unset environment variables. These are applied on top of the parent
   // process environment.  Empty (the default) means to inherit the same
   // environment. See internal::AlterEnvironment().
@@ -179,9 +239,9 @@ struct BASE_EXPORT LaunchOptions {
   // Clear the environment for the new process before processing changes from
   // |environment|.
   bool clear_environment = false;
-#endif  // OS_WIN || OS_POSIX || OS_FUCHSIA
+#endif  // BUILDFLAG(IS_WIN) || BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 
-#if defined(OS_LINUX)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
   // If non-zero, start the process using clone(), using flags as provided.
   // Unlike in clone, clone_flags may not contain a custom termination signal
   // that is sent to the parent when the child dies. The termination signal will
@@ -194,9 +254,13 @@ struct BASE_EXPORT LaunchOptions {
 
   // Sets parent process death signal to SIGKILL.
   bool kill_on_parent_death = false;
-#endif  // defined(OS_LINUX)
 
-#if defined(OS_MACOSX) && !defined(OS_IOS)
+  // File descriptors of the parent process with FD_CLOEXEC flag to be removed
+  // before calling exec*().
+  std::vector<int> fds_to_remove_cloexec;
+#endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
+
+#if BUILDFLAG(IS_MAC) || (BUILDFLAG(IS_IOS) && BUILDFLAG(USE_BLINK))
   // Mach ports that will be accessible to the child process. These are not
   // directly inherited across process creation, but they are stored by a Mach
   // IPC server that a child process can communicate with to retrieve them.
@@ -204,11 +268,29 @@ struct BASE_EXPORT LaunchOptions {
   // After calling LaunchProcess(), any rights that were transferred with MOVE
   // dispositions will be consumed, even on failure.
   //
-  // See base/mac/mach_port_rendezvous.h for details.
+  // See base/apple/mach_port_rendezvous.h for details.
   MachPortsForRendezvous mach_ports_for_rendezvous;
-#endif
 
-#if defined(OS_FUCHSIA)
+  // Apply a process scheduler policy to enable mitigations against CPU side-
+  // channel attacks.
+  bool enable_cpu_security_mitigations = false;
+#endif  // BUILDFLAG(IS_MAC) || (BUILDFLAG(IS_IOS) && BUILDFLAG(USE_BLINK))
+
+#if BUILDFLAG(IS_MAC)
+  // When a child process is launched, the system tracks the parent process
+  // with a concept of "responsibility". The responsible process will be
+  // associated with any requests for private data stored on the system via
+  // the TCC subsystem. When launching processes that run foreign/third-party
+  // code, the responsibility for the child process should be disclaimed so
+  // that any TCC requests are not associated with the parent.
+  bool disclaim_responsibility = false;
+
+  // A `ProcessRequirement` that will be used to validate the launched process
+  // before it can retrieve `mach_ports_for_rendezvous`.
+  std::optional<mac::ProcessRequirement> process_requirement;
+#endif  // BUILDFLAG(IS_MAC)
+
+#if BUILDFLAG(IS_FUCHSIA)
   // If valid, launches the application in that job object.
   zx_handle_t job_handle = ZX_HANDLE_INVALID;
 
@@ -230,8 +312,8 @@ struct BASE_EXPORT LaunchOptions {
 
   // Specifies which basic capabilities to grant to the child process.
   // By default the child process will receive the caller's complete namespace,
-  // access to the current base::fuchsia::DefaultJob(), handles for stdio and
-  // access to the dynamic library loader.
+  // access to the current base::GetDefaultJob(), handles for stdio and access
+  // to the dynamic library loader.
   // Note that the child is always provided access to the loader service.
   uint32_t spawn_flags = FDIO_SPAWN_CLONE_NAMESPACE | FDIO_SPAWN_CLONE_STDIO |
                          FDIO_SPAWN_CLONE_JOB;
@@ -240,46 +322,53 @@ struct BASE_EXPORT LaunchOptions {
   // the child process. If |paths_to_clone| is empty then the process will
   // receive either a full copy of the parent's namespace, or an empty one,
   // depending on whether FDIO_SPAWN_CLONE_NAMESPACE is set.
+  // Process launch will fail if `paths_to_clone` and `paths_to_transfer`
+  // together contain conflicting paths (e.g. overlaps or duplicates).
   std::vector<FilePath> paths_to_clone;
 
   // Specifies handles which will be installed as files or directories in the
-  // child process' namespace. Paths installed by |paths_to_clone| will be
-  // overridden by these entries.
+  // child process' namespace.
+  // Process launch will fail if `paths_to_clone` and `paths_to_transfer`
+  // together contain conflicting paths (e.g. overlaps or duplicates).
   std::vector<PathToTransfer> paths_to_transfer;
-#endif  // defined(OS_FUCHSIA)
 
-#if defined(OS_POSIX)
+  // Suffix that will be added to the process name. When specified process name
+  // will be set to "<binary_name><process_suffix>".
+  std::string process_name_suffix;
+#endif  // BUILDFLAG(IS_FUCHSIA)
+
+#if BUILDFLAG(IS_POSIX)
   // If not empty, launch the specified executable instead of
   // cmdline.GetProgram(). This is useful when it is necessary to pass a custom
   // argv[0].
   base::FilePath real_path;
 
-#if !defined(OS_MACOSX)
+#if !BUILDFLAG(IS_APPLE)
   // If non-null, a delegate to be run immediately prior to executing the new
   // program in the child process.
   //
   // WARNING: If LaunchProcess is called in the presence of multiple threads,
   // code running in this delegate essentially needs to be async-signal safe
   // (see man 7 signal for a list of allowed functions).
-  PreExecDelegate* pre_exec_delegate = nullptr;
-#endif  // !defined(OS_MACOSX)
+  raw_ptr<PreExecDelegate> pre_exec_delegate = nullptr;
+#endif  // !BUILDFLAG(IS_APPLE)
 
   // Each element is an RLIMIT_* constant that should be raised to its
   // rlim_max.  This pointer is owned by the caller and must live through
   // the call to LaunchProcess().
-  const std::vector<int>* maximize_rlimits = nullptr;
+  raw_ptr<const std::vector<int>> maximize_rlimits = nullptr;
 
   // If true, start the process in a new process group, instead of
   // inheriting the parent's process group.  The pgid of the child process
   // will be the same as its pid.
   bool new_process_group = false;
-#endif  // defined(OS_POSIX)
+#endif  // BUILDFLAG(IS_POSIX)
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS)
   // If non-negative, the specified file descriptor will be set as the launched
   // process' controlling terminal.
   int ctrl_terminal_fd = -1;
-#endif  // defined(OS_CHROMEOS)
+#endif  // BUILDFLAG(IS_CHROMEOS)
 };
 
 // Launch a process via the command line |cmdline|.
@@ -298,29 +387,23 @@ struct BASE_EXPORT LaunchOptions {
 BASE_EXPORT Process LaunchProcess(const CommandLine& cmdline,
                                   const LaunchOptions& options);
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Windows-specific LaunchProcess that takes the command line as a
 // string.  Useful for situations where you need to control the
 // command line arguments directly, but prefer the CommandLine version
-// if launching Chrome itself.
+// if launching Chrome itself. Also prefer the CommandLine version if
+// `options.elevated` is set because `cmdline` needs to be parsed for
+// ShellExecuteEx.
 //
 // The first command line argument should be the path to the process,
 // and don't forget to quote it.
 //
 // Example (including literal quotes)
 //  cmdline = "c:\windows\explorer.exe" -foo "c:\bar\"
-BASE_EXPORT Process LaunchProcess(const string16& cmdline,
+BASE_EXPORT Process LaunchProcess(const CommandLine::StringType& cmdline,
                                   const LaunchOptions& options);
 
-// Launches a process with elevated privileges.  This does not behave exactly
-// like LaunchProcess as it uses ShellExecuteEx instead of CreateProcess to
-// create the process.  This means the process will have elevated privileges
-// and thus some common operations like OpenProcess will fail. Currently the
-// only supported LaunchOptions are |start_hidden| and |wait|.
-BASE_EXPORT Process LaunchElevatedProcess(const CommandLine& cmdline,
-                                          const LaunchOptions& options);
-
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 // A POSIX-specific version of LaunchProcess that takes an argv array
 // instead of a CommandLine.  Useful for situations where you need to
 // control the command line arguments directly, but prefer the
@@ -328,15 +411,15 @@ BASE_EXPORT Process LaunchElevatedProcess(const CommandLine& cmdline,
 BASE_EXPORT Process LaunchProcess(const std::vector<std::string>& argv,
                                   const LaunchOptions& options);
 
-#if !defined(OS_MACOSX)
+#if !BUILDFLAG(IS_APPLE)
 // Close all file descriptors, except those which are a destination in the
 // given multimap. Only call this function in a child process where you know
 // that there aren't any other threads.
 BASE_EXPORT void CloseSuperfluousFds(const InjectiveMultimap& saved_map);
-#endif  // defined(OS_MACOSX)
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_APPLE)
+#endif  // BUILDFLAG(IS_WIN)
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Set |job_object|'s JOBOBJECT_EXTENDED_LIMIT_INFORMATION
 // BasicLimitInformation.LimitFlags to |limit_flags|.
 BASE_EXPORT bool SetJobObjectLimitFlags(HANDLE job_object, DWORD limit_flags);
@@ -344,7 +427,7 @@ BASE_EXPORT bool SetJobObjectLimitFlags(HANDLE job_object, DWORD limit_flags);
 // Output multi-process printf, cout, cerr, etc to the cmd.exe console that ran
 // chrome. This is not thread-safe: only call from main thread.
 BASE_EXPORT void RouteStdioToConsole(bool create_console_if_not_found);
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 // Executes the application specified by |cl| and wait for it to exit. Stores
 // the output (stdout) in |output|. Redirects stderr to /dev/null. Returns true
@@ -361,14 +444,16 @@ BASE_EXPORT bool GetAppOutputAndError(const CommandLine& cl,
 // this is the case the exit code of the application is available in
 // |*exit_code|.
 BASE_EXPORT bool GetAppOutputWithExitCode(const CommandLine& cl,
-                                          std::string* output, int* exit_code);
+                                          std::string* output,
+                                          int* exit_code);
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // A Windows-specific version of GetAppOutput that takes a command line string
 // instead of a CommandLine object. Useful for situations where you need to
 // control the command line arguments directly.
-BASE_EXPORT bool GetAppOutput(const StringPiece16& cl, std::string* output);
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+BASE_EXPORT bool GetAppOutput(CommandLine::StringViewType cl,
+                              std::string* output);
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
 // A POSIX-specific version of GetAppOutput that takes an argv array
 // instead of a CommandLine.  Useful for situations where you need to
 // control the command line arguments directly.
@@ -379,7 +464,7 @@ BASE_EXPORT bool GetAppOutput(const std::vector<std::string>& argv,
 // stderr.
 BASE_EXPORT bool GetAppOutputAndError(const std::vector<std::string>& argv,
                                       std::string* output);
-#endif  // defined(OS_WIN)
+#endif  // BUILDFLAG(IS_WIN)
 
 // If supported on the platform, and the user has sufficent rights, increase
 // the current process's scheduling priority to a high priority.
@@ -389,7 +474,7 @@ BASE_EXPORT void RaiseProcessToHighPriority();
 // binary. This should not be called in production/released code.
 BASE_EXPORT LaunchOptions LaunchOptionsForTest();
 
-#if defined(OS_LINUX) || defined(OS_NACL_NONSFI)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
 // A wrapper for clone with fork-like behavior, meaning that it returns the
 // child's pid in the parent and 0 in the child. |flags|, |ptid|, and |ctid| are
 // as in the clone system call (the CLONE_VM flag is not supported).
@@ -406,9 +491,20 @@ BASE_EXPORT LaunchOptions LaunchOptionsForTest();
 //
 // It is unsafe to use any pthread APIs after ForkWithFlags().
 // However, performing an exec() will lift this restriction.
-BASE_EXPORT pid_t ForkWithFlags(unsigned long flags, pid_t* ptid, pid_t* ctid);
+BASE_EXPORT pid_t ForkWithFlags(int flags, pid_t* ptid, pid_t* ctid);
 #endif
 
+namespace internal {
+
+// Friend and derived class of ScopedAllowBaseSyncPrimitives which allows
+// GetAppOutputInternal() to join a process. GetAppOutputInternal() can't itself
+// be a friend of ScopedAllowBaseSyncPrimitives because it is in the anonymous
+// namespace.
+class [[maybe_unused, nodiscard]] GetAppOutputScopedAllowBaseSyncPrimitives
+    : public base::ScopedAllowBaseSyncPrimitives {};
+
+}  // namespace internal
+
 }  // namespace base
 
 #endif  // BASE_PROCESS_LAUNCH_H_
