@@ -4,10 +4,11 @@
 
 #include "base/profiler/native_unwinder_win.h"
 
-#include <winnt.h>
+#include <windows.h>
 
 #include "base/check_op.h"
 #include "base/notreached.h"
+#include "base/profiler/register_context_registers.h"
 #include "base/profiler/win32_stack_frame_unwinder.h"
 #include "build/build_config.h"
 
@@ -20,7 +21,8 @@ bool NativeUnwinderWin::CanUnwindFrom(const Frame& current_frame) const {
 // Attempts to unwind the frame represented by the context values. If
 // successful appends frames onto the stack and returns true. Otherwise
 // returns false.
-UnwindResult NativeUnwinderWin::TryUnwind(RegisterContext* thread_context,
+UnwindResult NativeUnwinderWin::TryUnwind(UnwinderStateCapture* capture_state,
+                                          RegisterContext* thread_context,
                                           uintptr_t stack_top,
                                           std::vector<Frame>* stack) {
   // We expect the frame corresponding to the |thread_context| register state to
@@ -58,8 +60,9 @@ UnwindResult NativeUnwinderWin::TryUnwind(RegisterContext* thread_context,
       return UnwindResult::kAborted;
     }
 
-    if (RegisterContextInstructionPointer(thread_context) == 0)
+    if (RegisterContextInstructionPointer(thread_context) == 0) {
       return UnwindResult::kCompleted;
+    }
 
     // Exclusive range of expected stack pointer values after the unwind.
     struct {
@@ -89,7 +92,6 @@ UnwindResult NativeUnwinderWin::TryUnwind(RegisterContext* thread_context,
   }
 
   NOTREACHED();
-  return UnwindResult::kCompleted;
 }
 
 }  // namespace base
