@@ -2,9 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/profiler/stack_sampling_profiler.h"
+
 #include <stddef.h>
 #include <stdint.h>
 
+#include <algorithm>
+#include <array>
 #include <cstdlib>
 #include <memory>
 #include <set>
@@ -19,43 +23,44 @@
 #include "base/memory/ptr_util.h"
 #include "base/memory/raw_ptr.h"
 #include "base/metrics/metrics_hashes.h"
+#include "base/notimplemented.h"
 #include "base/profiler/profiler_buildflags.h"
 #include "base/profiler/sample_metadata.h"
 #include "base/profiler/stack_sampler.h"
-#include "base/profiler/stack_sampling_profiler.h"
 #include "base/profiler/stack_sampling_profiler_test_util.h"
 #include "base/profiler/unwinder.h"
-#include "base/ranges/algorithm.h"
 #include "base/run_loop.h"
 #include "base/scoped_native_library.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/synchronization/lock.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/test/bind.h"
+#include "base/test/task_environment.h"
 #include "base/threading/simple_thread.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 #if BUILDFLAG(IS_WIN)
+#include <windows.h>
+
 #include <intrin.h>
 #include <malloc.h>
-#include <windows.h>
 #else
 #include <alloca.h>
 #endif
 
 // STACK_SAMPLING_PROFILER_SUPPORTED is used to conditionally enable the tests
-// below for supported platforms (currently Win x64, Mac x64, iOS 64, some
+// below for supported platforms (currently Win x64, Mac, iOS 64, some
 // Android, and ChromeOS x64).
 // ChromeOS: These don't run under MSan because parts of the stack aren't
 // initialized.
-#if (BUILDFLAG(IS_WIN) && defined(ARCH_CPU_X86_64)) ||            \
-    (BUILDFLAG(IS_MAC) && defined(ARCH_CPU_X86_64)) ||            \
-    (BUILDFLAG(IS_IOS) && defined(ARCH_CPU_64_BITS)) ||           \
-    (BUILDFLAG(IS_ANDROID) && BUILDFLAG(ENABLE_ARM_CFI_TABLE)) || \
-    (BUILDFLAG(IS_CHROMEOS) &&                                    \
-     (defined(ARCH_CPU_X86_64) || defined(ARCH_CPU_ARM64)) &&     \
+#if (BUILDFLAG(IS_WIN) && defined(ARCH_CPU_X86_64)) || (BUILDFLAG(IS_MAC)) || \
+    (BUILDFLAG(IS_IOS) && defined(ARCH_CPU_64_BITS)) ||                       \
+    (BUILDFLAG(IS_ANDROID) &&                                                 \
+     (BUILDFLAG(ENABLE_ARM_CFI_TABLE) || defined(ARCH_CPU_ARM64))) ||         \
+    (BUILDFLAG(IS_CHROMEOS) &&                                                \
+     (defined(ARCH_CPU_X86_64) || defined(ARCH_CPU_ARM64)) &&                 \
      !defined(MEMORY_SANITIZER))
 #define STACK_SAMPLING_PROFILER_SUPPORTED 1
 #endif
@@ -273,10 +278,10 @@ size_t WaitForSamplingComplete(
     const std::vector<std::unique_ptr<TestProfilerInfo>>& infos) {
   // Map unique_ptrs to something that WaitMany can accept.
   std::vector<WaitableEvent*> sampling_completed_rawptrs(infos.size());
-  ranges::transform(infos, sampling_completed_rawptrs.begin(),
-                    [](const std::unique_ptr<TestProfilerInfo>& info) {
-                      return &info.get()->completed;
-                    });
+  std::ranges::transform(infos, sampling_completed_rawptrs.begin(),
+                         [](const std::unique_ptr<TestProfilerInfo>& info) {
+                           return &info.get()->completed;
+                         });
   // Wait for one profiler to finish.
   return WaitableEvent::WaitMany(sampling_completed_rawptrs.data(),
                                  sampling_completed_rawptrs.size());
@@ -306,8 +311,9 @@ void TestLibraryUnload(bool wait_until_unloaded, ModuleCache* module_cache) {
 
     void OnPreStackWalk() override {
       stack_copied_->Signal();
-      if (wait_to_walk_stack_)
+      if (wait_to_walk_stack_) {
         start_stack_walk_->Wait();
+      }
     }
 
    private:
@@ -322,12 +328,13 @@ void TestLibraryUnload(bool wait_until_unloaded, ModuleCache* module_cache) {
 
   NativeLibrary other_library = LoadOtherLibrary();
 
-  UnwindScenario scenario(
-      BindRepeating(&CallThroughOtherLibrary, Unretained(other_library)));
+  // TODO(crbug.com/40061562): Remove `UnsafeDanglingUntriaged`
+  UnwindScenario scenario(BindRepeating(
+      &CallThroughOtherLibrary, UnsafeDanglingUntriaged(other_library)));
 
   UnwindScenario::SampleEvents events;
   TargetThread target_thread(
-      BindLambdaForTesting([&]() { scenario.Execute(&events); }));
+      BindLambdaForTesting([&] { scenario.Execute(&events); }));
   target_thread.Start();
   events.ready_for_sample.Wait();
 
@@ -365,10 +372,11 @@ void TestLibraryUnload(bool wait_until_unloaded, ModuleCache* module_cache) {
   target_thread.Join();
 
   // Unload the library now that it's not being used.
-  if (wait_until_unloaded)
+  if (wait_until_unloaded) {
     SynchronousUnloadNativeLibrary(other_library);
-  else
+  } else {
     UnloadNativeLibrary(other_library);
+  }
 
   // Let the stack walk commence after unloading the library, if we're waiting
   // on that event.
@@ -423,6 +431,8 @@ void TestLibraryUnload(bool wait_until_unloaded, ModuleCache* module_cache) {
 // usable in a later test.
 class StackSamplingProfilerTest : public testing::Test {
  public:
+  StackSamplingProfilerTest() = default;
+
   void SetUp() override {
     // The idle-shutdown time is too long for convenient (and accurate) testing.
     // That behavior is checked instead by artificially triggering it through
@@ -441,6 +451,7 @@ class StackSamplingProfilerTest : public testing::Test {
 
  private:
   ModuleCache module_cache_;
+  base::test::TaskEnvironment task_environment_;
 };
 
 }  // namespace
@@ -449,7 +460,7 @@ class StackSamplingProfilerTest : public testing::Test {
 //
 // macOS ASAN is not yet supported - crbug.com/718628.
 //
-// TODO(https://crbug.com/1100175): Enable this test again for Android with
+// TODO(crbug.com/40702833): Enable this test again for Android with
 // ASAN. This is now disabled because the android-asan bot fails.
 //
 // If we're running the ChromeOS unit tests on Linux, this test will never pass
@@ -467,8 +478,9 @@ PROFILER_TEST_F(StackSamplingProfilerTest, MAYBE_Basic) {
   const std::vector<Frame>& sample = SampleScenario(&scenario, module_cache());
 
   // Check that all the modules are valid.
-  for (const auto& frame : sample)
+  for (const auto& frame : sample) {
     EXPECT_NE(nullptr, frame.module);
+  }
 
   // The stack should contain a full unwind.
   ExpectStackContains(sample, {scenario.GetWaitForSampleAddressRange(),
@@ -489,12 +501,14 @@ class TestAuxUnwinder : public Unwinder {
   TestAuxUnwinder& operator=(const TestAuxUnwinder&) = delete;
 
   void InitializeModules() override {
-    if (add_initial_modules_callback_)
+    if (add_initial_modules_callback_) {
       add_initial_modules_callback_.Run();
+    }
   }
   bool CanUnwindFrom(const Frame& current_frame) const override { return true; }
 
-  UnwindResult TryUnwind(RegisterContext* thread_context,
+  UnwindResult TryUnwind(UnwinderStateCapture* capture_state,
+                         RegisterContext* thread_context,
                          uintptr_t stack_top,
                          std::vector<Frame>* stack) override {
     stack->push_back(frame_to_report_);
@@ -537,7 +551,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, MAYBE_Alloca) {
 // iOS chrome doesn't support loading native libraries.
 // Android is not supported when EXCLUDE_UNWIND_TABLES |other_library| doesn't
 // have unwind tables.
-// TODO(https://crbug.com/1100175): Enable this test again for Android with
+// TODO(crbug.com/40702833): Enable this test again for Android with
 // ASAN. This is now disabled because the android-asan bot fails.
 // If we're running the ChromeOS unit tests on Linux, this test will never pass
 // because Ubuntu's libc isn't compiled with frame pointers. Skip if not a real
@@ -568,7 +582,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, MAYBE_OtherLibrary) {
 // Unloading is synchronous on the Mac, so this test is inapplicable.
 // Android is not supported when EXCLUDE_UNWIND_TABLES |other_library| doesn't
 // have unwind tables.
-// TODO(https://crbug.com/1100175): Enable this test again for Android with
+// TODO(crbug.com/40702833): Enable this test again for Android with
 // ASAN. This is now disabled because the android-asan bot fails.
 // If we're running the ChromeOS unit tests on Linux, this test will never pass
 // because Ubuntu's libc isn't compiled with frame pointers. Skip if not a real
@@ -657,7 +671,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, StopSafely) {
 
   WithTargetThread(
       BindLambdaForTesting([](SamplingProfilerThreadToken target_thread_token) {
-        SamplingParams params[2];
+        std::array<SamplingParams, 2> params;
 
         // Providing an initial delay makes it more likely that both will be
         // scheduled before either starts to run. Once started, samples will
@@ -671,7 +685,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, StopSafely) {
         params[1].sampling_interval = Milliseconds(1);
         params[1].samples_per_profile = 100000;
 
-        SampleRecordedCounter samples_recorded[std::size(params)];
+        std::array<SampleRecordedCounter, std::size(params)> samples_recorded;
         ModuleCache module_cache1, module_cache2;
         TestProfilerInfo profiler_info0(target_thread_token, params[0],
                                         &module_cache1, &samples_recorded[0]);
@@ -685,8 +699,10 @@ PROFILER_TEST_F(StackSamplingProfilerTest, StopSafely) {
         // possible but gets complicated later on because there's no way of
         // knowing if 0 or 1 additional sample will be taken after Stop() and
         // thus no way of knowing how many Wait() calls to make on it.
-        while (samples_recorded[0].Get() == 0 || samples_recorded[1].Get() == 0)
+        while (samples_recorded[0].Get() == 0 ||
+               samples_recorded[1].Get() == 0) {
           PlatformThread::Sleep(Milliseconds(1));
+        }
 
         // Ensure that the first sampler can be safely stopped while the second
         // continues to run. The stopped first profiler will still have a
@@ -700,8 +716,9 @@ PROFILER_TEST_F(StackSamplingProfilerTest, StopSafely) {
         // Waiting for the second sampler to collect a couple samples ensures
         // that the pending RecordSampleTask for the first has executed because
         // tasks are always ordered by their next scheduled time.
-        while (samples_recorded[1].Get() < count1 + 2)
+        while (samples_recorded[1].Get() < count1 + 2) {
           PlatformThread::Sleep(Milliseconds(1));
+        }
 
         // Ensure that the first profiler didn't do anything since it was
         // stopped.
@@ -836,24 +853,24 @@ PROFILER_TEST_F(StackSamplingProfilerTest, DestroyProfilerWhileProfiling) {
   params.sampling_interval = Milliseconds(10);
 
   Profile profile;
-  WithTargetThread(BindLambdaForTesting([&, this](SamplingProfilerThreadToken
-                                                      target_thread_token) {
-    std::unique_ptr<StackSamplingProfiler> profiler;
-    auto profile_builder = std::make_unique<TestProfileBuilder>(
-        module_cache(),
-        BindLambdaForTesting([&profile](Profile result_profile) {
-          profile = std::move(result_profile);
-        }));
-    profiler = std::make_unique<StackSamplingProfiler>(
-        target_thread_token, params, std::move(profile_builder),
-        CreateCoreUnwindersFactoryForTesting(module_cache()));
-    profiler->Start();
-    profiler.reset();
+  WithTargetThread(BindLambdaForTesting(
+      [&, this](SamplingProfilerThreadToken target_thread_token) {
+        std::unique_ptr<StackSamplingProfiler> profiler;
+        auto profile_builder = std::make_unique<TestProfileBuilder>(
+            module_cache(),
+            BindLambdaForTesting([&profile](Profile result_profile) {
+              profile = std::move(result_profile);
+            }));
+        profiler = std::make_unique<StackSamplingProfiler>(
+            target_thread_token, params, std::move(profile_builder),
+            CreateCoreUnwindersFactoryForTesting(module_cache()));
+        profiler->Start();
+        profiler.reset();
 
-    // Wait longer than a sample interval to catch any use-after-free actions by
-    // the profiler thread.
-    PlatformThread::Sleep(Milliseconds(50));
-  }));
+        // Wait longer than a sample interval to catch any use-after-free
+        // actions by the profiler thread.
+        PlatformThread::Sleep(Milliseconds(50));
+      }));
 }
 
 // Checks that the different profilers may be run.
@@ -946,8 +963,9 @@ PROFILER_TEST_F(StackSamplingProfilerTest, SamplerIdleShutdown) {
   // While the shutdown has been initiated, the actual exit of the thread still
   // happens asynchronously. Watch until the thread actually exits. This test
   // will time-out in the case of failure.
-  while (StackSamplingProfiler::TestPeer::IsSamplingThreadRunning())
+  while (StackSamplingProfiler::TestPeer::IsSamplingThreadRunning()) {
     PlatformThread::Sleep(Milliseconds(1));
+  }
 }
 
 // Checks that additional requests will restart a stopped profiler.
@@ -1085,7 +1103,8 @@ PROFILER_TEST_F(StackSamplingProfilerTest, IdleShutdownAbort) {
 PROFILER_TEST_F(StackSamplingProfilerTest, ConcurrentProfiling_InSync) {
   WithTargetThread(
       BindLambdaForTesting([](SamplingProfilerThreadToken target_thread_token) {
-        std::vector<ModuleCache> module_caches(2);
+        ModuleCache module_cache1;
+        ModuleCache module_cache2;
 
         // Providing an initial delay makes it more likely that both will be
         // scheduled before either starts to run. Once started, samples will
@@ -1099,13 +1118,13 @@ PROFILER_TEST_F(StackSamplingProfilerTest, ConcurrentProfiling_InSync) {
             SamplingParams{/*initial_delay=*/Milliseconds(10),
                            /*samples_per_profile=*/9,
                            /*sampling_interval=*/Milliseconds(1)},
-            &module_caches[0]));
+            &module_cache1));
         profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
             target_thread_token,
             SamplingParams{/*initial_delay=*/Milliseconds(11),
                            /*samples_per_profile=*/8,
                            /*sampling_interval=*/Milliseconds(1)},
-            &module_caches[1]));
+            &module_cache2));
 
         profiler_infos[0]->profiler.Start();
         profiler_infos[1]->profiler.Start();
@@ -1125,43 +1144,47 @@ PROFILER_TEST_F(StackSamplingProfilerTest, ConcurrentProfiling_InSync) {
 
 // Checks that several mixed sampling requests execute in parallel.
 PROFILER_TEST_F(StackSamplingProfilerTest, ConcurrentProfiling_Mixed) {
-  WithTargetThread(BindLambdaForTesting([](SamplingProfilerThreadToken
-                                               target_thread_token) {
-    std::vector<ModuleCache> module_caches(3);
+  WithTargetThread(
+      BindLambdaForTesting([](SamplingProfilerThreadToken target_thread_token) {
+        std::vector<ModuleCache> module_caches(3);
 
-    std::vector<std::unique_ptr<TestProfilerInfo>> profiler_infos;
-    profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
-        target_thread_token,
-        SamplingParams{/*initial_delay=*/Milliseconds(8),
-                       /*samples_per_profile=*/10,
-                       /*sampling_interval=*/Milliseconds(4)},
-        &module_caches[0]));
-    profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
-        target_thread_token,
-        SamplingParams{/*initial_delay=*/Milliseconds(9),
-                       /*samples_per_profile=*/10,
-                       /*sampling_interval=*/Milliseconds(3)},
-        &module_caches[1]));
-    profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
-        target_thread_token,
-        SamplingParams{/*initial_delay=*/Milliseconds(10),
-                       /*samples_per_profile=*/10,
-                       /*sampling_interval=*/Milliseconds(2)},
-        &module_caches[2]));
+        std::vector<std::unique_ptr<TestProfilerInfo>> profiler_infos;
+        profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
+            target_thread_token,
+            SamplingParams{/*initial_delay=*/Milliseconds(8),
+                           /*samples_per_profile=*/10,
+                           /*sampling_interval=*/Milliseconds(4)},
+            &module_caches[0]));
+        profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
+            target_thread_token,
+            SamplingParams{/*initial_delay=*/Milliseconds(9),
+                           /*samples_per_profile=*/10,
+                           /*sampling_interval=*/Milliseconds(3)},
+            &module_caches[1]));
+        profiler_infos.push_back(std::make_unique<TestProfilerInfo>(
+            target_thread_token,
+            SamplingParams{/*initial_delay=*/Milliseconds(10),
+                           /*samples_per_profile=*/10,
+                           /*sampling_interval=*/Milliseconds(2)},
+            &module_caches[2]));
 
-    for (auto& i : profiler_infos)
-      i->profiler.Start();
+        for (auto& i : profiler_infos) {
+          i->profiler.Start();
+        }
 
-    // Wait for one profiler to finish.
-    size_t completed_profiler = WaitForSamplingComplete(profiler_infos);
-    EXPECT_EQ(10u, profiler_infos[completed_profiler]->profile.samples.size());
-    // Stop and destroy all profilers, always in the same order. Don't
-    // crash.
-    for (auto& i : profiler_infos)
-      i->profiler.Stop();
-    for (auto& i : profiler_infos)
-      i.reset();
-  }));
+        // Wait for one profiler to finish.
+        size_t completed_profiler = WaitForSamplingComplete(profiler_infos);
+        EXPECT_EQ(10u,
+                  profiler_infos[completed_profiler]->profile.samples.size());
+        // Stop and destroy all profilers, always in the same order. Don't
+        // crash.
+        for (auto& i : profiler_infos) {
+          i->profiler.Stop();
+        }
+        for (auto& i : profiler_infos) {
+          i.reset();
+        }
+      }));
 }
 
 // Checks that different threads can be sampled in parallel.
@@ -1169,14 +1192,14 @@ PROFILER_TEST_F(StackSamplingProfilerTest, MultipleSampledThreads) {
   UnwindScenario scenario1(BindRepeating(&CallWithPlainFunction));
   UnwindScenario::SampleEvents events1;
   TargetThread target_thread1(
-      BindLambdaForTesting([&]() { scenario1.Execute(&events1); }));
+      BindLambdaForTesting([&] { scenario1.Execute(&events1); }));
   target_thread1.Start();
   events1.ready_for_sample.Wait();
 
   UnwindScenario scenario2(BindRepeating(&CallWithPlainFunction));
   UnwindScenario::SampleEvents events2;
   TargetThread target_thread2(
-      BindLambdaForTesting([&]() { scenario2.Execute(&events2); }));
+      BindLambdaForTesting([&] { scenario2.Execute(&events2); }));
   target_thread2.Start();
   events2.ready_for_sample.Wait();
 
@@ -1328,7 +1351,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, AddAuxUnwinder_BeforeStart) {
 
   int add_initial_modules_invocation_count = 0;
   const auto add_initial_modules_callback =
-      [&add_initial_modules_invocation_count]() {
+      [&add_initial_modules_invocation_count] {
         ++add_initial_modules_invocation_count;
       };
 
@@ -1359,7 +1382,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, AddAuxUnwinder_BeforeStart) {
 
   ASSERT_EQ(1, add_initial_modules_invocation_count);
 
-  // The sample should have one frame from the context values and one from the
+  // The sample should have one frame from the context values aFFnd one from the
   // TestAuxUnwinder.
   ASSERT_EQ(1u, profile.samples.size());
   const std::vector<Frame>& frames = profile.samples[0];
@@ -1378,7 +1401,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest, AddAuxUnwinder_AfterStart) {
 
   int add_initial_modules_invocation_count = 0;
   const auto add_initial_modules_callback =
-      [&add_initial_modules_invocation_count]() {
+      [&add_initial_modules_invocation_count] {
         ++add_initial_modules_invocation_count;
       };
 
@@ -1507,7 +1530,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest,
           [&](SamplingProfilerThreadToken target_thread_token) {
             SynchronizedSampleTimes synchronized_sample_times;
             WaitableEvent sample_seen(WaitableEvent::ResetPolicy::AUTOMATIC);
-            PostSampleInvoker post_sample_invoker(BindLambdaForTesting([&]() {
+            PostSampleInvoker post_sample_invoker(BindLambdaForTesting([&] {
               synchronized_sample_times.AddNow();
               sample_seen.Signal();
             }));
@@ -1523,8 +1546,9 @@ PROFILER_TEST_F(StackSamplingProfilerTest,
                 RepeatingClosure(), &post_sample_invoker);
             profiler.Start();
             // Wait for 5 samples to be collected.
-            for (int i = 0; i < 5; ++i)
+            for (int i = 0; i < 5; ++i) {
               sample_seen.Wait();
+            }
             sample_times = synchronized_sample_times.GetTimes();
             // Record metadata on past samples, with and without a key value.
             // The range [times[1], times[3]] is guaranteed to include only
@@ -1574,7 +1598,7 @@ PROFILER_TEST_F(
 
   Profile profile1;
   WaitableEvent sampling_completed1;
-  TargetThread target_thread1(BindLambdaForTesting([&]() {
+  TargetThread target_thread1(BindLambdaForTesting([&] {
     StackSamplingProfiler profiler1(
         target_thread1.thread_token(), params,
         std::make_unique<TestProfileBuilder>(
@@ -1602,7 +1626,7 @@ PROFILER_TEST_F(
 
   Profile profile2;
   WaitableEvent sampling_completed2;
-  TargetThread target_thread2(BindLambdaForTesting([&]() {
+  TargetThread target_thread2(BindLambdaForTesting([&] {
     StackSamplingProfiler profiler2(
         target_thread2.thread_token(), params,
         std::make_unique<TestProfileBuilder>(
@@ -1683,7 +1707,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest,
           [&](SamplingProfilerThreadToken target_thread_token) {
             WaitableEvent sample_seen(WaitableEvent::ResetPolicy::AUTOMATIC);
             PostSampleInvoker post_sample_invoker(
-                BindLambdaForTesting([&]() { sample_seen.Signal(); }));
+                BindLambdaForTesting([&] { sample_seen.Signal(); }));
 
             StackSamplingProfiler profiler(
                 target_thread_token, params,
@@ -1724,7 +1748,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest,
 
   Profile profile1;
   WaitableEvent sampling_completed1;
-  TargetThread target_thread1(BindLambdaForTesting([&]() {
+  TargetThread target_thread1(BindLambdaForTesting([&] {
     StackSamplingProfiler profiler1(
         target_thread1.thread_token(), params,
         std::make_unique<TestProfileBuilder>(
@@ -1748,7 +1772,7 @@ PROFILER_TEST_F(StackSamplingProfilerTest,
 
   Profile profile2;
   WaitableEvent sampling_completed2;
-  TargetThread target_thread2(BindLambdaForTesting([&]() {
+  TargetThread target_thread2(BindLambdaForTesting([&] {
     StackSamplingProfiler profiler2(
         target_thread2.thread_token(), params,
         std::make_unique<TestProfileBuilder>(
