@@ -32,11 +32,8 @@ using testing::_;
 using testing::ElementsAre;
 using testing::NotNull;
 
-namespace base {
-namespace sequence_manager {
-namespace internal {
 // To avoid symbol collisions in jumbo builds.
-namespace task_queue_selector_unittest {
+namespace base::sequence_manager::internal::task_queue_selector_unittest {
 
 namespace {
 const TaskQueue::QueuePriority kHighestPriority = 0;
@@ -366,8 +363,9 @@ class TaskQueueSelectorStarvationTest : public TaskQueueSelectorTest {
 TEST_F(TaskQueueSelectorStarvationTest,
        HigherPriorityWorkStarvesLowerPriorityWork) {
   size_t queue_order[kTaskQueueCount];
-  for (size_t i = 0; i < kTaskQueueCount; i++)
+  for (size_t i = 0; i < kTaskQueueCount; i++) {
     queue_order[i] = i;
+  }
   TestPriorityOrder(queue_order, kTaskQueueCount);
 }
 
@@ -376,8 +374,9 @@ TEST_F(TaskQueueSelectorStarvationTest,
   // Enqueue tasks in order from lowest to highest priority, and check that they
   // still run in order from highest to lowest priority.
   size_t queue_order[kTaskQueueCount];
-  for (size_t i = 0; i < kTaskQueueCount; i++)
+  for (size_t i = 0; i < kTaskQueueCount; i++) {
     queue_order[i] = (kTaskQueueCount - i) - 1;
+  }
   TestPriorityOrder(queue_order, kTaskQueueCount);
 }
 
@@ -680,7 +679,4 @@ TEST_F(ActivePriorityTrackerTest, HighestActivePriority) {
   EXPECT_FALSE(active_priority_tracker_.HasActivePriority());
 }
 
-}  // namespace task_queue_selector_unittest
-}  // namespace internal
-}  // namespace sequence_manager
-}  // namespace base
+}  // namespace base::sequence_manager::internal::task_queue_selector_unittest
