@@ -1,31 +1,36 @@
-// Copyright 2015 The Chromium Authors. All rights reserved.
+// Copyright 2015 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CC_ANIMATION_ELEMENT_ANIMATIONS_H_
 #define CC_ANIMATION_ELEMENT_ANIMATIONS_H_
 
-#include <memory>
-#include <vector>
-
+#include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
 #include "base/observer_list.h"
 #include "cc/animation/animation_export.h"
-#include "cc/animation/animation_target.h"
+#include "cc/animation/filter_animation_curve.h"
+#include "cc/animation/scroll_offset_animation_curve.h"
 #include "cc/paint/element_id.h"
+#include "cc/paint/paint_worklet_input.h"
 #include "cc/trees/property_animation_state.h"
 #include "cc/trees/target_property.h"
-#include "ui/gfx/geometry/scroll_offset.h"
-#include "ui/gfx/transform.h"
+#include "ui/gfx/animation/keyframe/animation_curve.h"
+#include "ui/gfx/animation/keyframe/target_property.h"
+#include "ui/gfx/geometry/point_f.h"
+#include "ui/gfx/geometry/transform.h"
+
+namespace gfx {
+class TransformOperations;
+}  // namespace gfx
 
 namespace cc {
 
 class AnimationHost;
 class FilterOperations;
 class KeyframeEffect;
-class TransformOperations;
+class KeyframeModel;
 enum class ElementListType;
-struct AnimationEvent;
 
 // An ElementAnimations owns a list of all KeyframeEffects attached to a single
 // target (represented by an ElementId).
@@ -34,7 +39,11 @@ struct AnimationEvent;
 // of the word; this naming is a legacy leftover. A target is just an amorphous
 // blob that has properties that can be animated.
 class CC_ANIMATION_EXPORT ElementAnimations
-    : public AnimationTarget,
+    : public gfx::FloatAnimationCurve::Target,
+      public gfx::ColorAnimationCurve::Target,
+      public gfx::TransformAnimationCurve::Target,
+      public ScrollOffsetAnimationCurve::Target,
+      public FilterAnimationCurve::Target,
       public base::RefCounted<ElementAnimations> {
  public:
   static scoped_refptr<ElementAnimations> Create(AnimationHost* host,
@@ -52,11 +61,7 @@ class CC_ANIMATION_EXPORT ElementAnimations
 
   void ClearAffectedElementTypes(const PropertyToElementIdMap& element_id_map);
 
-  // Called when |element_id| is available to animate in |list_type|.
-  void ElementIdRegistered(ElementId element_id, ElementListType list_type);
-
-  // Called when |element_id| is no longer avialable to animate in |list_type|.
-  void ElementIdUnregistered(ElementId element_id, ElementListType list_type);
+  void RemoveKeyframeEffects();
 
   void AddKeyframeEffect(KeyframeEffect* keyframe_effect);
   void RemoveKeyframeEffect(KeyframeEffect* keyframe_effect);
@@ -75,7 +80,8 @@ class CC_ANIMATION_EXPORT ElementAnimations
   // Returns true if there are any KeyframeModels at all to process.
   bool HasAnyKeyframeModel() const;
 
-  bool HasAnyAnimationTargetingProperty(TargetProperty::Type property) const;
+  bool HasAnyAnimationTargetingProperty(TargetProperty::Type property,
+                                        ElementId element_id) const;
 
   // Returns true if there is an animation that is either currently animating
   // the given property or scheduled to animate this property in the future, and
@@ -88,39 +94,12 @@ class CC_ANIMATION_EXPORT ElementAnimations
   bool IsCurrentlyAnimatingProperty(TargetProperty::Type target_property,
                                     ElementListType list_type) const;
 
-  void NotifyAnimationStarted(const AnimationEvent& event);
-  void NotifyAnimationFinished(const AnimationEvent& event);
-  void NotifyAnimationAborted(const AnimationEvent& event);
-  void NotifyAnimationTakeover(const AnimationEvent& event);
-
-  bool has_element_in_active_list() const {
-    return has_element_in_active_list_;
-  }
-  bool has_element_in_pending_list() const {
-    return has_element_in_pending_list_;
-  }
-  bool has_element_in_any_list() const {
-    return has_element_in_active_list_ || has_element_in_pending_list_;
-  }
-
-  void set_has_element_in_active_list(bool has_element_in_active_list) {
-    has_element_in_active_list_ = has_element_in_active_list;
-  }
-  void set_has_element_in_pending_list(bool has_element_in_pending_list) {
-    has_element_in_pending_list_ = has_element_in_pending_list;
-  }
-
   bool AnimationsPreserveAxisAlignment() const;
 
-  // Gets scales transform animations. On return, |maximum_scale| is the maximum
-  // scale along any dimension at any destination in active scale animations,
-  // and |starting_scale| is the maximum of starting animation scale along any
-  // dimension at any destination in active scale animations. They are set to
-  // kNotScaled if there is no active scale animation or the scales cannot be
-  // computed.
-  void GetAnimationScales(ElementListType list_type,
-                          float* maximum_scale,
-                          float* starting_scale) const;
+  // Returns the maximum scale along any dimension at any destination in active
+  // scale animations, or kInvalidScale if there is no active transform
+  // animation or the scale cannot be computed.
+  float MaximumScale(ElementId element_id, ElementListType list_type) const;
 
   bool ScrollOffsetAnimationWasInterrupted() const;
 
@@ -134,27 +113,30 @@ class CC_ANIMATION_EXPORT ElementAnimations
   // that have changed since the last update.
   void UpdateClientAnimationState();
 
-  void NotifyClientFloatAnimated(float opacity,
-                                 int target_property_id,
-                                 KeyframeModel* keyframe_model) override;
-  void NotifyClientFilterAnimated(const FilterOperations& filter,
-                                  int target_property_id,
-                                  KeyframeModel* keyframe_model) override;
-  void NotifyClientSizeAnimated(const gfx::SizeF& size,
-                                int target_property_id,
-                                KeyframeModel* keyframe_model) override {}
-  void NotifyClientColorAnimated(SkColor color,
-                                 int target_property_id,
-                                 KeyframeModel* keyframe_model) override {}
-  void NotifyClientTransformOperationsAnimated(
-      const TransformOperations& operations,
-      int target_property_id,
-      KeyframeModel* keyframe_model) override;
-  void NotifyClientScrollOffsetAnimated(const gfx::ScrollOffset& scroll_offset,
-                                        int target_property_id,
-                                        KeyframeModel* keyframe_model) override;
+  // TODO(crbug.com/40747850): Animation targets should be attached to curves
+  // when they're created and the concrete subclass is known. This function
+  // exists as a stopgap: the animation machinery previously expected to
+  // announce a target and then pass curves that would implicitly animate the
+  // target (i.e., the machinery handled the attachment).
+  void AttachToCurve(gfx::AnimationCurve* c);
 
-  gfx::ScrollOffset ScrollOffsetForAnimation() const;
+  void OnFloatAnimated(const float& value,
+                       int target_property_id,
+                       gfx::KeyframeModel* keyframe_model) override;
+  void OnFilterAnimated(const FilterOperations& filter,
+                        int target_property_id,
+                        gfx::KeyframeModel* keyframe_model) override;
+  void OnColorAnimated(const SkColor& color,
+                       int target_property_id,
+                       gfx::KeyframeModel* keyframe_model) override;
+  void OnTransformAnimated(const gfx::TransformOperations& operations,
+                           int target_property_id,
+                           gfx::KeyframeModel* keyframe_model) override;
+  void OnScrollOffsetAnimated(const gfx::PointF& scroll_offset,
+                              int target_property_id,
+                              gfx::KeyframeModel* keyframe_model) override;
+
+  std::optional<gfx::PointF> ScrollOffsetForAnimation() const;
 
   // Returns a map of target property to the ElementId for that property, for
   // KeyframeEffects associated with this ElementAnimations.
@@ -179,43 +161,57 @@ class CC_ANIMATION_EXPORT ElementAnimations
 
   void OnFilterAnimated(ElementListType list_type,
                         const FilterOperations& filters,
-                        KeyframeModel* keyframe_model);
+                        gfx::KeyframeModel* keyframe_model);
   void OnBackdropFilterAnimated(ElementListType list_type,
                                 const FilterOperations& backdrop_filters,
-                                KeyframeModel* keyframe_model);
+                                gfx::KeyframeModel* keyframe_model);
   void OnOpacityAnimated(ElementListType list_type,
                          float opacity,
-                         KeyframeModel* keyframe_model);
+                         gfx::KeyframeModel* keyframe_model);
+  // In addition to custom property animations, these also represent animations
+  // of native properties whose values are known to the Blink PaintWorklet
+  // responsible for painting them but not known to the compositor. The
+  // compositor animates a simple float progress which is then passed into blink
+  // code to interpolate. Unlike other native properties listed above, CC is not
+  // capable of drawing interpolations of these properties and defers to
+  // NativePaintWorklet subclasses to interpret the animation progress as it
+  // pertains to how to paint the native property.
+  void OnCustomPropertyAnimated(PaintWorkletInput::PropertyValue property_value,
+                                KeyframeModel* keyframe_model,
+                                int target_property_id);
   void OnTransformAnimated(ElementListType list_type,
                            const gfx::Transform& transform,
-                           KeyframeModel* keyframe_model);
+                           gfx::KeyframeModel* keyframe_model);
   void OnScrollOffsetAnimated(ElementListType list_type,
-                              const gfx::ScrollOffset& scroll_offset,
-                              KeyframeModel* keyframe_model);
+                              const gfx::PointF& scroll_offset,
+                              gfx::KeyframeModel* keyframe_model);
 
-  static TargetProperties GetPropertiesMaskForAnimationState();
+  static gfx::TargetProperties GetPropertiesMaskForAnimationState();
+
+  void UpdateMaximumScale(ElementId element_id,
+                          ElementListType list_type,
+                          float* cached_scale);
 
   void UpdateKeyframeEffectsTickingState() const;
   void RemoveKeyframeEffectsFromTicking() const;
 
-  bool KeyframeModelAffectsActiveElements(KeyframeModel* keyframe_model) const;
-  bool KeyframeModelAffectsPendingElements(KeyframeModel* keyframe_model) const;
+  bool KeyframeModelAffectsActiveElements(
+      gfx::KeyframeModel* keyframe_model) const;
+  bool KeyframeModelAffectsPendingElements(
+      gfx::KeyframeModel* keyframe_model) const;
 
   base::ObserverList<KeyframeEffect>::Unchecked keyframe_effects_list_;
-  AnimationHost* animation_host_;
+  raw_ptr<AnimationHost> animation_host_;
   ElementId element_id_;
 
-  bool has_element_in_active_list_;
-  bool has_element_in_pending_list_;
-
   mutable bool needs_push_properties_;
 
   PropertyAnimationState active_state_;
   PropertyAnimationState pending_state_;
-  float active_maximum_scale_;
-  float active_starting_scale_;
-  float pending_maximum_scale_;
-  float pending_starting_scale_;
+  float transform_property_active_maximum_scale_;
+  float transform_property_pending_maximum_scale_;
+  float scale_property_active_maximum_scale_;
+  float scale_property_pending_maximum_scale_;
 };
 
 }  // namespace cc
