@@ -14,58 +14,14 @@
 
 namespace features {
 
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kAnimatedImageResume);
-CC_BASE_EXPORT extern bool IsImpulseScrollAnimationEnabled();
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kAlignSurfaceLayerImplToPixelGrid);
 CC_BASE_EXPORT extern bool IsNaturalScrollAnimationEnabled();
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kSynchronizedScrolling);
 
-// When enabled, scrolling within a covering snap area avoids or snaps to inner
-// nested areas, avoiding resting on positions which do not snap the inner area.
-// E.g. when scrolling within snap area A, it will stop either before/after
-// snap area B or with B snapped.
-//   --------
-//  | A      |
-//  |        |
-//  |  ---   |
-//  | | B |  |
-//  |  ---   |
-//  |        |
-//   --------
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kScrollSnapCoveringAvoidNestedSnapAreas);
-
-// When enabled, scrolling within a covering snap area uses the native fling,
-// allowing much more natural scrolling within these areas.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kScrollSnapCoveringUseNativeFling);
-
-// When enabled, if a snap container is snapping to a large snap area, it will
-// snap to the closest covering position if it is closer than the aligned
-// position. This avoid unnecessary jumps that attempt to honor the
-// scroll-snap-align value.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kScrollSnapPreferCloserCovering);
-
 // Sets raster tree priority to NEW_CONTENT_TAKES_PRIORITY when performing a
 // unified scroll with main-thread repaint reasons.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kMainRepaintScrollPrefersNewContent);
 
-
-// Whether RenderSurface::common_ancestor_clip_id() is used to clip to the
-// common ancestor clip when any contributing layer escapes the clip of the
-// render surface's owning effect.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kRenderSurfaceCommonAncestorClip);
-
-// When enabled, the main thread does not block while commit is running on the
-// impl thread.
-// WARNING: This feature is not yet safe to enable. Work is needed to ensure
-// that main thread cc data structures are not modified on the main thread while
-// commit is running concurrently on the impl thread.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kNonBlockingCommit);
-
-// When enabled, LayerTreeImpl will not preserve the last mutation. This map
-// of the last mutated value should not be necessary as animations are always
-// ticked after the commit which should restore their animated values. Removing
-// this should improve performance and reduce technical complexity.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kNoPreserveLastMutation);
-
 // When enabled, the scheduler will allow deferring impl invalidation frames
 // for N frames (default 1) to reduce contention with main frames, allowing
 // main a chance to commit.
@@ -76,41 +32,17 @@ CC_BASE_EXPORT extern const base::FeatureParam<int>
 // Use DMSAA instead of MSAA for rastering tiles.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kUseDMSAAForTiles);
 
-#if BUILDFLAG(IS_ANDROID)
-// Use DMSAA instead of MSAA for rastering tiles on Android GL backend. Note
-// that the above flag kUseDMSAAForTiles is used for Android Vulkan backend.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kUseDMSAAForTilesAndroidGL);
-
-// Break synchronization of scrolling website content and browser controls in
-// android to see performance gains for moving browser controls to viz.
-// WARNING: Don't enable this feature! It should only be used to measure
-// performance on prestable channels.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kAndroidNoSurfaceSyncForBrowserControls);
-#endif
-
-// Updating browser controls state will IPC directly from browser main to the
-// compositor thread. Previously this proxied through the renderer main thread.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kUpdateBrowserControlsWithoutProxy);
-
 // Enables shared image cache for gpu used by CC instances instantiated for UI.
 // TODO(https://crbug.com/c/1378251): this shall also be possible to use by
 // renderers.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kUIEnableSharedImageCacheForGpu);
 
-// When LayerTreeHostImpl::ReclaimResources() is called in background, trigger a
-// flush to actually reclaim resources.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kReclaimResourcesFlushInBackground);
-
 // When LayerTreeHostImpl::ReclaimResources() is called in background, trigger a
 // additional delayed flush to reclaim resources.
 //
 // Enabled 03/2024, kept to run a holdback experiment.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kReclaimResourcesDelayedFlushInBackground);
 
-// Allow CC FrameRateEstimater to reduce the frame rate to half of the default
-// if the condition meets the requirement.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kReducedFrameRateEstimation);
-
 // Use 4x MSAA (vs 8) on High DPI screens.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kDetectHiDpiForMsaa);
 
@@ -153,15 +85,6 @@ CC_BASE_EXPORT BASE_DECLARE_FEATURE(kClearCanvasResourcesInBackground);
 // to when tracing is enabled.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kMetricsTracingCalculationReduction);
 
-// Temporary features to enable the fix for b/328665503 independently from
-// adding the implementation.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kPaintWithGainmapShader);
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kPaintWithGlobalToneMapFilter);
-
-// When enabled we will restore older FrameSequenceTracker sequence order
-// enforcing that can miss backfilled frames.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kMetricsBackfillAdjustmentHoldback);
-
 // When enabled we will submit the 'CopySharedImage' in one call and not batch
 // it up into 4MiB increments.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kNonBatchedCopySharedImage);
@@ -181,6 +104,10 @@ CC_BASE_EXPORT extern const base::FeatureParam<double>
 // tree Activation. See crbug.com/40335690.
 CC_BASE_EXPORT BASE_DECLARE_FEATURE(kDontAlwaysPushPictureLayerImpls);
 
+// When enabled, image quality settings will be preserved in the discardable
+// image map.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kPreserveDiscardableImageMapQuality);
+
 // When enabled, the renderer asks the compositor to request warming up and
 // create FrameSink speculatively even if invisible. Currently, this is intended
 // to be used when prerender initial navigation is happening in background.
@@ -196,10 +123,18 @@ CC_BASE_EXPORT bool IsCCSlimmingEnabled();
 // Modes for `kWaitForLateScrollEvents` changing event dispatch. Where the
 // default is to just always enqueue scroll events.
 //
-// `kScrollEventDispatchModeNameDispatchScrollEventsImmediately` will wait for
-// `kWaitForLateScrollEventsDeadlineRatio` of the frame interval for input.
-// During this time scroll events will be dispatched immediately. At the
-// deadline we will resume frame production and enqueuing input.
+// The ideal goal for both
+// `kScrollEventDispatchModeNameDispatchScrollEventsImmediately` and
+// `kScrollEventDispatchModeDispatchScrollEventsUntilDeadline` is that they will
+// wait for `kWaitForLateScrollEventsDeadlineRatio` of the frame interval for
+// input. During this time the first scroll event will be dispatched
+// immediately. Subsequent scroll events will be enqueued. At the deadline we
+// will resume frame production and enqueuing input.
+//
+// `kScrollEventDispatchModeNameDispatchScrollEventsImmediately` relies on
+// `cc::Scheduler` to control the deadline. However this is overridden if we are
+// waiting for Main-thread content. There are also fragile bugs which currently
+// prevent enforcing the deadline if frame production is no longer required.
 //
 // `kScrollEventDispatchModeNameUseScrollPredictorForEmptyQueue` checks when
 // we begin frame production, if the event queue is empty, we will generate a
@@ -211,6 +146,11 @@ CC_BASE_EXPORT bool IsCCSlimmingEnabled();
 // production, we will first attempt to generate a new prediction to dispatch.
 // As in `kScrollEventDispatchModeUseScrollPredictorForEmptyQueue`. After
 // which we will resume frame production and enqueuing input.
+//
+// `kScrollEventDispatchModeDispatchScrollEventsUntilDeadline` relies on
+// `blink::InputHandlerProxy` to directly enforce the deadline. This isolates us
+// from cc scheduling bugs. Allowing us to no longer dispatch events, even if
+// frame production has yet to complete.
 CC_BASE_EXPORT extern const base::FeatureParam<std::string>
     kScrollEventDispatchMode;
 CC_BASE_EXPORT extern const char
@@ -219,9 +159,14 @@ CC_BASE_EXPORT extern const char
     kScrollEventDispatchModeUseScrollPredictorForEmptyQueue[];
 CC_BASE_EXPORT extern const char
     kScrollEventDispatchModeUseScrollPredictorForDeadline[];
+CC_BASE_EXPORT extern const char
+    kScrollEventDispatchModeDispatchScrollEventsUntilDeadline[];
 
-// Enables GPU-side layer trees for content rendering.
-CC_BASE_EXPORT BASE_DECLARE_FEATURE(kVizLayers);
+// Enables Viz service-side layer trees for content rendering.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kTreesInViz);
+
+// Enables Viz service-side layer tree animations for content rendering.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kTreeAnimationsInViz);
 
 // When enabled HTMLImageElement::decode() will initiate the decode task right
 // away rather than piggy-backing on the next BeginMainFrame.
@@ -233,6 +178,40 @@ CC_BASE_EXPORT BASE_DECLARE_FEATURE(kThrottleFrameRateOnManyDidNotProduceFrame);
 CC_BASE_EXPORT extern const base::FeatureParam<int>
     kNumDidNotProduceFrameBeforeThrottle;
 
+// When enabled, the CC tree priority will be switched to
+// NEW_CONTENT_TAKES_PRIORITY during long scroll that cause checkerboarding.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kNewContentForCheckerboardedScrolls);
+
+// When enabled, impl-only scroll animations may execute concurrently.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kMultipleImplOnlyScrollAnimations);
+CC_BASE_EXPORT extern bool MultiImplOnlyScrollAnimationsSupported();
+
+// When enabled, and an image decode is requested by both a tile task and
+// explicitly via img.decode(), it will be decoded only once.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kPreventDuplicateImageDecodes);
+
+// When enabled, fix bug where an image decode cache entry last use timestamp is
+// initialized to 0 instead of now.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kInitImageDecodeLastUseTime);
+
+// The position affected by the safe area inset bottom will be handled by CC in
+// the Render Compositor Thread. The transform metrix y is adjusted for all
+// affected nodes.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kDynamicSafeAreaInsetsSupportedByCC);
+
+// On devices with a high refresh rate, whether to throttle main (not impl)
+// frame production to 60Hz.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kThrottleMainFrameTo60Hz);
+
+// When enabled, stops the export of most DFCMetrics.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kStopExportDFCMetrics);
+CC_BASE_EXPORT extern bool StopExportDFCMetrics();
+
+// When enabled, we save the `EventMetrics` for a scroll, even when the result
+// is no damage. So that the termination can be per properly attributed to the
+// end of frame production for the given VSync.
+CC_BASE_EXPORT BASE_DECLARE_FEATURE(kZeroScrollMetricsUpdate);
+
 }  // namespace features
 
 #endif  // CC_BASE_FEATURES_H_
