@@ -6,18 +6,22 @@
 #define CC_RASTER_GPU_RASTER_BUFFER_PROVIDER_H_
 
 #include <stdint.h>
+
 #include <memory>
 #include <vector>
 
 #include "base/memory/raw_ptr.h"
 #include "base/rand_util.h"
 #include "base/time/time.h"
+#include "cc/raster/raster_buffer.h"
 #include "cc/raster/raster_buffer_provider.h"
 #include "cc/raster/raster_query_queue.h"
 #include "cc/trees/raster_capabilities.h"
 #include "components/viz/common/resources/shared_image_format.h"
 #include "gpu/command_buffer/common/sync_token.h"
 
+class GURL;
+
 namespace gpu {
 namespace raster {
 class RasterInterface;
@@ -69,13 +73,11 @@ class CC_EXPORT GpuRasterBufferProvider : public RasterBufferProvider {
   void Flush() override;
 
  private:
-  class GpuRasterBacking;
-
   class RasterBufferImpl : public RasterBuffer {
    public:
     RasterBufferImpl(GpuRasterBufferProvider* client,
                      const ResourcePool::InUsePoolResource& in_use_resource,
-                     GpuRasterBacking* backing,
+                     ResourcePool::GpuBacking* backing,
                      bool resource_has_previous_content,
                      bool depends_on_at_raster_decodes,
                      bool depends_on_hardware_accelerated_jpeg_candidates,
@@ -124,7 +126,7 @@ class CC_EXPORT GpuRasterBufferProvider : public RasterBufferProvider {
 
     // These fields may only be used on the compositor thread.
     const raw_ptr<GpuRasterBufferProvider> client_;
-    raw_ptr<GpuRasterBacking> backing_;
+    raw_ptr<ResourcePool::GpuBacking> backing_;
 
     // These fields are for use on the worker thread.
     const gfx::Size resource_size_;
