@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,6 +6,7 @@
 
 #include <utility>
 
+#include "base/notreached.h"
 #include "cc/tiles/tiling_set_eviction_queue.h"
 
 namespace cc {
@@ -28,8 +29,7 @@ TilingSetEvictionQueue::TilingSetEvictionQueue(
   current_tile_ = *eventually_iterator_;
 }
 
-TilingSetEvictionQueue::~TilingSetEvictionQueue() {
-}
+TilingSetEvictionQueue::~TilingSetEvictionQueue() = default;
 
 void TilingSetEvictionQueue::GenerateTilingOrder(
     PictureLayerTilingSet* tiling_set) {
@@ -85,7 +85,6 @@ void TilingSetEvictionQueue::AdvancePhase() {
     switch (phase_) {
       case EVENTUALLY_RECT:
         NOTREACHED();
-        break;
       case SOON_BORDER_RECT:
         soon_iterator_ = SoonBorderTilingIterator(&tilings_, tree_);
         if (!soon_iterator_.done())
@@ -235,8 +234,8 @@ bool TilingSetEvictionQueue::EvictionRectIterator::GetFirstTileAndCheckIfValid(
     if (tiling->pending_visible_rect().Intersects(tile_rect))
       return false;
   }
-  prioritized_tile_ = (*tilings_)[tiling_index_]->MakePrioritizedTile(
-      tile, priority_rect_type_);
+  prioritized_tile_ = tiling->MakePrioritizedTile(tile, priority_rect_type_,
+                                                  tiling->IsTileOccluded(tile));
   // In other cases, the tile we got is a viable candidate, return true.
   return true;
 }
