@@ -1,4 +1,4 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,9 @@
 #include <memory>
 #include <utility>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/no_destructor.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/task/post_task.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/device_service.h"
@@ -103,13 +102,6 @@ void CountAndroidDevices(base::OnceCallback<void(int)> callback,
   std::move(callback).Run(info_list.size());
 }
 
-void BindDeviceServiceOnUIThread(
-    mojo::PendingReceiver<device::mojom::UsbDeviceManager> receiver) {
-  DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
-  // Bind to the DeviceService for USB device manager.
-  content::GetDeviceService().BindUsbDeviceManager(std::move(receiver));
-}
-
 }  // namespace
 
 AndroidInterfaceInfo::AndroidInterfaceInfo(
@@ -150,14 +142,15 @@ void UsbDeviceManagerHelper::SetUsbManagerForTesting(
   GetInstance()->SetUsbManagerForTestingInternal(std::move(fake_usb_manager));
 }
 
-UsbDeviceManagerHelper::UsbDeviceManagerHelper() {}
+UsbDeviceManagerHelper::UsbDeviceManagerHelper() = default;
 
 UsbDeviceManagerHelper::~UsbDeviceManagerHelper() {
-  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 }
 
 void UsbDeviceManagerHelper::GetAndroidDevices(
     AndroidDeviceInfoListCallback callback) {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   EnsureUsbDeviceManagerConnection();
 
   DCHECK(device_manager_);
@@ -171,15 +164,17 @@ void UsbDeviceManagerHelper::GetAndroidDevices(
 void UsbDeviceManagerHelper::GetDevice(
     const std::string& guid,
     mojo::PendingReceiver<device::mojom::UsbDevice> device_receiver) {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   EnsureUsbDeviceManagerConnection();
 
   DCHECK(device_manager_);
-  device_manager_->GetDevice(guid, std::move(device_receiver),
+  device_manager_->GetDevice(guid, /*blocked_interface_classes=*/{},
+                             std::move(device_receiver),
                              /*device_client=*/mojo::NullRemote());
 }
 
 void UsbDeviceManagerHelper::EnsureUsbDeviceManagerConnection() {
-  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   // Ensure connection with the Device Service.
   if (device_manager_)
     return;
@@ -193,9 +188,8 @@ void UsbDeviceManagerHelper::EnsureUsbDeviceManagerConnection() {
     return;
   }
 
-  base::PostTask(FROM_HERE, {content::BrowserThread::UI},
-                 base::BindOnce(&BindDeviceServiceOnUIThread,
-                                device_manager_.BindNewPipeAndPassReceiver()));
+  content::GetDeviceService().BindUsbDeviceManager(
+      device_manager_.BindNewPipeAndPassReceiver());
 
   device_manager_.set_disconnect_handler(
       base::BindOnce(&UsbDeviceManagerHelper::OnDeviceManagerConnectionError,
@@ -204,6 +198,7 @@ void UsbDeviceManagerHelper::EnsureUsbDeviceManagerConnection() {
 
 void UsbDeviceManagerHelper::CountDevicesInternal(
     base::OnceCallback<void(int)> callback) {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   EnsureUsbDeviceManagerConnection();
 
   DCHECK(device_manager_);
@@ -217,12 +212,12 @@ void UsbDeviceManagerHelper::CountDevicesInternal(
 
 void UsbDeviceManagerHelper::SetUsbManagerForTestingInternal(
     mojo::PendingRemote<device::mojom::UsbDeviceManager> fake_usb_manager) {
-  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(fake_usb_manager);
   testing_device_manager_ = std::move(fake_usb_manager);
 }
 
 void UsbDeviceManagerHelper::OnDeviceManagerConnectionError() {
-  DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   device_manager_.reset();
 }
