@@ -13,10 +13,10 @@
 #include "chrome/browser/extensions/api/braille_display_private/braille_controller.h"
 #include "chrome/browser/profiles/profile.h"
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
 #include "chrome/browser/ash/accessibility/accessibility_manager.h"
 #include "chrome/browser/ash/login/lock/screen_locker.h"
-#include "chrome/browser/profiles/profile_manager.h"
+#include "extensions/browser/extensions_browser_client.h"
 #endif
 
 namespace OnDisplayStateChanged =
@@ -48,8 +48,7 @@ BrailleDisplayPrivateAPI::BrailleDisplayPrivateAPI(
     : profile_(Profile::FromBrowserContext(context)),
       event_delegate_(new DefaultEventDelegate(this, profile_)) {}
 
-BrailleDisplayPrivateAPI::~BrailleDisplayPrivateAPI() {
-}
+BrailleDisplayPrivateAPI::~BrailleDisplayPrivateAPI() = default;
 
 void BrailleDisplayPrivateAPI::Shutdown() {
   event_delegate_.reset();
@@ -86,12 +85,9 @@ void BrailleDisplayPrivateAPI::OnBrailleKeyEvent(const KeyEvent& key_event) {
 }
 
 bool BrailleDisplayPrivateAPI::IsProfileActive() {
-#if BUILDFLAG(IS_CHROMEOS_ASH)
-  // Since we are creating one instance per profile / user, we should be fine
-  // comparing against the active user. That said - if we ever change that,
-  // this code will need to be changed.
-  return profile_->IsSameOrParent(ProfileManager::GetActiveUserProfile());
-#else  // !BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
+  return extensions::ExtensionsBrowserClient::Get()->IsActiveContext(profile_);
+#else  // !BUILDFLAG(IS_CHROMEOS)
   return true;
 #endif
 }
@@ -191,16 +187,15 @@ void BrailleDisplayPrivateWriteDotsFunction::WriteDotsOnIO() {
 
 ExtensionFunction::ResponseAction
 BrailleDisplayPrivateUpdateBluetoothBrailleDisplayAddressFunction::Run() {
-#if !BUILDFLAG(IS_CHROMEOS_ASH)
-  NOTREACHED_IN_MIGRATION();
-  return RespondNow(Error("Unsupported on this platform."));
-#else
+#if BUILDFLAG(IS_CHROMEOS)
   EXTENSION_FUNCTION_VALIDATE(args().size() >= 1);
   EXTENSION_FUNCTION_VALIDATE(args()[0].is_string());
   const std::string& address = args()[0].GetString();
   ash::AccessibilityManager::Get()->UpdateBluetoothBrailleDisplayAddress(
       address);
   return RespondNow(NoArguments());
+#else
+  NOTREACHED();
 #endif
 }
 
