@@ -1,4 +1,4 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,28 +7,27 @@
 #include <memory>
 #include <utility>
 
-#include "base/bind.h"
+#include "base/check.h"
 #include "base/files/file_path.h"
+#include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/strings/string_util.h"
-#include "base/task/post_task.h"
 #include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
 #include "chrome/browser/extensions/api/automation_internal/chrome_automation_internal_api_delegate.h"
 #include "chrome/browser/extensions/api/chrome_device_permissions_prompt.h"
 #include "chrome/browser/extensions/api/declarative_content/chrome_content_rules_registry.h"
 #include "chrome/browser/extensions/api/declarative_content/default_content_predicate_evaluators.h"
-#include "chrome/browser/extensions/api/extension_action/extension_action_api.h"
 #include "chrome/browser/extensions/api/feedback_private/chrome_feedback_private_delegate.h"
 #include "chrome/browser/extensions/api/file_system/chrome_file_system_delegate.h"
+#include "chrome/browser/extensions/api/file_system/consent_provider_impl.h"
 #include "chrome/browser/extensions/api/management/chrome_management_api_delegate.h"
 #include "chrome/browser/extensions/api/messaging/chrome_messaging_delegate.h"
 #include "chrome/browser/extensions/api/metrics_private/chrome_metrics_private_delegate.h"
-#include "chrome/browser/extensions/api/networking_cast_private/chrome_networking_cast_private_delegate.h"
 #include "chrome/browser/extensions/api/storage/managed_value_store_cache.h"
 #include "chrome/browser/extensions/api/storage/sync_value_store_cache.h"
-#include "chrome/browser/extensions/chrome_extension_web_contents_observer.h"
-#include "chrome/browser/extensions/extension_action.h"
-#include "chrome/browser/extensions/extension_action_manager.h"
+#include "chrome/browser/extensions/extension_action_dispatcher.h"
 #include "chrome/browser/extensions/extension_action_runner.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/extensions/system_display/display_info_provider.h"
@@ -39,16 +38,20 @@
 #include "chrome/browser/guest_view/mime_handler_view/chrome_mime_handler_view_guest_delegate.h"
 #include "chrome/browser/guest_view/web_view/chrome_web_view_guest_delegate.h"
 #include "chrome/browser/guest_view/web_view/chrome_web_view_permission_helper_delegate.h"
+#include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/search/instant_service.h"
 #include "chrome/browser/search/instant_service_factory.h"
-#include "chrome/browser/ui/pdf/chrome_pdf_web_contents_helper_client.h"
-#include "chrome/browser/ui/webui/devtools_ui.h"
+#include "chrome/browser/supervised_user/supervised_user_extensions_delegate_impl.h"
+#include "chrome/browser/supervised_user/supervised_user_service_factory.h"
+#include "chrome/browser/ui/browser_finder.h"
+#include "chrome/browser/ui/browser_navigator.h"
+#include "chrome/browser/ui/browser_navigator_params.h"
+#include "chrome/browser/ui/webui/devtools/devtools_ui.h"
+#include "chrome/common/buildflags.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/common/webui_url_constants.h"
-#include "components/pdf/browser/pdf_web_contents_helper.h"
-#include "components/performance_manager/performance_manager_tab_helper.h"
-#include "components/performance_manager/public/performance_manager.h"
 #include "components/signin/core/browser/signin_header_helper.h"
+#include "components/value_store/value_store_factory.h"
 #include "content/public/browser/browser_context.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
@@ -57,18 +60,29 @@
 #include "extensions/browser/api/system_display/display_info_provider.h"
 #include "extensions/browser/api/virtual_keyboard_private/virtual_keyboard_delegate.h"
 #include "extensions/browser/api/web_request/web_request_info.h"
+#include "extensions/browser/extension_action.h"
+#include "extensions/browser/extension_action_manager.h"
 #include "extensions/browser/extension_registry.h"
 #include "extensions/browser/guest_view/web_view/web_view_guest.h"
 #include "extensions/browser/guest_view/web_view/web_view_permission_helper.h"
-#include "extensions/browser/value_store/value_store_factory.h"
+#include "extensions/browser/supervised_user_extensions_delegate.h"
 #include "google_apis/gaia/gaia_urls.h"
+#include "pdf/buildflags.h"
 #include "printing/buildflags/buildflags.h"
+#include "services/network/public/mojom/fetch_api.mojom-shared.h"
+#include "ui/base/page_transition_types.h"
+#include "ui/base/window_open_disposition.h"
 #include "url/gurl.h"
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS)
 #include "chrome/browser/extensions/api/file_handlers/non_native_file_system_delegate_chromeos.h"
+#include "chrome/browser/extensions/api/file_system/chrome_file_system_delegate_ash.h"
 #include "chrome/browser/extensions/api/media_perception_private/media_perception_api_delegate_chromeos.h"
 #include "chrome/browser/extensions/api/virtual_keyboard_private/chrome_virtual_keyboard_delegate.h"
+#include "chromeos/ash/components/settings/cros_settings.h"
+#endif
+
+#if BUILDFLAG(IS_CHROMEOS)
 #include "chrome/browser/extensions/clipboard_extension_helper_chromeos.h"
 #endif
 
@@ -78,40 +92,31 @@
 
 namespace extensions {
 
-ChromeExtensionsAPIClient::ChromeExtensionsAPIClient() {}
+ChromeExtensionsAPIClient::ChromeExtensionsAPIClient() = default;
 
-ChromeExtensionsAPIClient::~ChromeExtensionsAPIClient() {}
+ChromeExtensionsAPIClient::~ChromeExtensionsAPIClient() = default;
 
 void ChromeExtensionsAPIClient::AddAdditionalValueStoreCaches(
     content::BrowserContext* context,
-    const scoped_refptr<ValueStoreFactory>& factory,
-    const scoped_refptr<base::ObserverListThreadSafe<SettingsObserver>>&
-        observers,
-    std::map<settings_namespace::Namespace, ValueStoreCache*>* caches) {
+    const scoped_refptr<value_store::ValueStoreFactory>& factory,
+    SettingsChangedCallback observer,
+    std::map<settings_namespace::Namespace,
+             raw_ptr<ValueStoreCache, CtnExperimental>>* caches) {
   // Add support for chrome.storage.sync.
   (*caches)[settings_namespace::SYNC] =
-      new SyncValueStoreCache(factory, observers, context->GetPath());
+      new SyncValueStoreCache(factory, observer, context->GetPath());
 
   // Add support for chrome.storage.managed.
-  (*caches)[settings_namespace::MANAGED] =
-      new ManagedValueStoreCache(context, factory, observers);
+  (*caches)[settings_namespace::MANAGED] = new ManagedValueStoreCache(
+      *Profile::FromBrowserContext(context), factory, observer);
 }
 
 void ChromeExtensionsAPIClient::AttachWebContentsHelpers(
     content::WebContents* web_contents) const {
   favicon::CreateContentFaviconDriverForWebContents(web_contents);
 #if BUILDFLAG(ENABLE_PRINTING)
-  printing::InitializePrinting(web_contents);
+  printing::InitializePrintingForWebContents(web_contents);
 #endif
-  pdf::PDFWebContentsHelper::CreateForWebContentsWithClient(
-      web_contents, std::make_unique<ChromePDFWebContentsHelperClient>());
-
-  extensions::ChromeExtensionWebContentsObserver::CreateForWebContents(
-      web_contents);
-  if (performance_manager::PerformanceManager::IsAvailable()) {
-    performance_manager::PerformanceManagerTabHelper::CreateForWebContents(
-        web_contents);
-  }
 }
 
 bool ChromeExtensionsAPIClient::ShouldHideResponseHeader(
@@ -119,9 +124,6 @@ bool ChromeExtensionsAPIClient::ShouldHideResponseHeader(
     const std::string& header_name) const {
   // Gaia may send a OAUth2 authorization code in the Dice response header,
   // which could allow an extension to generate a refresh token for the account.
-  // TODO(crbug.com/890006): Determine if the code here can be cleaned up
-  // since browser initiated non-navigation requests are now hidden from
-  // extensions.
   return (
       (url.host_piece() == GaiaUrls::GetInstance()->gaia_url().host_piece()) &&
       (base::CompareCaseInsensitiveASCII(header_name,
@@ -137,8 +139,9 @@ bool ChromeExtensionsAPIClient::ShouldHideBrowserNetworkRequest(
   // But we do still need to protect some sensitive sub-frame navigation
   // requests.
   // Exclude main frame navigation requests.
-  bool is_browser_request = request.render_process_id == -1 &&
-                            request.type != content::ResourceType::kMainFrame;
+  bool is_browser_request =
+      request.render_process_id == -1 &&
+      request.web_request_type != WebRequestResourceType::MAIN_FRAME;
 
   // Hide requests made by the Devtools frontend.
   bool is_sensitive_request =
@@ -150,15 +153,17 @@ bool ChromeExtensionsAPIClient::ShouldHideBrowserNetworkRequest(
        request.initiator ==
            url::Origin::Create(GURL(chrome::kChromeUINewTabURL)));
 
-  // Hide requests made by the browser on behalf of the local NTP.
+  // Hide requests made by the browser on behalf of the 1P WebUI NTP.
   is_sensitive_request |=
       (is_browser_request &&
        request.initiator ==
-           url::Origin::Create(GURL(chrome::kChromeSearchLocalNtpUrl)));
+           url::Origin::Create(GURL(chrome::kChromeUINewTabPageURL)));
 
   // Hide requests made by the NTP Instant renderer.
   auto* instant_service =
-      InstantServiceFactory::GetForProfile(static_cast<Profile*>(context));
+      context
+          ? InstantServiceFactory::GetForProfile(static_cast<Profile*>(context))
+          : nullptr;
   if (instant_service) {
     is_sensitive_request |=
         instant_service->IsInstantProcess(request.render_process_id);
@@ -175,17 +180,19 @@ void ChromeExtensionsAPIClient::NotifyWebRequestWithheld(
 
   // Track down the ExtensionActionRunner and the extension. Since this is
   // asynchronous, we could hit a null anywhere along the path.
-  content::RenderFrameHost* rfh =
+  content::RenderFrameHost* render_frame_host =
       content::RenderFrameHost::FromID(render_process_id, render_frame_id);
-  if (!rfh)
+  if (!render_frame_host) {
     return;
-  // We don't count subframe blocked actions as yet, since there's no way to
-  // surface this to the user. Ignore these (which is also what we do for
-  // content scripts).
-  if (rfh->GetParent())
+  }
+  // We don't count subframes and prerendering blocked actions as yet, since
+  // there's no way to surface this to the user. Ignore these (which is also
+  // what we do for content scripts).
+  if (!render_frame_host->IsInPrimaryMainFrame()) {
     return;
+  }
   content::WebContents* web_contents =
-      content::WebContents::FromRenderFrameHost(rfh);
+      content::WebContents::FromRenderFrameHost(render_frame_host);
   if (!web_contents)
     return;
   extensions::ExtensionActionRunner* runner =
@@ -211,7 +218,7 @@ void ChromeExtensionsAPIClient::NotifyWebRequestWithheld(
   if (!extension->permissions_data()
            ->withheld_permissions()
            .explicit_hosts()
-           .MatchesURL(rfh->GetLastCommittedURL())) {
+           .MatchesURL(render_frame_host->GetLastCommittedURL())) {
     return;
   }
 
@@ -245,8 +252,8 @@ void ChromeExtensionsAPIClient::UpdateActionCount(
   if (ExtensionTabUtil::GetTabById(
           tab_id, context, true /* include_incognito */, &tab_contents) &&
       tab_contents) {
-    ExtensionActionAPI::Get(context)->NotifyChange(action, tab_contents,
-                                                   context);
+    ExtensionActionDispatcher::Get(context)->NotifyChange(action, tab_contents,
+                                                          context);
   }
 }
 
@@ -264,11 +271,25 @@ void ChromeExtensionsAPIClient::ClearActionCount(
           context, true /* include_incognito */);
 
   for (auto* active_contents : contents_to_notify) {
-    ExtensionActionAPI::Get(context)->NotifyChange(action, active_contents,
-                                                   context);
+    ExtensionActionDispatcher::Get(context)->NotifyChange(
+        action, active_contents, context);
   }
 }
 
+void ChromeExtensionsAPIClient::OpenFileUrl(
+    const GURL& file_url,
+    content::BrowserContext* browser_context) {
+  CHECK(file_url.is_valid());
+  CHECK(file_url.SchemeIsFile());
+  Profile* profile = Profile::FromBrowserContext(browser_context);
+  NavigateParams navigate_params(profile, file_url,
+                                 ui::PAGE_TRANSITION_FROM_API);
+  navigate_params.disposition = WindowOpenDisposition::CURRENT_TAB;
+  navigate_params.browser =
+      chrome::FindTabbedBrowser(profile, /*match_original_profiles=*/false);
+  Navigate(&navigate_params);
+}
+
 AppViewGuestDelegate* ChromeExtensionsAPIClient::CreateAppViewGuestDelegate()
     const {
   return new ChromeAppViewGuestDelegate();
@@ -281,9 +302,8 @@ ChromeExtensionsAPIClient::CreateExtensionOptionsGuestDelegate(
 }
 
 std::unique_ptr<guest_view::GuestViewManagerDelegate>
-ChromeExtensionsAPIClient::CreateGuestViewManagerDelegate(
-    content::BrowserContext* context) const {
-  return std::make_unique<ChromeGuestViewManagerDelegate>(context);
+ChromeExtensionsAPIClient::CreateGuestViewManagerDelegate() const {
+  return std::make_unique<ChromeGuestViewManagerDelegate>();
 }
 
 std::unique_ptr<MimeHandlerViewGuestDelegate>
@@ -303,14 +323,26 @@ ChromeExtensionsAPIClient::CreateWebViewPermissionHelperDelegate(
   return new ChromeWebViewPermissionHelperDelegate(web_view_permission_helper);
 }
 
+#if BUILDFLAG(IS_CHROMEOS)
+std::unique_ptr<ConsentProvider>
+ChromeExtensionsAPIClient::CreateConsentProvider(
+    content::BrowserContext* browser_context) const {
+  auto consent_provider_delegate =
+      std::make_unique<file_system_api::ConsentProviderDelegate>(
+          Profile::FromBrowserContext(browser_context));
+  return std::make_unique<file_system_api::ConsentProviderImpl>(
+      std::move(consent_provider_delegate));
+}
+#endif  // BUILDFLAG(IS_CHROMEOS)
+
 scoped_refptr<ContentRulesRegistry>
 ChromeExtensionsAPIClient::CreateContentRulesRegistry(
     content::BrowserContext* browser_context,
     RulesCacheDelegate* cache_delegate) const {
   return base::MakeRefCounted<ChromeContentRulesRegistry>(
       browser_context, cache_delegate,
-      base::Bind(&CreateDefaultContentPredicateEvaluators,
-                 base::Unretained(browser_context)));
+      base::BindOnce(&CreateDefaultContentPredicateEvaluators,
+                     base::Unretained(browser_context)));
 }
 
 std::unique_ptr<DevicePermissionsPrompt>
@@ -319,10 +351,30 @@ ChromeExtensionsAPIClient::CreateDevicePermissionsPrompt(
   return std::make_unique<ChromeDevicePermissionsPrompt>(web_contents);
 }
 
+#if BUILDFLAG(IS_CHROMEOS)
+bool ChromeExtensionsAPIClient::ShouldAllowDetachingUsb(int vid,
+                                                        int pid) const {
+  const base::Value::List* policy_list;
+  if (ash::CrosSettings::Get()->GetList(ash::kUsbDetachableAllowlist,
+                                        &policy_list)) {
+    for (const auto& entry : *policy_list) {
+      const base::Value::Dict* entry_dict = entry.GetIfDict();
+      if (entry_dict &&
+          entry_dict->FindInt(ash::kUsbDetachableAllowlistKeyVid) == vid &&
+          entry_dict->FindInt(ash::kUsbDetachableAllowlistKeyPid) == pid) {
+        return true;
+      }
+    }
+  }
+
+  return false;
+}
+#endif  // BUILDFLAG(IS_CHROMEOS)
+
 std::unique_ptr<VirtualKeyboardDelegate>
 ChromeExtensionsAPIClient::CreateVirtualKeyboardDelegate(
     content::BrowserContext* browser_context) const {
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS)
   return std::make_unique<ChromeVirtualKeyboardDelegate>(browser_context);
 #else
   return nullptr;
@@ -334,6 +386,13 @@ ManagementAPIDelegate* ChromeExtensionsAPIClient::CreateManagementAPIDelegate()
   return new ChromeManagementAPIDelegate;
 }
 
+std::unique_ptr<SupervisedUserExtensionsDelegate>
+ChromeExtensionsAPIClient::CreateSupervisedUserExtensionsDelegate(
+    content::BrowserContext* browser_context) const {
+  return std::make_unique<SupervisedUserExtensionsDelegateImpl>(
+      browser_context);
+}
+
 std::unique_ptr<DisplayInfoProvider>
 ChromeExtensionsAPIClient::CreateDisplayInfoProvider() const {
   return CreateChromeDisplayInfoProvider();
@@ -341,23 +400,19 @@ ChromeExtensionsAPIClient::CreateDisplayInfoProvider() const {
 
 MetricsPrivateDelegate* ChromeExtensionsAPIClient::GetMetricsPrivateDelegate() {
   if (!metrics_private_delegate_)
-    metrics_private_delegate_.reset(new ChromeMetricsPrivateDelegate());
+    metrics_private_delegate_ =
+        std::make_unique<ChromeMetricsPrivateDelegate>();
   return metrics_private_delegate_.get();
 }
 
-NetworkingCastPrivateDelegate*
-ChromeExtensionsAPIClient::GetNetworkingCastPrivateDelegate() {
-#if defined(OS_CHROMEOS) || defined(OS_WIN) || defined(OS_MACOSX)
-  if (!networking_cast_private_delegate_)
-    networking_cast_private_delegate_ =
-        ChromeNetworkingCastPrivateDelegate::Create();
-#endif
-  return networking_cast_private_delegate_.get();
-}
-
 FileSystemDelegate* ChromeExtensionsAPIClient::GetFileSystemDelegate() {
+#if BUILDFLAG(IS_CHROMEOS)
+  using ChromeFileSystemDelegate_Use = ChromeFileSystemDelegateAsh;
+#else
+  using ChromeFileSystemDelegate_Use = ChromeFileSystemDelegate;
+#endif
   if (!file_system_delegate_)
-    file_system_delegate_ = std::make_unique<ChromeFileSystemDelegate>();
+    file_system_delegate_ = std::make_unique<ChromeFileSystemDelegate_Use>();
   return file_system_delegate_.get();
 }
 
@@ -376,7 +431,7 @@ ChromeExtensionsAPIClient::GetFeedbackPrivateDelegate() {
   return feedback_private_delegate_.get();
 }
 
-#if defined(OS_CHROMEOS)
+#if BUILDFLAG(IS_CHROMEOS)
 MediaPerceptionAPIDelegate*
 ChromeExtensionsAPIClient::GetMediaPerceptionAPIDelegate() {
   if (!media_perception_api_delegate_) {
@@ -396,18 +451,18 @@ ChromeExtensionsAPIClient::GetNonNativeFileSystemDelegate() {
 }
 
 void ChromeExtensionsAPIClient::SaveImageDataToClipboard(
-    const std::vector<char>& image_data,
+    std::vector<uint8_t> image_data,
     api::clipboard::ImageType type,
     AdditionalDataItemList additional_items,
-    const base::Closure& success_callback,
-    const base::Callback<void(const std::string&)>& error_callback) {
+    base::OnceClosure success_callback,
+    base::OnceCallback<void(const std::string&)> error_callback) {
   if (!clipboard_extension_helper_)
     clipboard_extension_helper_ = std::make_unique<ClipboardExtensionHelper>();
   clipboard_extension_helper_->DecodeAndSaveImageData(
-      image_data, type, std::move(additional_items), success_callback,
-      error_callback);
+      std::move(image_data), type, std::move(additional_items),
+      std::move(success_callback), std::move(error_callback));
 }
-#endif
+#endif  // BUILDFLAG(IS_CHROMEOS)
 
 AutomationInternalApiDelegate*
 ChromeExtensionsAPIClient::GetAutomationInternalApiDelegate() {
@@ -420,7 +475,12 @@ ChromeExtensionsAPIClient::GetAutomationInternalApiDelegate() {
 
 std::vector<KeyedServiceBaseFactory*>
 ChromeExtensionsAPIClient::GetFactoryDependencies() {
-  return {InstantServiceFactory::GetInstance()};
+  // clang-format off
+  return {
+      InstantServiceFactory::GetInstance(),
+      SupervisedUserServiceFactory::GetInstance(),
+  };
+  // clang-format on
 }
 
 }  // namespace extensions
