@@ -1,23 +1,26 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "extensions/browser/api/device_permissions_manager.h"
+
 #include <stdint.h>
 
 #include <memory>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
 #include "base/run_loop.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/values_test_util.h"
 #include "chrome/browser/extensions/test_extension_environment.h"
 #include "chrome/test/base/testing_profile.h"
-#include "extensions/browser/api/device_permissions_manager.h"
 #include "extensions/browser/api/hid/hid_device_manager.h"
 #include "extensions/browser/api/usb/usb_device_manager.h"
 #include "extensions/browser/extension_prefs.h"
 #include "extensions/common/extension.h"
-#include "services/device/public/cpp/hid/fake_hid_manager.h"
+#include "mojo/public/cpp/bindings/pending_remote.h"
+#include "services/device/public/cpp/test/fake_hid_manager.h"
 #include "services/device/public/cpp/test/fake_usb_device_manager.h"
 #include "services/device/public/mojom/hid.mojom.h"
 #include "services/device/public/mojom/usb_device.mojom.h"
@@ -41,22 +44,22 @@ class DevicePermissionsManagerTest : public testing::Test {
  protected:
   void SetUp() override {
     testing::Test::SetUp();
-    env_.reset(new extensions::TestExtensionEnvironment());
-    extension_ = env_->MakeExtension(*base::test::ParseJsonDeprecated(
-        "{"
-        "  \"app\": {"
-        "    \"background\": {"
-        "      \"scripts\": [\"background.js\"]"
-        "    }"
-        "  },"
-        "  \"permissions\": [ \"hid\", \"usb\" ]"
-        "}"));
+    env_ = std::make_unique<extensions::TestExtensionEnvironment>();
+    extension_ = env_->MakeExtension(
+        base::test::ParseJsonDict("{"
+                                  "  \"app\": {"
+                                  "    \"background\": {"
+                                  "      \"scripts\": [\"background.js\"]"
+                                  "    }"
+                                  "  },"
+                                  "  \"permissions\": [ \"hid\", \"usb\" ]"
+                                  "}"));
 
     // Set fake device manager for extensions::UsbDeviceManager.
-    device::mojom::UsbDeviceManagerPtr usb_manager_ptr;
-    fake_usb_manager_.AddBinding(mojo::MakeRequest(&usb_manager_ptr));
+    mojo::PendingRemote<device::mojom::UsbDeviceManager> usb_manager;
+    fake_usb_manager_.AddReceiver(usb_manager.InitWithNewPipeAndPassReceiver());
     UsbDeviceManager::Get(env_->profile())
-        ->SetDeviceManagerForTesting(std::move(usb_manager_ptr));
+        ->SetDeviceManagerForTesting(std::move(usb_manager));
     base::RunLoop().RunUntilIdle();
 
     device0_ = fake_usb_manager_.CreateAndAddDevice(0, 0, "Test Manufacturer",
@@ -68,26 +71,28 @@ class DevicePermissionsManagerTest : public testing::Test {
     device3_ = fake_usb_manager_.CreateAndAddDevice(0, 0, "Test Manufacturer",
                                                     "Test Product", "");
 
-    device::mojom::HidManagerPtr hid_manager_ptr;
-    fake_hid_manager_.Bind(mojo::MakeRequest(&hid_manager_ptr));
-    HidDeviceManager::Get(env_->profile())
-        ->SetFakeHidManagerForTesting(std::move(hid_manager_ptr));
+    HidDeviceManager::OverrideHidManagerBinderForTesting(base::BindRepeating(
+        &device::FakeHidManager::Bind, base::Unretained(&fake_hid_manager_)));
+    HidDeviceManager::Get(env_->profile())->LazyInitialize();
     base::RunLoop().RunUntilIdle();
 
     device4_ = fake_hid_manager_.CreateAndAddDevice(
-        0, 0, "Test HID Device", "abcde", HidBusType::kHIDBusTypeUSB);
-    device5_ = fake_hid_manager_.CreateAndAddDevice(0, 0, "Test HID Device", "",
-                                                    HidBusType::kHIDBusTypeUSB);
+        "4", 0, 0, "Test HID Device", "abcde", HidBusType::kHIDBusTypeUSB);
+    device5_ = fake_hid_manager_.CreateAndAddDevice(
+        "5", 0, 0, "Test HID Device", "", HidBusType::kHIDBusTypeUSB);
     device6_ = fake_hid_manager_.CreateAndAddDevice(
-        0, 0, "Test HID Device", "67890", HidBusType::kHIDBusTypeUSB);
-    device7_ = fake_hid_manager_.CreateAndAddDevice(0, 0, "Test HID Device", "",
-                                                    HidBusType::kHIDBusTypeUSB);
+        "6", 0, 0, "Test HID Device", "67890", HidBusType::kHIDBusTypeUSB);
+    device7_ = fake_hid_manager_.CreateAndAddDevice(
+        "7", 0, 0, "Test HID Device", "", HidBusType::kHIDBusTypeUSB);
   }
 
-  void TearDown() override { env_.reset(nullptr); }
+  void TearDown() override {
+    extension_ = nullptr;
+    env_.reset(nullptr);
+  }
 
   std::unique_ptr<extensions::TestExtensionEnvironment> env_;
-  const extensions::Extension* extension_;
+  raw_ptr<const extensions::Extension> extension_ = nullptr;
   device::FakeUsbDeviceManager fake_usb_manager_;
   device::mojom::UsbDeviceInfoPtr device0_;
   device::mojom::UsbDeviceInfoPtr device1_;
@@ -129,15 +134,13 @@ TEST_F(DevicePermissionsManagerTest, AllowAndClearDevices) {
   EXPECT_FALSE(device_permissions->FindHidDeviceEntry(*device7_));
   EXPECT_EQ(4U, device_permissions->entries().size());
 
-  EXPECT_EQ(base::ASCIIToUTF16(
-                "Test Product from Test Manufacturer (serial number ABCDE)"),
+  EXPECT_EQ(u"Test Product from Test Manufacturer (serial number ABCDE)",
             device0_entry->GetPermissionMessageString());
-  EXPECT_EQ(base::ASCIIToUTF16("Test Product from Test Manufacturer"),
+  EXPECT_EQ(u"Test Product from Test Manufacturer",
             device1_entry->GetPermissionMessageString());
-  EXPECT_EQ(base::ASCIIToUTF16("Test HID Device (serial number abcde)"),
+  EXPECT_EQ(u"Test HID Device (serial number abcde)",
             device4_entry->GetPermissionMessageString());
-  EXPECT_EQ(base::ASCIIToUTF16("Test HID Device"),
-            device5_entry->GetPermissionMessageString());
+  EXPECT_EQ(u"Test HID Device", device5_entry->GetPermissionMessageString());
 
   manager->Clear(extension_->id());
   // The device_permissions object is deleted by Clear.
@@ -291,7 +294,7 @@ TEST_F(DevicePermissionsManagerTest, UpdateLastUsed) {
 }
 
 TEST_F(DevicePermissionsManagerTest, LoadPrefs) {
-  std::unique_ptr<base::Value> prefs_value = base::test::ParseJsonDeprecated(
+  base::Value prefs_value = base::test::ParseJson(
       "["
       "  {"
       "    \"manufacturer_string\": \"Test Manufacturer\","
@@ -329,75 +332,70 @@ TEST_F(DevicePermissionsManagerTest, LoadPrefs) {
   EXPECT_FALSE(device_permissions->FindHidDeviceEntry(*device6_));
   EXPECT_FALSE(device_permissions->FindHidDeviceEntry(*device7_));
 
-  EXPECT_EQ(base::ASCIIToUTF16(
-                "Test Product from Test Manufacturer (serial number ABCDE)"),
+  EXPECT_EQ(u"Test Product from Test Manufacturer (serial number ABCDE)",
             device0_entry->GetPermissionMessageString());
-  EXPECT_EQ(base::ASCIIToUTF16("Test HID Device (serial number abcde)"),
+  EXPECT_EQ(u"Test HID Device (serial number abcde)",
             device4_entry->GetPermissionMessageString());
 }
 
 TEST_F(DevicePermissionsManagerTest, PermissionMessages) {
-  base::string16 empty;
-  base::string16 product(base::ASCIIToUTF16("Widget"));
-  base::string16 manufacturer(base::ASCIIToUTF16("ACME"));
-  base::string16 serial_number(base::ASCIIToUTF16("A"));
+  std::u16string empty;
+  std::u16string product(u"Widget");
+  std::u16string manufacturer(u"ACME");
+  std::u16string serial_number(u"A");
 
-  EXPECT_EQ(base::ASCIIToUTF16("Unknown product 0001 from vendor 0000"),
+  EXPECT_EQ(u"Unknown product 0001 from vendor 0000",
             DevicePermissionsManager::GetPermissionMessage(
                 0x0000, 0x0001, empty, empty, empty, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16(
-                "Unknown product 0001 from vendor 0000 (serial number A)"),
+  EXPECT_EQ(u"Unknown product 0001 from vendor 0000 (serial number A)",
             DevicePermissionsManager::GetPermissionMessage(
-                0x0000, 0x0001, empty, empty, base::ASCIIToUTF16("A"), false));
+                0x0000, 0x0001, empty, empty, u"A", false));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Unknown product 0001 from Google Inc."),
+  EXPECT_EQ(u"Unknown product 0001 from Google Inc.",
             DevicePermissionsManager::GetPermissionMessage(
                 0x18D1, 0x0001, empty, empty, empty, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16(
-                "Unknown product 0001 from Google Inc. (serial number A)"),
+  EXPECT_EQ(u"Unknown product 0001 from Google Inc. (serial number A)",
             DevicePermissionsManager::GetPermissionMessage(
                 0x18D1, 0x0001, empty, empty, serial_number, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Nexus One from Google Inc."),
+  EXPECT_EQ(u"Nexus One from Google Inc.",
             DevicePermissionsManager::GetPermissionMessage(
                 0x18D1, 0x4E11, empty, empty, empty, true));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Nexus One from Google Inc. (serial number A)"),
+  EXPECT_EQ(u"Nexus One from Google Inc. (serial number A)",
             DevicePermissionsManager::GetPermissionMessage(
                 0x18D1, 0x4E11, empty, empty, serial_number, true));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Nexus One"),
-            DevicePermissionsManager::GetPermissionMessage(
-                0x18D1, 0x4E11, empty, empty, empty, false));
+  EXPECT_EQ(u"Nexus One", DevicePermissionsManager::GetPermissionMessage(
+                              0x18D1, 0x4E11, empty, empty, empty, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Nexus One (serial number A)"),
+  EXPECT_EQ(u"Nexus One (serial number A)",
             DevicePermissionsManager::GetPermissionMessage(
                 0x18D1, 0x4E11, empty, empty, serial_number, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Unknown product 0001 from ACME"),
+  EXPECT_EQ(u"Unknown product 0001 from ACME",
             DevicePermissionsManager::GetPermissionMessage(
                 0x0000, 0x0001, manufacturer, empty, empty, false));
 
-  EXPECT_EQ(
-      base::ASCIIToUTF16("Unknown product 0001 from ACME (serial number A)"),
-      DevicePermissionsManager::GetPermissionMessage(
-          0x0000, 0x0001, manufacturer, empty, serial_number, false));
+  EXPECT_EQ(u"Unknown product 0001 from ACME (serial number A)",
+            DevicePermissionsManager::GetPermissionMessage(
+                0x0000, 0x0001, manufacturer, empty, serial_number, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Widget from ACME"),
+  EXPECT_EQ(u"Widget from ACME",
             DevicePermissionsManager::GetPermissionMessage(
                 0x0001, 0x0000, manufacturer, product, empty, true));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Widget from ACME (serial number A)"),
+  EXPECT_EQ(u"Widget from ACME (serial number A)",
             DevicePermissionsManager::GetPermissionMessage(
                 0x0001, 0x0000, manufacturer, product, serial_number, true));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Widget"),
+  EXPECT_EQ(u"Widget",
             DevicePermissionsManager::GetPermissionMessage(
                 0x0001, 0x0000, manufacturer, product, empty, false));
 
-  EXPECT_EQ(base::ASCIIToUTF16("Widget (serial number A)"),
+  EXPECT_EQ(u"Widget (serial number A)",
             DevicePermissionsManager::GetPermissionMessage(
                 0x0001, 0x0000, manufacturer, product, serial_number, false));
 }
