@@ -1,26 +1,24 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/extensions/api/image_writer_private/image_writer_utility_client.h"
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
-#include "base/macros.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/run_loop.h"
-#include "base/sequenced_task_runner.h"
-#include "base/task/post_task.h"
+#include "base/task/sequenced_task_runner.h"
+#include "base/task/thread_pool.h"
 #include "build/build_config.h"
 #include "chrome/browser/extensions/api/image_writer_private/operation.h"
 #include "chrome/services/removable_storage_writer/public/mojom/removable_storage_writer.mojom.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
-#include "content/public/browser/system_connector.h"
-#include "services/service_manager/public/cpp/connector.h"
+#include "content/public/test/browser_test.h"
 
 namespace extensions {
 namespace image_writer {
@@ -38,12 +36,16 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
     EXPECT_TRUE(temp_dir_.CreateUniqueTempDir());
   }
 
+  ImageWriterUtilityClientTest(const ImageWriterUtilityClientTest&) = delete;
+  ImageWriterUtilityClientTest& operator=(const ImageWriterUtilityClientTest&) =
+      delete;
+
   void FillImageFileWithPattern(char pattern) {
     base::ScopedAllowBlockingForTesting allow_blocking;
     EXPECT_TRUE(base::CreateTemporaryFileInDir(temp_dir_.GetPath(), &image_));
 
     base::RunLoop run_loop;
-    base::PostTaskWithTraitsAndReply(
+    base::ThreadPool::PostTaskAndReply(
         FROM_HERE, {base::MayBlock()},
         base::BindOnce(&ImageWriterUtilityClientTest::FillFile, image_,
                        pattern),
@@ -56,7 +58,7 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
     device_ = image_.ReplaceExtension(FILE_PATH_LITERAL("out"));
 
     base::RunLoop run_loop;
-    base::PostTaskWithTraitsAndReply(
+    base::ThreadPool::PostTaskAndReply(
         FROM_HERE, {base::MayBlock()},
         base::BindOnce(&ImageWriterUtilityClientTest::FillFile, device_,
                        pattern),
@@ -102,26 +104,23 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
   const std::string& error() const { return error_; }
 
  private:
-  void SetUpOnMainThread() override {
-    connector_ = content::GetSystemConnector()->Clone();
-  }
-
   void StartWriteTest() {
     DCHECK(IsRunningInCorrectSequence());
 
-    if (!image_writer_utility_client_)
+    if (!image_writer_utility_client_) {
       image_writer_utility_client_ =
-          new ImageWriterUtilityClient(GetTaskRunner(), std::move(connector_));
+          new ImageWriterUtilityClient(GetTaskRunner());
+    }
     success_ = false;
     progress_ = 0;
 
     image_writer_utility_client_->Write(
-        base::Bind(&ImageWriterUtilityClientTest::Progress,
-                   base::Unretained(this)),
-        base::Bind(&ImageWriterUtilityClientTest::Success,
-                   base::Unretained(this)),
-        base::Bind(&ImageWriterUtilityClientTest::Failure,
-                   base::Unretained(this)),
+        base::BindRepeating(&ImageWriterUtilityClientTest::Progress,
+                            base::Unretained(this)),
+        base::BindOnce(&ImageWriterUtilityClientTest::Success,
+                       base::Unretained(this)),
+        base::BindOnce(&ImageWriterUtilityClientTest::Failure,
+                       base::Unretained(this)),
         image_, test_device_);
   }
 
@@ -129,10 +128,11 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
     DCHECK(IsRunningInCorrectSequence());
 
     progress_ = progress;
-    if (!cancel_)
+    if (!cancel_) {
       return;
+    }
 
-    image_writer_utility_client_->Cancel(base::Bind(
+    image_writer_utility_client_->Cancel(base::BindOnce(
         &ImageWriterUtilityClientTest::Cancelled, base::Unretained(this)));
   }
 
@@ -156,19 +156,20 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
   void StartVerifyTest() {
     DCHECK(IsRunningInCorrectSequence());
 
-    if (!image_writer_utility_client_)
+    if (!image_writer_utility_client_) {
       image_writer_utility_client_ =
-          new ImageWriterUtilityClient(GetTaskRunner(), std::move(connector_));
+          new ImageWriterUtilityClient(GetTaskRunner());
+    }
     success_ = false;
     progress_ = 0;
 
     image_writer_utility_client_->Verify(
-        base::Bind(&ImageWriterUtilityClientTest::Progress,
-                   base::Unretained(this)),
-        base::Bind(&ImageWriterUtilityClientTest::Verified,
-                   base::Unretained(this)),
-        base::Bind(&ImageWriterUtilityClientTest::Failure,
-                   base::Unretained(this)),
+        base::BindRepeating(&ImageWriterUtilityClientTest::Progress,
+                            base::Unretained(this)),
+        base::BindOnce(&ImageWriterUtilityClientTest::Verified,
+                       base::Unretained(this)),
+        base::BindOnce(&ImageWriterUtilityClientTest::Failure,
+                       base::Unretained(this)),
         image_, test_device_);
   }
 
@@ -203,8 +204,7 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
     success_ = cancel_;
 
     quit_called_ = true;
-    base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
-                             quit_closure_);
+    content::GetUIThreadTaskRunner({})->PostTask(FROM_HERE, quit_closure_);
   }
 
   void Shutdown() {
@@ -213,18 +213,17 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
     image_writer_utility_client_->Shutdown();
 
     quit_called_ = true;
-    base::PostTaskWithTraits(FROM_HERE, {content::BrowserThread::UI},
-                             quit_closure_);
+    content::GetUIThreadTaskRunner({})->PostTask(FROM_HERE, quit_closure_);
   }
 
   static void FillFile(const base::FilePath& path, char pattern) {
-    const std::vector<char> fill(kTestFileSize, pattern);
-    EXPECT_TRUE(base::WriteFile(path, fill.data(), kTestFileSize));
+    const std::string fill(kTestFileSize, pattern);
+    EXPECT_TRUE(base::WriteFile(path, fill));
   }
 
   base::SequencedTaskRunner* CreateTaskRunner() {
     DCHECK(!task_runner_.get());
-    task_runner_ = base::CreateSequencedTaskRunnerWithTraits(
+    task_runner_ = base::ThreadPool::CreateSequencedTaskRunner(
         Operation::blocking_task_traits());
     return task_runner_.get();
   }
@@ -244,7 +243,7 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
   base::FilePath device_;
   base::FilePath image_;
 
-  base::Closure quit_closure_;
+  base::RepeatingClosure quit_closure_;
   bool quit_called_ = false;
 
   // Lives on |task_runner_|.
@@ -255,9 +254,6 @@ class ImageWriterUtilityClientTest : public InProcessBrowserTest {
   bool cancel_ = false;
   std::string error_;
   scoped_refptr<base::SequencedTaskRunner> task_runner_;
-  std::unique_ptr<service_manager::Connector> connector_;
-
-  DISALLOW_COPY_AND_ASSIGN(ImageWriterUtilityClientTest);
 };
 
 IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, WriteNoImage) {
@@ -276,13 +272,7 @@ IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, WriteNoDevice) {
   EXPECT_FALSE(error().empty());
 }
 
-// Flaky on Win. http://crbug.com/927218
-#if defined(OS_WIN)
-#define MAYBE_Write DISABLED_Write
-#else
-#define MAYBE_Write Write
-#endif
-IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_Write) {
+IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, Write) {
   FillImageFileWithPattern('i');
   FillDeviceFileWithPattern(0);
 
@@ -292,13 +282,7 @@ IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_Write) {
   EXPECT_TRUE(error().empty());
 }
 
-// Flaky on Win. http://crbug.com/927218
-#if defined(OS_WIN)
-#define MAYBE_WriteVerify DISABLED_WriteVerify
-#else
-#define MAYBE_WriteVerify WriteVerify
-#endif
-IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_WriteVerify) {
+IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, WriteVerify) {
   FillImageFileWithPattern('m');
   FillDeviceFileWithPattern(0);
 
@@ -308,13 +292,7 @@ IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_WriteVerify) {
   EXPECT_TRUE(error().empty());
 }
 
-// Flaky on Win. http://crbug.com/927218
-#if defined(OS_WIN)
-#define MAYBE_WriteCancel DISABLED_WriteCancel
-#else
-#define MAYBE_WriteCancel WriteCancel
-#endif
-IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_WriteCancel) {
+IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, WriteCancel) {
   FillImageFileWithPattern('a');
   FillDeviceFileWithPattern(0);
 
@@ -350,13 +328,7 @@ IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, VerifyFailure) {
   EXPECT_FALSE(error().empty());
 }
 
-// Flaky on Win. http://crbug.com/927218
-#if defined(OS_WIN)
-#define MAYBE_Verify DISABLED_Verify
-#else
-#define MAYBE_Verify Verify
-#endif
-IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_Verify) {
+IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, Verify) {
   FillImageFileWithPattern('e');
   FillDeviceFileWithPattern('e');
 
@@ -366,13 +338,7 @@ IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_Verify) {
   EXPECT_TRUE(error().empty());
 }
 
-// Flaky on Win. http://crbug.com/927218
-#if defined(OS_WIN)
-#define MAYBE_VerifyCancel DISABLED_VerifyCancel
-#else
-#define MAYBE_VerifyCancel VerifyCancel
-#endif
-IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, MAYBE_VerifyCancel) {
+IN_PROC_BROWSER_TEST_F(ImageWriterUtilityClientTest, VerifyCancel) {
   FillImageFileWithPattern('s');
   FillDeviceFileWithPattern('s');
 
