@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,25 +7,31 @@
 #include <stdint.h>
 
 #include <algorithm>
+#include <memory>
+#include <set>
+#include <utility>
 
+#include "base/functional/bind.h"
 #include "base/lazy_instance.h"
-#include "base/metrics/histogram_macros.h"
+#include "base/metrics/histogram.h"
 #include "base/process/process.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/utf_string_conversions.h"
-#include "chrome/browser/extensions/api/tabs/tabs_constants.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/task_manager/task_manager_interface.h"
 #include "chrome/common/extensions/api/processes.h"
 #include "content/public/browser/browser_child_process_host.h"
+#include "content/public/browser/browser_task_traits.h"
+#include "content/public/browser/browser_thread.h"
 #include "content/public/browser/child_process_data.h"
+#include "content/public/browser/child_process_host.h"
+#include "content/public/browser/render_frame_host.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/web_contents.h"
-#include "content/public/common/child_process_host.h"
 #include "content/public/common/result_codes.h"
 #include "extensions/common/error_utils.h"
-#include "third_party/WebKit/public/platform/WebCache.h"
+#include "third_party/blink/public/mojom/devtools/console_message.mojom.h"
 
 namespace extensions {
 
@@ -56,11 +62,11 @@ int64_t GetRefreshTypesForProcessOptionalData() {
       task_manager::REFRESH_TYPE_WEBCACHE_STATS;
 }
 
-std::unique_ptr<api::processes::Cache> CreateCacheData(
-    const blink::WebCache::ResourceTypeStat& stat) {
-  std::unique_ptr<api::processes::Cache> cache(new api::processes::Cache());
-  cache->size = static_cast<double>(stat.size);
-  cache->live_size = static_cast<double>(stat.size);
+api::processes::Cache CreateCacheData(
+    const blink::WebCacheResourceTypeStat& stat) {
+  api::processes::Cache cache;
+  cache.size = static_cast<double>(stat.size);
+  cache.live_size = static_cast<double>(stat.size);
   return cache;
 }
 
@@ -68,39 +74,46 @@ api::processes::ProcessType GetProcessType(
     task_manager::Task::Type task_type) {
   switch (task_type) {
     case task_manager::Task::BROWSER:
-      return api::processes::PROCESS_TYPE_BROWSER;
+      return api::processes::ProcessType::kBrowser;
 
     case task_manager::Task::RENDERER:
-      return api::processes::PROCESS_TYPE_RENDERER;
+      return api::processes::ProcessType::kRenderer;
 
     case task_manager::Task::EXTENSION:
     case task_manager::Task::GUEST:
-      return api::processes::PROCESS_TYPE_EXTENSION;
+      return api::processes::ProcessType::kExtension;
 
     case task_manager::Task::PLUGIN:
-      return api::processes::PROCESS_TYPE_PLUGIN;
-
-    case task_manager::Task::WORKER:
-      return api::processes::PROCESS_TYPE_WORKER;
+      return api::processes::ProcessType::kPlugin;
 
     case task_manager::Task::NACL:
-      return api::processes::PROCESS_TYPE_NACL;
+      return api::processes::ProcessType::kNacl;
+
+    // TODO(crbug.com/40117341): Assign a different process type for each
+    //                                  worker type.
+    case task_manager::Task::DEDICATED_WORKER:
+    case task_manager::Task::SHARED_WORKER:
+      return api::processes::ProcessType::kWorker;
+
+    case task_manager::Task::SERVICE_WORKER:
+      return api::processes::ProcessType::kServiceWorker;
 
     case task_manager::Task::UTILITY:
-      return api::processes::PROCESS_TYPE_UTILITY;
+      return api::processes::ProcessType::kUtility;
 
     case task_manager::Task::GPU:
-      return api::processes::PROCESS_TYPE_GPU;
+      return api::processes::ProcessType::kGpu;
 
     case task_manager::Task::UNKNOWN:
     case task_manager::Task::ARC:
+    case task_manager::Task::CROSTINI:
+    case task_manager::Task::PLUGIN_VM:
     case task_manager::Task::SANDBOX_HELPER:
     case task_manager::Task::ZYGOTE:
-      return api::processes::PROCESS_TYPE_OTHER;
+      return api::processes::ProcessType::kOther;
   }
 
   NOTREACHED() << "Unknown task type.";
-  return api::processes::PROCESS_TYPE_NONE;
 }
 
 // Fills |out_process| with the data of the process in which the task with |id|
@@ -126,9 +139,9 @@ void FillProcessData(
   for (const auto& task_id : tasks_on_process) {
     api::processes::TaskInfo task_info;
     task_info.title = base::UTF16ToUTF8(task_manager->GetTitle(task_id));
-    const int tab_id = task_manager->GetTabId(task_id);
-    if (tab_id != -1)
-      task_info.tab_id.reset(new int(tab_id));
+    const SessionID tab_id = task_manager->GetTabId(task_id);
+    if (tab_id.is_valid())
+      task_info.tab_id = tab_id.id();
 
     out_process->tasks.push_back(std::move(task_info));
   }
@@ -137,26 +150,26 @@ void FillProcessData(
   if (!include_optional)
     return;
 
-  out_process->cpu.reset(new double(task_manager->GetCpuUsage(id)));
+  const double cpu_usage = task_manager->GetPlatformIndependentCPUUsage(id);
+  if (!std::isnan(cpu_usage))
+    out_process->cpu = cpu_usage;
 
-  out_process->network.reset(new double(static_cast<double>(
-      task_manager->GetProcessTotalNetworkUsage(id))));
+  const int64_t network_usage = task_manager->GetProcessTotalNetworkUsage(id);
+  if (network_usage != -1)
+    out_process->network = network_usage;
 
   int64_t v8_allocated = 0;
   int64_t v8_used = 0;
   if (task_manager->GetV8Memory(id, &v8_allocated, &v8_used)) {
-    out_process->js_memory_allocated.reset(new double(static_cast<double>(
-        v8_allocated)));
-    out_process->js_memory_used.reset(new double(static_cast<double>(v8_used)));
+    out_process->js_memory_allocated = v8_allocated;
+    out_process->js_memory_used = v8_used;
   }
 
   const int64_t sqlite_bytes = task_manager->GetSqliteMemoryUsed(id);
-  if (sqlite_bytes != -1) {
-    out_process->sqlite_memory.reset(new double(static_cast<double>(
-        sqlite_bytes)));
-  }
+  if (sqlite_bytes != -1)
+    out_process->sqlite_memory = sqlite_bytes;
 
-  blink::WebCache::ResourceTypeStats cache_stats;
+  blink::WebCacheResourceTypeStats cache_stats;
   if (task_manager->GetWebCacheStats(id, &cache_stats)) {
     out_process->image_cache = CreateCacheData(cache_stats.images);
     out_process->script_cache = CreateCacheData(cache_stats.scripts);
@@ -171,14 +184,12 @@ void FillProcessData(
 ////////////////////////////////////////////////////////////////////////////////
 
 ProcessesEventRouter::ProcessesEventRouter(content::BrowserContext* context)
-    : task_manager::TaskManagerObserver(base::TimeDelta::FromSeconds(1),
+    : task_manager::TaskManagerObserver(base::Seconds(1),
                                         task_manager::REFRESH_TYPE_NONE),
       browser_context_(context),
-      listeners_(0) {
-}
+      listeners_(0) {}
 
-ProcessesEventRouter::~ProcessesEventRouter() {
-}
+ProcessesEventRouter::~ProcessesEventRouter() = default;
 
 void ProcessesEventRouter::ListenerAdded() {
   UpdateRefreshTypesFlagsBasedOnListeners();
@@ -248,7 +259,7 @@ void ProcessesEventRouter::OnTasksRefreshedWithBackgroundCalculations(
 
   // Get the data of tasks sharing the same process only once.
   std::set<base::ProcessId> seen_processes;
-  base::DictionaryValue processes_dictionary;
+  base::Value::Dict processes_dictionary;
   for (const auto& task_id : task_ids) {
     // We are not interested in tasks, but rather the processes on which they
     // run.
@@ -272,16 +283,15 @@ void ProcessesEventRouter::OnTasksRefreshedWithBackgroundCalculations(
                     &process);
 
     if (has_on_updated_with_memory_listeners) {
-      // Append the private memory usage to the process data.
-      const int64_t private_memory =
-          observed_task_manager()->GetPrivateMemoryUsage(task_id);
-      process.private_memory.reset(new double(static_cast<double>(
-          private_memory)));
+      // Append the memory footprint to the process data.
+      const int64_t memory_footprint =
+          observed_task_manager()->GetMemoryFootprintUsage(task_id);
+      process.private_memory = static_cast<double>(memory_footprint);
     }
 
     // Store each process indexed by the string version of its ChildProcessHost
     // ID.
-    processes_dictionary.Set(base::IntToString(child_process_host_id),
+    processes_dictionary.Set(base::NumberToString(child_process_host_id),
                              process.ToValue());
   }
 
@@ -290,7 +300,7 @@ void ProcessesEventRouter::OnTasksRefreshedWithBackgroundCalculations(
   DCHECK(has_on_updated_listeners || has_on_updated_with_memory_listeners);
   if (has_on_updated_listeners) {
     api::processes::OnUpdated::Processes processes;
-    processes.additional_properties.MergeDictionary(&processes_dictionary);
+    processes.additional_properties.Merge(processes_dictionary.Clone());
     // NOTE: If there are listeners to the updates with memory as well,
     // listeners to onUpdated (without memory) will also get the memory info
     // of processes as an added bonus.
@@ -301,7 +311,7 @@ void ProcessesEventRouter::OnTasksRefreshedWithBackgroundCalculations(
 
   if (has_on_updated_with_memory_listeners) {
     api::processes::OnUpdatedWithMemory::Processes processes;
-    processes.additional_properties.MergeDictionary(&processes_dictionary);
+    processes.additional_properties.Merge(std::move(processes_dictionary));
     DispatchEvent(events::PROCESSES_ON_UPDATED_WITH_MEMORY,
                   api::processes::OnUpdatedWithMemory::kEventName,
                   api::processes::OnUpdatedWithMemory::Create(processes));
@@ -322,10 +332,9 @@ void ProcessesEventRouter::OnTaskUnresponsive(task_manager::TaskId id) {
                 api::processes::OnUnresponsive::Create(process));
 }
 
-void ProcessesEventRouter::DispatchEvent(
-    events::HistogramValue histogram_value,
-    const std::string& event_name,
-    std::unique_ptr<base::ListValue> event_args) const {
+void ProcessesEventRouter::DispatchEvent(events::HistogramValue histogram_value,
+                                         const std::string& event_name,
+                                         base::Value::List event_args) const {
   EventRouter* event_router = EventRouter::Get(browser_context_);
   if (event_router) {
     std::unique_ptr<Event> event(
@@ -368,11 +377,14 @@ void ProcessesEventRouter::UpdateRefreshTypesFlagsBasedOnListeners() {
     refresh_types |= GetRefreshTypesFlagOnlyEssentialData();
   }
 
+  const int64_t on_updated_types = GetRefreshTypesForProcessOptionalData();
   if (HasEventListeners(api::processes::OnUpdated::kEventName))
-    refresh_types |= GetRefreshTypesForProcessOptionalData();
+    refresh_types |= on_updated_types;
 
-  if (HasEventListeners(api::processes::OnUpdatedWithMemory::kEventName))
-    refresh_types |= task_manager::REFRESH_TYPE_MEMORY;
+  if (HasEventListeners(api::processes::OnUpdatedWithMemory::kEventName)) {
+    refresh_types |=
+        (on_updated_types | task_manager::REFRESH_TYPE_MEMORY_FOOTPRINT);
+  }
 
   SetRefreshTypesFlags(refresh_types);
 }
@@ -430,7 +442,8 @@ void ProcessesAPI::OnListenerRemoved(const EventListenerInfo& details) {
 
 ProcessesEventRouter* ProcessesAPI::processes_event_router() {
   if (!processes_event_router_.get())
-    processes_event_router_.reset(new ProcessesEventRouter(browser_context_));
+    processes_event_router_ =
+        std::make_unique<ProcessesEventRouter>(browser_context_);
   return processes_event_router_.get();
 }
 
@@ -440,26 +453,23 @@ ProcessesEventRouter* ProcessesAPI::processes_event_router() {
 
 ExtensionFunction::ResponseAction ProcessesGetProcessIdForTabFunction::Run() {
   // For this function, the task manager doesn't even need to be running.
-  std::unique_ptr<api::processes::GetProcessIdForTab::Params> params(
-      api::processes::GetProcessIdForTab::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<api::processes::GetProcessIdForTab::Params> params =
+      api::processes::GetProcessIdForTab::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   const int tab_id = params->tab_id;
   content::WebContents* contents = nullptr;
-  int tab_index = -1;
-  if (!ExtensionTabUtil::GetTabById(
-          tab_id,
-          Profile::FromBrowserContext(browser_context()),
-          include_incognito(),
-          nullptr,
-          nullptr,
-          &contents,
-          &tab_index)) {
-    return RespondNow(Error(tabs_constants::kTabNotFoundError,
-                            base::IntToString(tab_id)));
+  if (!ExtensionTabUtil::GetTabById(tab_id, browser_context(),
+                                    include_incognito_information(),
+                                    &contents)) {
+    return RespondNow(Error(ExtensionTabUtil::kTabNotFoundError,
+                            base::NumberToString(tab_id)));
   }
 
-  const int process_id = contents->GetRenderProcessHost()->GetID();
+  // TODO(crbug.com/41345944): chrome.processes.getProcessIdForTab API
+  // incorrectly assumes a *single* renderer process per tab.
+  const int process_id =
+      contents->GetPrimaryMainFrame()->GetProcess()->GetDeprecatedID();
   return RespondNow(ArgumentList(
       api::processes::GetProcessIdForTab::Results::Create(process_id)));
 }
@@ -472,36 +482,35 @@ ExtensionFunction::ResponseAction ProcessesTerminateFunction::Run() {
   DCHECK_CURRENTLY_ON(content::BrowserThread::UI);
 
   // For this function, the task manager doesn't even need to be running.
-  std::unique_ptr<api::processes::Terminate::Params> params(
-      api::processes::Terminate::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<api::processes::Terminate::Params> params =
+      api::processes::Terminate::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
 
   child_process_host_id_ = params->process_id;
   if (child_process_host_id_ < 0) {
     return RespondNow(Error(errors::kInvalidArgument,
-                            base::IntToString(child_process_host_id_)));
+                            base::NumberToString(child_process_host_id_)));
   } else if (child_process_host_id_ == 0) {
     // Cannot kill the browser process.
     return RespondNow(Error(errors::kNotAllowedToTerminate,
-                            base::IntToString(child_process_host_id_)));
+                            base::NumberToString(child_process_host_id_)));
   }
 
   // Check if it's a renderer.
   auto* render_process_host =
       content::RenderProcessHost::FromID(child_process_host_id_);
   if (render_process_host)
-    return RespondNow(TerminateIfAllowed(render_process_host->GetHandle()));
+    return RespondNow(
+        TerminateIfAllowed(render_process_host->GetProcess().Handle()));
 
   // This could be a non-renderer child process like a plugin or a nacl
   // process. Try to get its handle from the BrowserChildProcessHost on the
   // IO thread.
-  content::BrowserThread::PostTaskAndReplyWithResult(
-      content::BrowserThread::IO,
+  content::GetIOThreadTaskRunner({})->PostTaskAndReplyWithResult(
       FROM_HERE,
-      base::Bind(&ProcessesTerminateFunction::GetProcessHandleOnIO,
-                 this,
-                 child_process_host_id_),
-      base::Bind(&ProcessesTerminateFunction::OnProcessHandleOnUI, this));
+      base::BindOnce(&ProcessesTerminateFunction::GetProcessHandleOnIO, this,
+                     child_process_host_id_),
+      base::BindOnce(&ProcessesTerminateFunction::OnProcessHandleOnUI, this));
 
   // Promise to respond later.
   return RespondLater();
@@ -513,7 +522,7 @@ base::ProcessHandle ProcessesTerminateFunction::GetProcessHandleOnIO(
 
   auto* host = content::BrowserChildProcessHost::FromID(child_process_host_id);
   if (host)
-    return host->GetData().handle;
+    return host->GetData().GetProcess().Handle();
 
   return base::kNullProcessHandle;
 }
@@ -529,25 +538,23 @@ ExtensionFunction::ResponseValue
 ProcessesTerminateFunction::TerminateIfAllowed(base::ProcessHandle handle) {
   if (handle == base::kNullProcessHandle) {
     return Error(errors::kProcessNotFound,
-                 base::IntToString(child_process_host_id_));
+                 base::NumberToString(child_process_host_id_));
   }
 
   if (handle == base::GetCurrentProcessHandle()) {
     // Cannot kill the browser process.
     return Error(errors::kNotAllowedToTerminate,
-                 base::IntToString(child_process_host_id_));
+                 base::NumberToString(child_process_host_id_));
   }
 
   base::Process process = base::Process::Open(base::GetProcId(handle));
   if (!process.IsValid()) {
     return Error(errors::kProcessNotFound,
-                 base::IntToString(child_process_host_id_));
+                 base::NumberToString(child_process_host_id_));
   }
 
   const bool did_terminate =
       process.Terminate(content::RESULT_CODE_KILLED, true /* wait */);
-  if (did_terminate)
-    UMA_HISTOGRAM_COUNTS("ChildProcess.KilledByExtensionAPI", 1);
 
   return ArgumentList(
       api::processes::Terminate::Results::Create(did_terminate));
@@ -559,14 +566,13 @@ ProcessesTerminateFunction::TerminateIfAllowed(base::ProcessHandle handle) {
 
 ProcessesGetProcessInfoFunction::ProcessesGetProcessInfoFunction()
     : task_manager::TaskManagerObserver(
-          base::TimeDelta::FromSeconds(1),
-          GetRefreshTypesFlagOnlyEssentialData()) {
-}
+          base::Seconds(1),
+          GetRefreshTypesFlagOnlyEssentialData()) {}
 
 ExtensionFunction::ResponseAction ProcessesGetProcessInfoFunction::Run() {
-  std::unique_ptr<api::processes::GetProcessInfo::Params> params(
-      api::processes::GetProcessInfo::Params::Create(*args_));
-  EXTENSION_FUNCTION_VALIDATE(params.get());
+  std::optional<api::processes::GetProcessInfo::Params> params =
+      api::processes::GetProcessInfo::Params::Create(args());
+  EXTENSION_FUNCTION_VALIDATE(params);
   if (params->process_ids.as_integer)
     process_host_ids_.push_back(*params->process_ids.as_integer);
   else
@@ -574,7 +580,7 @@ ExtensionFunction::ResponseAction ProcessesGetProcessInfoFunction::Run() {
 
   include_memory_ = params->include_memory;
   if (include_memory_)
-    AddRefreshType(task_manager::REFRESH_TYPE_MEMORY);
+    AddRefreshType(task_manager::REFRESH_TYPE_MEMORY_FOOTPRINT);
 
   // Keep this object alive until the first of either OnTasksRefreshed() or
   // OnTasksRefreshedWithBackgroundCalculations() is received depending on
@@ -607,7 +613,7 @@ ProcessesGetProcessInfoFunction::OnTasksRefreshedWithBackgroundCalculations(
   GatherDataAndRespond(task_ids);
 }
 
-ProcessesGetProcessInfoFunction::~ProcessesGetProcessInfoFunction() {}
+ProcessesGetProcessInfoFunction::~ProcessesGetProcessInfoFunction() = default;
 
 void ProcessesGetProcessInfoFunction::GatherDataAndRespond(
     const task_manager::TaskIdList& task_ids) {
@@ -634,9 +640,7 @@ void ProcessesGetProcessInfoFunction::GatherDataAndRespond(
     if (specific_processes_requested) {
       // Note: we can't use |!process_host_ids_.empty()| directly in the above
       // condition as we will erase from |process_host_ids_| below.
-      auto itr = std::find(process_host_ids_.begin(),
-                           process_host_ids_.end(),
-                           child_process_host_id);
+      auto itr = std::ranges::find(process_host_ids_, child_process_host_id);
       if (itr == process_host_ids_.end())
         continue;
 
@@ -656,25 +660,24 @@ void ProcessesGetProcessInfoFunction::GatherDataAndRespond(
                     &process);
 
     if (include_memory_) {
-      // Append the private memory usage to the process data.
-      const int64_t private_memory =
-          observed_task_manager()->GetPrivateMemoryUsage(task_id);
-      process.private_memory.reset(new double(static_cast<double>(
-          private_memory)));
+      // Append the memory footprint to the process data.
+      const int64_t memory_footprint =
+          observed_task_manager()->GetMemoryFootprintUsage(task_id);
+      process.private_memory = static_cast<double>(memory_footprint);
     }
 
     // Store each process indexed by the string version of its
     // ChildProcessHost ID.
     processes.additional_properties.Set(
-        base::IntToString(child_process_host_id),
-        process.ToValue());
+        base::NumberToString(child_process_host_id), process.ToValue());
   }
 
   // Report the invalid host ids sent in the arguments.
   for (const auto& host_id : process_host_ids_) {
-    WriteToConsole(content::CONSOLE_MESSAGE_LEVEL_ERROR,
-                   ErrorUtils::FormatErrorMessage(errors::kProcessNotFound,
-                                                  base::IntToString(host_id)));
+    WriteToConsole(
+        blink::mojom::ConsoleMessageLevel::kError,
+        ErrorUtils::FormatErrorMessage(errors::kProcessNotFound,
+                                       base::NumberToString(host_id)));
   }
 
   // Send the response.
