@@ -6,10 +6,12 @@
 
 #include <memory>
 
+#include "base/location.h"
 #include "base/test/values_test_util.h"
 #include "base/time/time.h"
 #include "chrome/browser/extensions/api/reading_list/reading_list_api_constants.h"
 #include "chrome/browser/extensions/api/reading_list/reading_list_event_router.h"
+#include "chrome/browser/extensions/api/reading_list/reading_list_event_router_factory.h"
 #include "chrome/browser/extensions/extension_service_test_base.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/reading_list/reading_list_model_factory.h"
@@ -38,7 +40,7 @@ namespace {
 // Create an extension with "readingList" permission.
 scoped_refptr<const Extension> CreateReadingListExtension() {
   return ExtensionBuilder("Extension with readingList permission")
-      .AddPermission("readingList")
+      .AddAPIPermission("readingList")
       .Build();
 }
 
@@ -82,7 +84,6 @@ class ReadingListApiUnitTest : public ExtensionServiceTestBase {
 
   std::unique_ptr<TestBrowserWindow> browser_window_;
   std::unique_ptr<Browser> browser_;
-  ScopedCurrentChannel channel_{version_info::Channel::UNKNOWN};
 };
 
 void ReadingListApiUnitTest::SetUp() {
@@ -96,16 +97,16 @@ void ReadingListApiUnitTest::SetUp() {
   params.window = browser_window_.get();
   browser_ = std::unique_ptr<Browser>(Browser::Create(params));
 
-  ReadingListEventRouter::GetFactoryInstance()->SetTestingFactory(
+  ReadingListEventRouterFactory::GetInstance()->SetTestingFactory(
       browser_context(), base::BindRepeating(&BuildReadingListEventRouter));
 
   EventRouterFactory::GetInstance()->SetTestingFactory(
       browser_context(), base::BindRepeating(&BuildEventRouter));
 
-  // We need to call ReadingListEventRouterFactory::GetForProfile() in order to
-  // instantiate the keyed service, since it's not created by default in unit
-  // tests.
-  ReadingListEventRouter::Get(browser_context());
+  // We need to call ReadingListEventRouterFactory::GetForBrowserContext() in
+  // order to instantiate the keyed service, since it's not created by default
+  // in unit tests.
+  ReadingListEventRouterFactory::GetForBrowserContext(browser_context());
 }
 
 void ReadingListApiUnitTest::TearDown() {
@@ -413,7 +414,7 @@ TEST_F(ReadingListApiUnitTest, RetrieveCertainEntries) {
       reading_list_model->GetEntryByURL(GURL("https://www.example3.com"));
 
   // Expect that the first entry is equivalent to `e2`.
-  absl::optional<api::reading_list::ReadingListEntry> entry1_actual =
+  std::optional<api::reading_list::ReadingListEntry> entry1_actual =
       api::reading_list::ReadingListEntry::FromValue(entries->GetList()[0]);
   int64_t e2_update_in_milliseconds =
       base::Microseconds(e2->UpdateTime()).InMilliseconds();
@@ -426,7 +427,7 @@ TEST_F(ReadingListApiUnitTest, RetrieveCertainEntries) {
   EXPECT_EQ(entry1_actual->creation_time, e2_creation_in_milliseconds);
 
   // Expect that the second entry is equivalent to `e3`.
-  absl::optional<api::reading_list::ReadingListEntry> entry2_actual =
+  std::optional<api::reading_list::ReadingListEntry> entry2_actual =
       api::reading_list::ReadingListEntry::FromValue(entries->GetList()[1]);
   int64_t e3_update_in_milliseconds =
       base::Microseconds(e3->UpdateTime()).InMilliseconds();
@@ -502,7 +503,7 @@ TEST_F(ReadingListApiUnitTest, ReadingListOnEntryRemoved) {
 
   TestEventRouterObserver event_observer(EventRouter::Get(browser_context()));
 
-  reading_list_model->RemoveEntryByURL(url);
+  reading_list_model->RemoveEntryByURL(url, FROM_HERE);
   EXPECT_EQ(reading_list_model->size(), 0u);
 
   EXPECT_EQ(event_observer.events().size(), 1u);
@@ -527,6 +528,13 @@ TEST_F(ReadingListApiUnitTest, ReadingListOnEntryUpdated) {
 
   TestEventRouterObserver event_observer(EventRouter::Get(browser_context()));
 
+  reading_list_model->SetReadStatusIfExists(url, /*read=*/true);
+  EXPECT_TRUE(reading_list_model->GetEntryByURL(url)->IsRead());
+
+  EXPECT_EQ(event_observer.events().size(), 1u);
+  EXPECT_TRUE(base::Contains(event_observer.events(),
+                             api::reading_list::OnEntryUpdated::kEventName));
+
   reading_list_model->SetEntryTitleIfExists(url, "New title");
   EXPECT_EQ(reading_list_model->GetEntryByURL(url)->Title(), "New title");
 
