@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,10 +6,10 @@
 
 #include <memory>
 
-#include "base/bind.h"
-#include "base/callback.h"
 #include "base/files/file_path.h"
 #include "base/files/scoped_temp_dir.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback.h"
 #include "base/json/json_writer.h"
 #include "base/memory/ref_counted.h"
 #include "components/policy/core/common/external_data_fetcher.h"
@@ -22,6 +22,7 @@
 #include "extensions/browser/api/storage/backend_task_runner.h"
 #include "extensions/browser/api/storage/settings_observer.h"
 #include "extensions/browser/api/storage/storage_area_namespace.h"
+#include "extensions/common/extension_id.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -52,15 +53,19 @@ std::string ValueStoreChangeToJson(value_store::ValueStoreChangeList changes) {
 
 class MockSettingsObserver {
  public:
-  MOCK_METHOD3(OnSettingsChangedJSON,
-               void(const std::string& extension_id,
-                    StorageAreaNamespace storage_area,
-                    const std::string& changes_json));
+  MOCK_METHOD4(
+      OnSettingsChangedJSON,
+      void(const ExtensionId& extension_id,
+           StorageAreaNamespace storage_area,
+           std::optional<api::storage::AccessLevel> session_access_level,
+           const std::string& changes_json));
 
-  void OnSettingsChanged(const std::string& extension_id,
-                         StorageAreaNamespace storage_area,
-                         base::Value changes) {
-    OnSettingsChangedJSON(extension_id, storage_area,
+  void OnSettingsChanged(
+      const ExtensionId& extension_id,
+      StorageAreaNamespace storage_area,
+      std::optional<api::storage::AccessLevel> session_access_level,
+      base::Value changes) {
+    OnSettingsChangedJSON(extension_id, storage_area, session_access_level,
                           ValueToJson(std::move(changes)));
   }
 };
@@ -90,7 +95,7 @@ class MutablePolicyValueStore : public PolicyValueStore {
   }
 
   WriteResult Set(WriteOptions options,
-                  const base::DictionaryValue& values) override {
+                  const base::Value::Dict& values) override {
     return delegate()->Set(options, values);
   }
 
@@ -118,7 +123,7 @@ INSTANTIATE_TEST_SUITE_P(PolicyValueStoreTest,
 class PolicyValueStoreTest : public testing::Test {
  public:
   PolicyValueStoreTest() = default;
-  ~PolicyValueStoreTest() override {}
+  ~PolicyValueStoreTest() override = default;
 
   void SetUp() override {
     ASSERT_TRUE(scoped_temp_dir_.CreateUniqueTempDir());
@@ -166,10 +171,11 @@ TEST_F(PolicyValueStoreTest, DontProvideRecommendedPolicies) {
 
   ValueStore::ReadResult result = store_->Get();
   ASSERT_TRUE(result.status().ok());
-  EXPECT_EQ(1u, result.settings().DictSize());
-  base::Value* value = NULL;
-  EXPECT_FALSE(result.settings().Get("may", &value));
-  EXPECT_TRUE(result.settings().Get("must", &value));
+  EXPECT_EQ(1u, result.settings().size());
+  base::Value* value = result.settings().Find("may");
+  EXPECT_FALSE(value);
+  value = result.settings().Find("must");
+  ASSERT_TRUE(value);
   EXPECT_EQ(expected, *value);
 }
 
@@ -179,8 +185,8 @@ TEST_F(PolicyValueStoreTest, ReadOnly) {
   base::Value string_value("value");
   EXPECT_FALSE(store_->Set(options, "key", string_value).status().ok());
 
-  base::DictionaryValue dict;
-  dict.SetStringKey("key", "value");
+  base::Value::Dict dict;
+  dict.Set("key", "value");
   EXPECT_FALSE(store_->Set(options, dict).status().ok());
 
   EXPECT_FALSE(store_->Remove("key").status().ok());
@@ -196,10 +202,12 @@ TEST_F(PolicyValueStoreTest, NotifyOnChanges) {
   {
     value_store::ValueStoreChangeList changes;
     changes.push_back(
-        value_store::ValueStoreChange("aaa", absl::nullopt, value.Clone()));
-    EXPECT_CALL(observer_, OnSettingsChangedJSON(
-                               kTestExtensionId, StorageAreaNamespace::kManaged,
-                               ValueStoreChangeToJson(std::move(changes))));
+        value_store::ValueStoreChange("aaa", std::nullopt, value.Clone()));
+    EXPECT_CALL(observer_,
+                OnSettingsChangedJSON(
+                    kTestExtensionId, StorageAreaNamespace::kManaged,
+                    /*session_access_level=*/testing::Eq(std::nullopt),
+                    ValueStoreChangeToJson(std::move(changes))));
   }
 
   policy::PolicyMap policies;
@@ -212,10 +220,12 @@ TEST_F(PolicyValueStoreTest, NotifyOnChanges) {
   {
     value_store::ValueStoreChangeList changes;
     changes.push_back(
-        value_store::ValueStoreChange("bbb", absl::nullopt, value.Clone()));
-    EXPECT_CALL(observer_, OnSettingsChangedJSON(
-                               kTestExtensionId, StorageAreaNamespace::kManaged,
-                               ValueStoreChangeToJson(std::move(changes))));
+        value_store::ValueStoreChange("bbb", std::nullopt, value.Clone()));
+    EXPECT_CALL(observer_,
+                OnSettingsChangedJSON(
+                    kTestExtensionId, StorageAreaNamespace::kManaged,
+                    /*session_access_level=*/testing::Eq(std::nullopt),
+                    ValueStoreChangeToJson(std::move(changes))));
   }
 
   policies.Set("bbb", policy::POLICY_LEVEL_MANDATORY, policy::POLICY_SCOPE_USER,
@@ -229,9 +239,11 @@ TEST_F(PolicyValueStoreTest, NotifyOnChanges) {
     value_store::ValueStoreChangeList changes;
     changes.push_back(
         value_store::ValueStoreChange("bbb", value.Clone(), new_value.Clone()));
-    EXPECT_CALL(observer_, OnSettingsChangedJSON(
-                               kTestExtensionId, StorageAreaNamespace::kManaged,
-                               ValueStoreChangeToJson(std::move(changes))));
+    EXPECT_CALL(observer_,
+                OnSettingsChangedJSON(
+                    kTestExtensionId, StorageAreaNamespace::kManaged,
+                    /*session_access_level=*/testing::Eq(std::nullopt),
+                    ValueStoreChangeToJson(std::move(changes))));
   }
 
   policies.Set("bbb", policy::POLICY_LEVEL_MANDATORY, policy::POLICY_SCOPE_USER,
@@ -243,10 +255,13 @@ TEST_F(PolicyValueStoreTest, NotifyOnChanges) {
   {
     value_store::ValueStoreChangeList changes;
     changes.push_back(
-        value_store::ValueStoreChange("bbb", new_value.Clone(), absl::nullopt));
-    EXPECT_CALL(observer_, OnSettingsChangedJSON(
-                               kTestExtensionId, StorageAreaNamespace::kManaged,
-                               ValueStoreChangeToJson(std::move(changes))));
+        value_store::ValueStoreChange("bbb", new_value.Clone(), std::nullopt));
+    EXPECT_CALL(observer_,
+                OnSettingsChangedJSON(
+                    kTestExtensionId, StorageAreaNamespace::kManaged,
+
+                    /*session_access_level=*/testing::Eq(std::nullopt),
+                    ValueStoreChangeToJson(std::move(changes))));
   }
 
   policies.Erase("bbb");
@@ -254,7 +269,7 @@ TEST_F(PolicyValueStoreTest, NotifyOnChanges) {
   Mock::VerifyAndClearExpectations(&observer_);
 
   // Don't notify when there aren't any changes.
-  EXPECT_CALL(observer_, OnSettingsChangedJSON(_, _, _)).Times(0);
+  EXPECT_CALL(observer_, OnSettingsChangedJSON(_, _, _, _)).Times(0);
   SetCurrentPolicy(policies);
   Mock::VerifyAndClearExpectations(&observer_);
 }
