@@ -4,6 +4,7 @@
 
 #include "chrome/browser/extensions/api/system_private/system_private_api.h"
 
+#include <array>
 #include <memory>
 #include <utility>
 
@@ -18,7 +19,7 @@
 #include "components/prefs/pref_service.h"
 #include "google_apis/google_api_keys.h"
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
 #include "chromeos/ash/components/dbus/update_engine/update_engine_client.h"
 #else
 #include "chrome/browser/upgrade_detector/upgrade_detector.h"
@@ -28,11 +29,11 @@ namespace {
 
 // Maps policy::policy_prefs::kIncognitoModeAvailability values (0 = enabled,
 // ...) to strings exposed to extensions.
-const char* const kIncognitoModeAvailabilityStrings[] = {
-  "enabled",
-  "disabled",
-  "forced"
-};
+const auto kIncognitoModeAvailabilityStrings = std::to_array<const char*>({
+    "enabled",
+    "disabled",
+    "forced",
+});
 
 // Property keys.
 const char kDownloadProgressKey[] = "downloadProgress";
@@ -42,9 +43,9 @@ const char kStateKey[] = "state";
 const char kNotAvailableState[] = "NotAvailable";
 const char kNeedRestartState[] = "NeedRestart";
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
 const char kUpdatingState[] = "Updating";
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+#endif  // BUILDFLAG(IS_CHROMEOS)
 
 }  // namespace
 
@@ -67,7 +68,7 @@ SystemPrivateGetIncognitoModeAvailabilityFunction::Run() {
 ExtensionFunction::ResponseAction SystemPrivateGetUpdateStatusFunction::Run() {
   std::string state;
   double download_progress = 0;
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
   // With UpdateEngineClient, we can provide more detailed information about
   // system updates on ChromeOS.
   const update_engine::StatusResult status =
