@@ -1,4 +1,4 @@
-// Copyright 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,11 @@
 #include <stddef.h>
 
 #include <memory>
+#include <optional>
 #include <utility>
+#include <vector>
 
-#include "base/bind.h"
-#include "base/values.h"
+#include "base/functional/bind.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/api/tabs/tabs_constants.h"
 #include "chrome/browser/extensions/api/tabs/tabs_windows_api.h"
@@ -18,6 +19,8 @@
 #include "chrome/browser/extensions/browser_extension_window_controller.h"
 #include "chrome/browser/extensions/extension_tab_util.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/resource_coordinator/lifecycle_unit_state.mojom-shared.h"
+#include "chrome/browser/resource_coordinator/tab_manager.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/recently_audible_helper.h"
@@ -28,10 +31,10 @@
 #include "content/public/browser/navigation_entry.h"
 #include "content/public/browser/web_contents.h"
 #include "extensions/common/features/feature.h"
+#include "extensions/common/mojom/context_type.mojom.h"
+#include "extensions/common/mojom/event_dispatcher.mojom-forward.h"
 #include "third_party/blink/public/common/page/page_zoom.h"
 
-using base::DictionaryValue;
-using base::ListValue;
 using base::Value;
 using content::WebContents;
 using zoom::ZoomController;
@@ -40,53 +43,75 @@ namespace extensions {
 
 namespace {
 
+constexpr char kFromIndexKey[] = "fromIndex";
+constexpr char kGroupIdKey[] = "groupId";
+constexpr char kNewPositionKey[] = "newPosition";
+constexpr char kNewWindowIdKey[] = "newWindowId";
+constexpr char kOldPositionKey[] = "oldPosition";
+constexpr char kOldWindowIdKey[] = "oldWindowId";
+constexpr char kPinnedKey[] = "pinned";
+constexpr char kAudibleKey[] = "audible";
+constexpr char kFrozenKey[] = "frozen";
+constexpr char kDiscardedKey[] = "discarded";
+constexpr char kAutoDiscardableKey[] = "autoDiscardable";
+constexpr char kMutedInfoKey[] = "mutedInfo";
+constexpr char kSelectedKey[] = "selected";
+constexpr char kStatusKey[] = "status";
+constexpr char kTabIdKey[] = "tabId";
+constexpr char kTabIdsKey[] = "tabIds";
+constexpr char kToIndexKey[] = "toIndex";
+constexpr char kWindowClosing[] = "isWindowClosing";
+
 bool WillDispatchTabUpdatedEvent(
     WebContents* contents,
-    const std::set<std::string> changed_property_names,
+    const std::set<std::string>& changed_property_names,
     content::BrowserContext* browser_context,
-    Feature::Context target_context,
+    mojom::ContextType target_context,
     const Extension* extension,
-    Event* event,
-    const base::DictionaryValue* listener_filter) {
+    const base::Value::Dict* listener_filter,
+    std::optional<base::Value::List>& event_args_out,
+    mojom::EventFilteringInfoPtr& event_filtering_info_out) {
   ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
       ExtensionTabUtil::GetScrubTabBehavior(extension, target_context,
                                             contents);
-  std::unique_ptr<api::tabs::Tab> tab_object =
-      ExtensionTabUtil::CreateTabObject(contents, scrub_tab_behavior,
-                                        extension);
+  api::tabs::Tab tab_object = ExtensionTabUtil::CreateTabObject(
+      contents, scrub_tab_behavior, extension);
 
-  std::unique_ptr<base::DictionaryValue> tab_value = tab_object->ToValue();
+  base::Value::Dict tab_value = tab_object.ToValue();
 
-  auto changed_properties = std::make_unique<base::DictionaryValue>();
-  const base::Value* value = nullptr;
+  base::Value::Dict changed_properties;
   for (const auto& property : changed_property_names) {
-    if (tab_value->Get(property, &value))
-      changed_properties->Set(property,
-                              std::make_unique<base::Value>(value->Clone()));
+    if (const base::Value* value = tab_value.Find(property))
+      changed_properties.Set(property, value->Clone());
   }
 
-  event->event_args->Set(1, std::move(changed_properties));
-  event->event_args->Set(2, std::move(tab_value));
+  event_args_out.emplace();
+  event_args_out->Append(ExtensionTabUtil::GetTabId(contents));
+  event_args_out->Append(std::move(changed_properties));
+  event_args_out->Append(std::move(tab_value));
   return true;
 }
 
-bool WillDispatchTabCreatedEvent(WebContents* contents,
-                                 bool active,
-                                 content::BrowserContext* browser_context,
-                                 Feature::Context target_context,
-                                 const Extension* extension,
-                                 Event* event,
-                                 const base::DictionaryValue* listener_filter) {
-  event->event_args->Clear();
+bool WillDispatchTabCreatedEvent(
+    WebContents* contents,
+    bool active,
+    content::BrowserContext* browser_context,
+    mojom::ContextType target_context,
+    const Extension* extension,
+    const base::Value::Dict* listener_filter,
+    std::optional<base::Value::List>& event_args_out,
+    mojom::EventFilteringInfoPtr& event_filtering_info_out) {
   ExtensionTabUtil::ScrubTabBehavior scrub_tab_behavior =
       ExtensionTabUtil::GetScrubTabBehavior(extension, target_context,
                                             contents);
-  std::unique_ptr<base::DictionaryValue> tab_value =
+  base::Value::Dict tab_value =
       ExtensionTabUtil::CreateTabObject(contents, scrub_tab_behavior, extension)
-          ->ToValue();
-  tab_value->SetBoolean(tabs_constants::kSelectedKey, active);
-  tab_value->SetBoolean(tabs_constants::kActiveKey, active);
-  event->event_args->Append(std::move(tab_value));
+          .ToValue();
+  tab_value.Set(kSelectedKey, active);
+  tab_value.Set(tabs_constants::kActiveKey, active);
+
+  event_args_out.emplace();
+  event_args_out->Append(std::move(tab_value));
   return true;
 }
 
@@ -114,7 +139,7 @@ std::set<std::string> TabsEventRouter::TabEntry::UpdateLoadState() {
   // Send 'status' of tab change. Expecting 'complete' is fired.
   complete_waiting_on_load_ = false;
   std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kStatusKey);
+  changed_property_names.insert(kStatusKey);
   return changed_property_names;
 }
 
@@ -137,7 +162,7 @@ void TabsEventRouter::TabEntry::NavigationEntryCommitted(
   // Send 'status' of tab change. Expecting 'loading' is fired.
   complete_waiting_on_load_ = true;
   std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kStatusKey);
+  changed_property_names.insert(kStatusKey);
 
   if (web_contents()->GetURL() != url_) {
     url_ = web_contents()->GetURL();
@@ -169,7 +194,7 @@ TabsEventRouter::TabsEventRouter(Profile* profile)
   BrowserList::AddObserver(this);
   browser_tab_strip_tracker_.Init();
 
-  tab_manager_scoped_observer_.Add(g_browser_process->GetTabManager());
+  tab_manager_scoped_observation_.Observe(g_browser_process->GetTabManager());
 }
 
 TabsEventRouter::~TabsEventRouter() {
@@ -177,7 +202,7 @@ TabsEventRouter::~TabsEventRouter() {
 }
 
 bool TabsEventRouter::ShouldTrackBrowser(Browser* browser) {
-  return profile_->IsSameProfile(browser->profile()) &&
+  return profile_->IsSameOrParent(browser->profile()) &&
          ExtensionTabUtil::BrowserSupportsTabs(browser);
 }
 
@@ -185,7 +210,7 @@ void TabsEventRouter::OnBrowserSetLastActive(Browser* browser) {
   TabsWindowsAPI* tabs_window_api = TabsWindowsAPI::Get(profile_);
   if (tabs_window_api) {
     tabs_window_api->windows_event_router()->OnActiveWindowChanged(
-        browser ? browser->extension_window_controller() : NULL);
+        browser ? browser->extension_window_controller() : nullptr);
   }
 }
 
@@ -203,9 +228,9 @@ void TabsEventRouter::OnTabStripModelChanged(
       break;
     }
     case TabStripModelChange::kRemoved: {
-      const bool will_be_deleted = change.GetRemove()->will_be_deleted;
       for (const auto& contents : change.GetRemove()->contents) {
-        if (will_be_deleted) {
+        if (contents.remove_reason ==
+            TabStripModelChange::RemoveReason::kDeleted) {
           DispatchTabClosingAt(tab_strip_model, contents.contents,
                                contents.index);
         }
@@ -233,10 +258,8 @@ void TabsEventRouter::OnTabStripModelChanged(
   if (tab_strip_model->empty())
     return;
 
-  if (selection.active_tab_changed()) {
-    DispatchActiveTabChanged(selection.old_contents, selection.new_contents,
-                             selection.new_model.active());
-  }
+  if (selection.active_tab_changed())
+    DispatchActiveTabChanged(selection.old_contents, selection.new_contents);
 
   if (selection.selection_changed()) {
     DispatchTabSelectionChanged(tab_strip_model, selection.old_model);
@@ -257,10 +280,27 @@ void TabsEventRouter::TabPinnedStateChanged(TabStripModel* tab_strip_model,
                                             WebContents* contents,
                                             int index) {
   std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kPinnedKey);
+  changed_property_names.insert(kPinnedKey);
   DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
 }
 
+void TabsEventRouter::TabGroupedStateChanged(
+    std::optional<tab_groups::TabGroupId> group,
+    tabs::TabInterface* tab,
+    int index) {
+  std::set<std::string> changed_property_names;
+  changed_property_names.insert(kGroupIdKey);
+  DispatchTabUpdatedEvent(tab->GetContents(),
+                          std::move(changed_property_names));
+}
+
+void TabsEventRouter::OnZoomControllerDestroyed(
+    zoom::ZoomController* zoom_controller) {
+  if (zoom_scoped_observations_.IsObservingSource(zoom_controller)) {
+    zoom_scoped_observations_.RemoveObservation(zoom_controller);
+  }
+}
+
 void TabsEventRouter::OnZoomChanged(
     const ZoomController::ZoomChangedEventData& data) {
   DCHECK(data.web_contents);
@@ -272,9 +312,9 @@ void TabsEventRouter::OnZoomChanged(
   api::tabs::OnZoomChange::ZoomChangeInfo zoom_change_info;
   zoom_change_info.tab_id = tab_id;
   zoom_change_info.old_zoom_factor =
-      blink::PageZoomLevelToZoomFactor(data.old_zoom_level);
+      blink::ZoomLevelToZoomFactor(data.old_zoom_level);
   zoom_change_info.new_zoom_factor =
-      blink::PageZoomLevelToZoomFactor(data.new_zoom_level);
+      blink::ZoomLevelToZoomFactor(data.new_zoom_level);
   ZoomModeToZoomSettings(data.zoom_mode, &zoom_change_info.zoom_settings);
 
   // Dispatch the |onZoomChange| event.
@@ -299,25 +339,41 @@ void TabsEventRouter::OnFaviconUpdated(
   }
 }
 
-void TabsEventRouter::OnDiscardedStateChange(
-    WebContents* contents,
-    ::mojom::LifecycleUnitDiscardReason reason,
-    bool is_discarded) {
+void TabsEventRouter::OnTabLifecycleStateChange(
+    content::WebContents* contents,
+    ::mojom::LifecycleUnitState previous_state,
+    ::mojom::LifecycleUnitState new_state,
+    std::optional<LifecycleUnitDiscardReason> discard_reason) {
+  auto previous_or_new_state_is = [&](::mojom::LifecycleUnitState state) {
+    return previous_state == state || new_state == state;
+  };
+
   std::set<std::string> changed_property_names;
-  // If the "discarded" property changes, so does the "status" property:
-  // - a discarded tab has status "unloaded", and will transition to "loading"
-  //   on un-discarding; and,
-  // - a tab can only be discarded if its status is "complete" or "loading", in
-  //   which case it will transition to "unloaded".
-  changed_property_names.insert(tabs_constants::kDiscardedKey);
-  changed_property_names.insert(tabs_constants::kStatusKey);
-  DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
+
+  if (previous_or_new_state_is(::mojom::LifecycleUnitState::DISCARDED)) {
+    // If the "discarded" property changes, so does the "status" property:
+    // - a discarded tab has status "unloaded", and will transition to "loading"
+    //   on un-discarding; and,
+    // - a tab can only be discarded if its status is "complete" or "loading",
+    //   in which case it will transition to "unloaded".
+    changed_property_names.insert(kDiscardedKey);
+    changed_property_names.insert(kStatusKey);
+  }
+
+  if (previous_or_new_state_is(::mojom::LifecycleUnitState::FROZEN)) {
+    changed_property_names.insert(kFrozenKey);
+  }
+
+  if (!changed_property_names.empty()) {
+    DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
+  }
 }
 
-void TabsEventRouter::OnAutoDiscardableStateChange(WebContents* contents,
-                                                   bool is_auto_discardable) {
+void TabsEventRouter::OnTabAutoDiscardableStateChange(
+    WebContents* contents,
+    bool is_auto_discardable) {
   std::set<std::string> changed_property_names;
-  changed_property_names.insert(tabs_constants::kAutoDiscardableKey);
+  changed_property_names.insert(kAutoDiscardableKey);
   DispatchTabUpdatedEvent(contents, std::move(changed_property_names));
 }
 
@@ -336,17 +392,14 @@ void TabsEventRouter::DispatchTabInsertedAt(TabStripModel* tab_strip_model,
   }
 
   int tab_id = ExtensionTabUtil::GetTabId(contents);
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(tab_id);
+  base::Value::List args;
+  args.Append(tab_id);
 
-  std::unique_ptr<base::DictionaryValue> object_args(
-      new base::DictionaryValue());
-  object_args->Set(
-      tabs_constants::kNewWindowIdKey,
-      std::make_unique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
-  object_args->Set(tabs_constants::kNewPositionKey,
-                   std::make_unique<Value>(index));
-  args->Append(std::move(object_args));
+  base::Value::Dict object_args;
+  object_args.Set(kNewWindowIdKey,
+                  Value(ExtensionTabUtil::GetWindowIdOfTab(contents)));
+  object_args.Set(kNewPositionKey, Value(index));
+  args.Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
   DispatchEvent(profile, events::TABS_ON_ATTACHED,
@@ -359,16 +412,14 @@ void TabsEventRouter::DispatchTabClosingAt(TabStripModel* tab_strip_model,
                                            int index) {
   int tab_id = ExtensionTabUtil::GetTabId(contents);
 
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(tab_id);
+  base::Value::List args;
+  args.Append(tab_id);
 
-  std::unique_ptr<base::DictionaryValue> object_args(
-      new base::DictionaryValue());
-  object_args->SetInteger(tabs_constants::kWindowIdKey,
-                          ExtensionTabUtil::GetWindowIdOfTab(contents));
-  object_args->SetBoolean(tabs_constants::kWindowClosing,
-                          tab_strip_model->closing_all());
-  args->Append(std::move(object_args));
+  base::Value::Dict object_args;
+  object_args.Set(tabs_constants::kWindowIdKey,
+                  ExtensionTabUtil::GetWindowIdOfTab(contents));
+  object_args.Set(kWindowClosing, tab_strip_model->closing_all());
+  args.Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
   DispatchEvent(profile, events::TABS_ON_REMOVED,
@@ -386,17 +437,14 @@ void TabsEventRouter::DispatchTabDetachedAt(WebContents* contents,
     return;
   }
 
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(ExtensionTabUtil::GetTabId(contents));
+  base::Value::List args;
+  args.Append(ExtensionTabUtil::GetTabId(contents));
 
-  std::unique_ptr<base::DictionaryValue> object_args(
-      new base::DictionaryValue());
-  object_args->Set(
-      tabs_constants::kOldWindowIdKey,
-      std::make_unique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
-  object_args->Set(tabs_constants::kOldPositionKey,
-                   std::make_unique<Value>(index));
-  args->Append(std::move(object_args));
+  base::Value::Dict object_args;
+  object_args.Set(kOldWindowIdKey,
+                  ExtensionTabUtil::GetWindowIdOfTab(contents));
+  object_args.Set(kOldPositionKey, index);
+  args.Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
   DispatchEvent(profile, events::TABS_ON_DETACHED,
@@ -405,17 +453,15 @@ void TabsEventRouter::DispatchTabDetachedAt(WebContents* contents,
 }
 
 void TabsEventRouter::DispatchActiveTabChanged(WebContents* old_contents,
-                                               WebContents* new_contents,
-                                               int index) {
-  auto args = std::make_unique<base::ListValue>();
+                                               WebContents* new_contents) {
+  base::Value::List args;
   int tab_id = ExtensionTabUtil::GetTabId(new_contents);
-  args->AppendInteger(tab_id);
+  args.Append(tab_id);
 
-  auto object_args = std::make_unique<base::DictionaryValue>();
-  object_args->Set(tabs_constants::kWindowIdKey,
-                   std::make_unique<Value>(
-                       ExtensionTabUtil::GetWindowIdOfTab(new_contents)));
-  args->Append(object_args->CreateDeepCopy());
+  base::Value::Dict object_args;
+  object_args.Set(tabs_constants::kWindowIdKey,
+                  ExtensionTabUtil::GetWindowIdOfTab(new_contents));
+  args.Append(object_args.Clone());
 
   // The onActivated event replaced onActiveChanged and onSelectionChanged. The
   // deprecated events take two arguments: tabId, {windowId}.
@@ -423,16 +469,16 @@ void TabsEventRouter::DispatchActiveTabChanged(WebContents* old_contents,
       Profile::FromBrowserContext(new_contents->GetBrowserContext());
 
   DispatchEvent(profile, events::TABS_ON_SELECTION_CHANGED,
-                api::tabs::OnSelectionChanged::kEventName,
-                args->CreateDeepCopy(), EventRouter::USER_GESTURE_UNKNOWN);
+                api::tabs::OnSelectionChanged::kEventName, args.Clone(),
+                EventRouter::USER_GESTURE_UNKNOWN);
   DispatchEvent(profile, events::TABS_ON_ACTIVE_CHANGED,
                 api::tabs::OnActiveChanged::kEventName, std::move(args),
                 EventRouter::USER_GESTURE_UNKNOWN);
 
   // The onActivated event takes one argument: {windowId, tabId}.
-  auto on_activated_args = std::make_unique<base::ListValue>();
-  object_args->Set(tabs_constants::kTabIdKey, std::make_unique<Value>(tab_id));
-  on_activated_args->Append(std::move(object_args));
+  base::Value::List on_activated_args;
+  object_args.Set(kTabIdKey, tab_id);
+  on_activated_args.Append(std::move(object_args));
   DispatchEvent(
       profile, events::TABS_ON_ACTIVATED, api::tabs::OnActivated::kEventName,
       std::move(on_activated_args), EventRouter::USER_GESTURE_UNKNOWN);
@@ -443,33 +489,30 @@ void TabsEventRouter::DispatchTabSelectionChanged(
     const ui::ListSelectionModel& old_model) {
   ui::ListSelectionModel::SelectedIndices new_selection =
       tab_strip_model->selection_model().selected_indices();
-  std::unique_ptr<base::ListValue> all_tabs(new base::ListValue);
+  base::Value::List all_tabs;
 
-  for (size_t i = 0; i < new_selection.size(); ++i) {
-    int index = new_selection[i];
+  for (int index : new_selection) {
     WebContents* contents = tab_strip_model->GetWebContentsAt(index);
     if (!contents)
       break;
     int tab_id = ExtensionTabUtil::GetTabId(contents);
-    all_tabs->AppendInteger(tab_id);
+    all_tabs.Append(tab_id);
   }
 
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  std::unique_ptr<base::DictionaryValue> select_info(new base::DictionaryValue);
+  base::Value::List args;
+  base::Value::Dict select_info;
 
-  select_info->Set(
+  select_info.Set(
       tabs_constants::kWindowIdKey,
-      std::make_unique<Value>(
-          ExtensionTabUtil::GetWindowIdOfTabStripModel(tab_strip_model)));
+      ExtensionTabUtil::GetWindowIdOfTabStripModel(tab_strip_model));
 
-  select_info->Set(tabs_constants::kTabIdsKey, std::move(all_tabs));
-  args->Append(std::move(select_info));
+  select_info.Set(kTabIdsKey, std::move(all_tabs));
+  args.Append(std::move(select_info));
 
   // The onHighlighted event replaced onHighlightChanged.
   Profile* profile = tab_strip_model->profile();
   DispatchEvent(profile, events::TABS_ON_HIGHLIGHT_CHANGED,
-                api::tabs::OnHighlightChanged::kEventName,
-                std::unique_ptr<base::ListValue>(args->DeepCopy()),
+                api::tabs::OnHighlightChanged::kEventName, args.Clone(),
                 EventRouter::USER_GESTURE_UNKNOWN);
   DispatchEvent(profile, events::TABS_ON_HIGHLIGHTED,
                 api::tabs::OnHighlighted::kEventName, std::move(args),
@@ -479,19 +522,15 @@ void TabsEventRouter::DispatchTabSelectionChanged(
 void TabsEventRouter::DispatchTabMoved(WebContents* contents,
                                        int from_index,
                                        int to_index) {
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(ExtensionTabUtil::GetTabId(contents));
+  base::Value::List args;
+  args.Append(ExtensionTabUtil::GetTabId(contents));
 
-  std::unique_ptr<base::DictionaryValue> object_args(
-      new base::DictionaryValue());
-  object_args->Set(
-      tabs_constants::kWindowIdKey,
-      std::make_unique<Value>(ExtensionTabUtil::GetWindowIdOfTab(contents)));
-  object_args->Set(tabs_constants::kFromIndexKey,
-                   std::make_unique<Value>(from_index));
-  object_args->Set(tabs_constants::kToIndexKey,
-                   std::make_unique<Value>(to_index));
-  args->Append(std::move(object_args));
+  base::Value::Dict object_args;
+  object_args.Set(tabs_constants::kWindowIdKey,
+                  ExtensionTabUtil::GetWindowIdOfTab(contents));
+  object_args.Set(kFromIndexKey, from_index);
+  object_args.Set(kToIndexKey, to_index);
+  args.Append(std::move(object_args));
 
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
   DispatchEvent(profile, events::TABS_ON_MOVED, api::tabs::OnMoved::kEventName,
@@ -505,9 +544,9 @@ void TabsEventRouter::DispatchTabReplacedAt(WebContents* old_contents,
   // WebContents being swapped.
   const int new_tab_id = ExtensionTabUtil::GetTabId(new_contents);
   const int old_tab_id = ExtensionTabUtil::GetTabId(old_contents);
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
-  args->AppendInteger(new_tab_id);
-  args->AppendInteger(old_tab_id);
+  base::Value::List args;
+  args.Append(new_tab_id);
+  args.Append(old_tab_id);
 
   DispatchEvent(Profile::FromBrowserContext(new_contents->GetBrowserContext()),
                 events::TABS_ON_REPLACED, api::tabs::OnReplaced::kEventName,
@@ -523,10 +562,9 @@ void TabsEventRouter::TabCreatedAt(WebContents* contents,
                                    int index,
                                    bool active) {
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
-  std::unique_ptr<base::ListValue> args(new base::ListValue);
   auto event = std::make_unique<Event>(events::TABS_ON_CREATED,
                                        api::tabs::OnCreated::kEventName,
-                                       std::move(args), profile);
+                                       base::Value::List(), profile);
   event->user_gesture = EventRouter::USER_GESTURE_NOT_ENABLED;
   event->will_dispatch_callback =
       base::BindRepeating(&WillDispatchTabCreatedEvent, contents, active);
@@ -541,12 +579,12 @@ void TabsEventRouter::TabUpdated(TabEntry* entry,
       RecentlyAudibleHelper::FromWebContents(entry->web_contents());
   bool audible = audible_helper->WasRecentlyAudible();
   if (entry->SetAudible(audible)) {
-    changed_property_names.insert(tabs_constants::kAudibleKey);
+    changed_property_names.insert(kAudibleKey);
   }
 
   bool muted = entry->web_contents()->IsAudioMuted();
   if (entry->SetMuted(muted)) {
-    changed_property_names.insert(tabs_constants::kMutedInfoKey);
+    changed_property_names.insert(kMutedInfoKey);
   }
 
   if (!changed_property_names.empty()) {
@@ -568,10 +606,10 @@ void TabsEventRouter::DispatchEvent(
     Profile* profile,
     events::HistogramValue histogram_value,
     const std::string& event_name,
-    std::unique_ptr<base::ListValue> args,
+    base::Value::List args,
     EventRouter::UserGestureState user_gesture) {
   EventRouter* event_router = EventRouter::Get(profile);
-  if (!profile_->IsSameProfile(profile) || !event_router)
+  if (!profile_->IsSameOrParent(profile) || !event_router)
     return;
 
   auto event = std::make_unique<Event>(histogram_value, event_name,
@@ -582,28 +620,17 @@ void TabsEventRouter::DispatchEvent(
 
 void TabsEventRouter::DispatchTabUpdatedEvent(
     WebContents* contents,
-    const std::set<std::string> changed_property_names) {
+    std::set<std::string> changed_property_names) {
   DCHECK(!changed_property_names.empty());
   DCHECK(contents);
 
-  // The state of the tab (as seen from the extension point of view) has
-  // changed.  Send a notification to the extension.
-  std::unique_ptr<base::ListValue> args_base(new base::ListValue);
-
-  // First arg: The id of the tab that changed.
-  args_base->AppendInteger(ExtensionTabUtil::GetTabId(contents));
-
-  // Second arg: An object containing the changes to the tab state.  Filled in
-  // by WillDispatchTabUpdatedEvent as a copy of changed_properties, if the
-  // extension has the tabs permission.
-
-  // Third arg: An object containing the state of the tab. Filled in by
-  // WillDispatchTabUpdatedEvent.
   Profile* profile = Profile::FromBrowserContext(contents->GetBrowserContext());
 
-  auto event = std::make_unique<Event>(events::TABS_ON_UPDATED,
-                                       api::tabs::OnUpdated::kEventName,
-                                       std::move(args_base), profile);
+  auto event = std::make_unique<Event>(
+      events::TABS_ON_UPDATED, api::tabs::OnUpdated::kEventName,
+      // The event arguments depend on the extension's permission. They are set
+      // in WillDispatchTabUpdatedEvent().
+      base::Value::List(), profile);
   event->user_gesture = EventRouter::USER_GESTURE_NOT_ENABLED;
   event->will_dispatch_callback =
       base::BindRepeating(&WillDispatchTabUpdatedEvent, contents,
@@ -612,10 +639,10 @@ void TabsEventRouter::DispatchTabUpdatedEvent(
 }
 
 void TabsEventRouter::RegisterForTabNotifications(WebContents* contents) {
-  favicon_scoped_observer_.Add(
+  favicon_scoped_observations_.AddObservation(
       favicon::ContentFaviconDriver::FromWebContents(contents));
-
-  ZoomController::FromWebContents(contents)->AddObserver(this);
+  zoom_scoped_observations_.AddObservation(
+      ZoomController::FromWebContents(contents));
 
   int tab_id = ExtensionTabUtil::GetTabId(contents);
   DCHECK(tab_entries_.find(tab_id) == tab_entries_.end());
@@ -623,11 +650,13 @@ void TabsEventRouter::RegisterForTabNotifications(WebContents* contents) {
 }
 
 void TabsEventRouter::UnregisterForTabNotifications(WebContents* contents) {
-  favicon_scoped_observer_.Remove(
+  if (auto* zoom_controller = ZoomController::FromWebContents(contents);
+      zoom_scoped_observations_.IsObservingSource(zoom_controller)) {
+    zoom_scoped_observations_.RemoveObservation(zoom_controller);
+  }
+  favicon_scoped_observations_.RemoveObservation(
       favicon::ContentFaviconDriver::FromWebContents(contents));
 
-  ZoomController::FromWebContents(contents)->RemoveObserver(this);
-
   int tab_id = ExtensionTabUtil::GetTabId(contents);
   int removed_count = tab_entries_.erase(tab_id);
   DCHECK_GT(removed_count, 0);
