@@ -10,6 +10,7 @@
 #include "base/metrics/field_trial.h"
 #include "base/metrics/field_trial_params.h"
 #include "base/run_loop.h"
+#include "base/test/scoped_feature_list.h"
 #include "chrome/browser/content_settings/host_content_settings_map_factory.h"
 #include "chrome/browser/media/webrtc/media_capture_devices_dispatcher.h"
 #include "chrome/browser/media/webrtc/media_stream_capture_indicator.h"
@@ -26,6 +27,7 @@
 #include "chrome/test/base/ui_test_utils.h"
 #include "components/content_settings/browser/page_specific_content_settings.h"
 #include "components/content_settings/core/browser/host_content_settings_map.h"
+#include "components/content_settings/core/common/features.h"
 #include "components/permissions/contexts/camera_pan_tilt_zoom_permission_context.h"
 #include "components/permissions/permission_context_base.h"
 #include "components/permissions/permission_manager.h"
@@ -53,6 +55,11 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
         example_video_id_("fake_video_dev"),
         media_stream_result_(
             blink::mojom::MediaStreamRequestResult::NUM_MEDIA_REQUEST_RESULTS) {
+    // `kLeftHandSideActivityIndicators` should be disabled as it changes the UI
+    // of the camera/mic activity indicator. The new UI will be tested
+    // separately.
+    scoped_feature_list_.InitWithFeatures(
+        {}, {content_settings::features::kLeftHandSideActivityIndicators});
   }
 
   void OnMediaStreamResponse(
@@ -172,15 +179,19 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
                     ->GetLastCommittedOrigin()
                     .GetURL());
     }
-    int render_process_id =
-        GetWebContents()->GetPrimaryMainFrame()->GetProcess()->GetID();
+    int render_process_id = GetWebContents()
+                                ->GetPrimaryMainFrame()
+                                ->GetProcess()
+                                ->GetDeprecatedID();
     int render_frame_id =
         GetWebContents()->GetPrimaryMainFrame()->GetRoutingID();
     return content::MediaStreamRequest(
         render_process_id, render_frame_id, 0,
-        example_url().DeprecatedGetOriginAsURL(), false, request_type, audio_id,
-        video_id, audio_type, video_type,
-        /*disable_local_echo=*/false, request_pan_tilt_zoom_permission);
+        url::Origin::Create(example_url()), false, request_type,
+        /*requested_audio_device_ids=*/{audio_id},
+        /*requested_video_device_ids=*/{video_id}, audio_type, video_type,
+        /*disable_local_echo=*/false, request_pan_tilt_zoom_permission,
+        /*captured_surface_control_active=*/false);
   }
 
   content::MediaStreamRequest CreateRequest(
@@ -189,14 +200,14 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
       bool request_pan_tilt_zoom_permission) {
     return CreateRequestWithType(audio_id, video_id,
                                  request_pan_tilt_zoom_permission,
-                                 blink::MEDIA_DEVICE_ACCESS);
+                                 blink::MEDIA_GENERATE_STREAM);
   }
 
   void InitWithUrl(const GURL& url) {
     DCHECK(example_url_.is_empty());
     example_url_ = url;
     ASSERT_TRUE(ui_test_utils::NavigateToURL(browser(), example_url_));
-    EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Empty());
+    EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().empty());
   }
 
   virtual media::VideoCaptureControlSupport GetControlSupport() const {
@@ -250,7 +261,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
     blink::MediaStreamDevice fake_video_device(
         blink::mojom::MediaStreamType::DEVICE_VIDEO_CAPTURE, example_video_id_,
         "Fake Video Device", GetControlSupport(),
-        media::MEDIA_VIDEO_FACING_NONE, absl::nullopt);
+        media::MEDIA_VIDEO_FACING_NONE, std::nullopt);
     video_devices.push_back(fake_video_device);
     MediaCaptureDevicesDispatcher::GetInstance()->SetTestVideoCaptureDevices(
         video_devices);
@@ -276,6 +287,7 @@ class MediaStreamDevicesControllerTest : public WebRtcTestBase {
 
   std::unique_ptr<PermissionBubbleMediaAccessHandler>
       permission_bubble_media_access_handler_;
+  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 class MediaStreamDevicesControllerPtzTest
@@ -304,14 +316,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest, RequestAndAllowMic) {
       ContentSettingsType::MEDIASTREAM_MIC));
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Has(
       PageSpecificContentSettings::kMicrophoneAccessed));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request and allow camera access.
@@ -331,14 +335,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest, RequestAndAllowCam) {
       ContentSettingsType::MEDIASTREAM_CAMERA));
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Has(
       PageSpecificContentSettings::kCameraAccessed));
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request and block microphone access.
@@ -359,14 +355,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest, RequestAndBlockMic) {
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().HasAll(
       {PageSpecificContentSettings::kMicrophoneAccessed,
        PageSpecificContentSettings::kMicrophoneBlocked}));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request and block camera access.
@@ -387,14 +375,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest, RequestAndBlockCam) {
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().HasAll(
       {PageSpecificContentSettings::kCameraAccessed,
        PageSpecificContentSettings::kCameraBlocked}));
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(std::string(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request and allow microphone and camera access.
@@ -422,14 +402,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().HasAll(
       {PageSpecificContentSettings::kMicrophoneAccessed,
        PageSpecificContentSettings::kCameraAccessed}));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request and block microphone and camera access.
@@ -459,14 +431,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
        PageSpecificContentSettings::kMicrophoneBlocked,
        PageSpecificContentSettings::kCameraAccessed,
        PageSpecificContentSettings::kCameraBlocked}));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request microphone and camera access. Camera is denied, thus everything
@@ -497,14 +461,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
        PageSpecificContentSettings::kMicrophoneBlocked,
        PageSpecificContentSettings::kCameraAccessed,
        PageSpecificContentSettings::kCameraBlocked}));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request microphone and camera access. Microphone is denied, thus everything
@@ -535,14 +491,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
        PageSpecificContentSettings::kMicrophoneBlocked,
        PageSpecificContentSettings::kCameraAccessed,
        PageSpecificContentSettings::kCameraBlocked}));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 }
 
 // Request microphone access. Requesting camera should not change microphone
@@ -562,10 +510,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
       ContentSettingsType::MEDIASTREAM_MIC));
   EXPECT_TRUE(GetContentSettings()->IsContentBlocked(
       ContentSettingsType::MEDIASTREAM_MIC));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
 
   // Request cam and allow
   SetDevicePolicy(DEVICE_TYPE_VIDEO, ACCESS_ALLOWED);
@@ -575,20 +519,12 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
       ContentSettingsType::MEDIASTREAM_CAMERA));
   EXPECT_FALSE(GetContentSettings()->IsContentBlocked(
       ContentSettingsType::MEDIASTREAM_CAMERA));
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
 
   // Mic state should not have changed.
   EXPECT_FALSE(GetContentSettings()->IsContentAllowed(
       ContentSettingsType::MEDIASTREAM_MIC));
   EXPECT_TRUE(GetContentSettings()->IsContentBlocked(
       ContentSettingsType::MEDIASTREAM_MIC));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
 }
 
 // Denying mic access after camera access should still show the camera as state.
@@ -607,10 +543,6 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
       ContentSettingsType::MEDIASTREAM_CAMERA));
   EXPECT_FALSE(GetContentSettings()->IsContentBlocked(
       ContentSettingsType::MEDIASTREAM_CAMERA));
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Has(
       PageSpecificContentSettings::kCameraAccessed));
 
@@ -642,20 +574,12 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
       ContentSettingsType::MEDIASTREAM_MIC));
   EXPECT_TRUE(GetContentSettings()->IsContentBlocked(
       ContentSettingsType::MEDIASTREAM_MIC));
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_requested_audio_device());
-  EXPECT_EQ(example_audio_id(),
-            GetContentSettings()->media_stream_selected_audio_device());
 
   // Cam should still be included in the state.
   EXPECT_TRUE(GetContentSettings()->IsContentAllowed(
       ContentSettingsType::MEDIASTREAM_CAMERA));
   EXPECT_FALSE(GetContentSettings()->IsContentBlocked(
       ContentSettingsType::MEDIASTREAM_CAMERA));
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_requested_video_device());
-  EXPECT_EQ(example_video_id(),
-            GetContentSettings()->media_stream_selected_video_device());
   EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().HasAll(
       {PageSpecificContentSettings::kMicrophoneAccessed,
        PageSpecificContentSettings::kMicrophoneBlocked,
@@ -1034,7 +958,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
   content::MediaStreamRequest request =
       CreateRequest(example_audio_id(), example_video_id(), false);
   // Make the child frame the source of the request.
-  request.render_process_id = child_frame->GetProcess()->GetID();
+  request.render_process_id = child_frame->GetProcess()->GetDeprecatedID();
   request.render_frame_id = child_frame->GetRoutingID();
   request.security_origin = child_frame->GetLastCommittedOrigin().GetURL();
 
@@ -1044,7 +968,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
 
   VerifyResultState(blink::mojom::MediaStreamRequestResult::PERMISSION_DENIED,
                     false, false);
-  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Empty());
+  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().empty());
 }
 
 IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
@@ -1065,7 +989,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
   content::MediaStreamRequest request =
       CreateRequest(std::string(), example_video_id(), false);
   // Make the child frame the source of the request.
-  request.render_process_id = child_frame->GetProcess()->GetID();
+  request.render_process_id = child_frame->GetProcess()->GetDeprecatedID();
   request.render_frame_id = child_frame->GetRoutingID();
   request.security_origin = child_frame->GetLastCommittedOrigin().GetURL();
 
@@ -1075,7 +999,7 @@ IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
 
   VerifyResultState(blink::mojom::MediaStreamRequestResult::PERMISSION_DENIED,
                     false, false);
-  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().Empty());
+  EXPECT_TRUE(GetContentSettings()->GetMicrophoneCameraState().empty());
 }
 
 IN_PROC_BROWSER_TEST_F(MediaStreamDevicesControllerTest,
