@@ -20,6 +20,7 @@
 #include "components/signin/public/identity_manager/tribool.h"
 #include "net/cookies/canonical_cookie.h"
 
+class GaiaId;
 class Profile;
 
 namespace signin {
@@ -155,7 +156,7 @@ void RecordEnterpriseProfileCreationUserChoice(bool enforced_by_policy,
 PrimaryAccountError SetPrimaryAccountWithInvalidToken(
     Profile* profile,
     const std::string& user_email,
-    const std::string& gaia_id,
+    const GaiaId& gaia_id,
     bool is_under_advanced_protection,
     signin_metrics::AccessPoint access_point,
     signin_metrics::SourceForRefreshTokenOperation source);
@@ -166,7 +167,7 @@ PrimaryAccountError SetPrimaryAccountWithInvalidToken(
 bool IsSigninPending(signin::IdentityManager* identity_manager);
 
 // Returns the current state of the primary account that is used in Chrome.
-SignedInState GetSignedInState(signin::IdentityManager* identity_manager);
+SignedInState GetSignedInState(const signin::IdentityManager* identity_manager);
 
 }  // namespace signin_util
 
