@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,7 +6,8 @@
 
 #include <memory>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
+#include "base/memory/raw_ptr.h"
 #include "base/run_loop.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/test/scoped_feature_list.h"
@@ -33,9 +34,8 @@ class SpellCheckHostChromeImplWinBrowserTest : public InProcessBrowserTest {
 
   void SetUp() override {
     // Don't delay initialization of the SpellcheckService on browser launch.
-    feature_list_.InitWithFeatures(
-        /*enabled_features=*/{spellcheck::kWinUseBrowserSpellChecker},
-        /*disabled_features=*/{spellcheck::kWinDelaySpellcheckServiceInit});
+    feature_list_.InitAndDisableFeature(
+        spellcheck::kWinDelaySpellcheckServiceInit);
     InProcessBrowserTest::SetUp();
   }
 
@@ -44,7 +44,8 @@ class SpellCheckHostChromeImplWinBrowserTest : public InProcessBrowserTest {
     renderer_ = std::make_unique<content::MockRenderProcessHost>(context);
 
     SpellCheckHostChromeImpl::Create(
-        renderer_->GetID(), spell_check_host_.BindNewPipeAndPassReceiver());
+        renderer_->GetDeprecatedID(),
+        spell_check_host_.BindNewPipeAndPassReceiver());
 
     InitializeSpellcheckService();
 
@@ -91,7 +92,7 @@ class SpellCheckHostChromeImplWinBrowserTest : public InProcessBrowserTest {
   void RunSpellCheckReturnMessageTest();
 
  protected:
-  PlatformSpellChecker* platform_spell_checker_;
+  raw_ptr<PlatformSpellChecker, DanglingUntriaged> platform_spell_checker_;
   base::test::ScopedFeatureList feature_list_;
   std::unique_ptr<content::MockRenderProcessHost> renderer_;
   mojo::Remote<spellcheck::mojom::SpellCheckHost> spell_check_host_;
@@ -109,10 +110,6 @@ IN_PROC_BROWSER_TEST_F(SpellCheckHostChromeImplWinBrowserTest,
 }
 
 void SpellCheckHostChromeImplWinBrowserTest::RunSpellCheckReturnMessageTest() {
-  if (!spellcheck::WindowsVersionSupportsSpellchecker()) {
-    return;
-  }
-
   spellcheck_platform::SetLanguage(
       platform_spell_checker_, "en-US",
       base::BindOnce(&SpellCheckHostChromeImplWinBrowserTest::
@@ -122,7 +119,6 @@ void SpellCheckHostChromeImplWinBrowserTest::RunSpellCheckReturnMessageTest() {
 
   spell_check_host_->RequestTextCheck(
       u"zz.",
-      /*route_id=*/123,
       base::BindOnce(
           &SpellCheckHostChromeImplWinBrowserTest::OnSpellcheckResult,
           base::Unretained(this)));
@@ -141,10 +137,8 @@ class SpellCheckHostChromeImplWinBrowserTestDelayInit
 
   void SetUp() override {
     // Don't initialize the SpellcheckService on browser launch.
-    feature_list_.InitWithFeatures(
-        /*enabled_features=*/{spellcheck::kWinUseBrowserSpellChecker,
-                              spellcheck::kWinDelaySpellcheckServiceInit},
-        /*disabled_features=*/{});
+    feature_list_.InitAndEnableFeature(
+        spellcheck::kWinDelaySpellcheckServiceInit);
     InProcessBrowserTest::SetUp();
   }
 
