@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,29 +6,27 @@
 
 #include <stddef.h>
 
+#include <array>
 #include <memory>
 #include <string>
 #include <vector>
 
-#include "base/bind.h"
+#include "base/functional/bind.h"
 #include "base/json/json_reader.h"
-#include "base/stl_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/test/bind_test_util.h"
-#include "base/test/scoped_feature_list.h"
+#include "base/test/bind.h"
 #include "base/values.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/prefs/pref_service.h"
 #include "components/spellcheck/browser/pref_names.h"
-#include "components/spellcheck/common/spellcheck_features.h"
 #include "components/spellcheck/common/spellcheck_result.h"
-#include "content/public/test/test_browser_thread_bundle.h"
+#include "content/public/test/browser_task_environment.h"
 #include "net/base/load_flags.h"
 #include "net/http/http_util.h"
-#include "services/network/public/cpp/resource_response.h"
 #include "services/network/public/cpp/shared_url_loader_factory.h"
 #include "services/network/public/cpp/weak_wrapper_shared_url_loader_factory.h"
+#include "services/network/public/mojom/url_response_head.mojom.h"
 #include "services/network/test/test_url_loader_factory.h"
 #include "services/network/test/test_utils.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -44,7 +42,6 @@ struct SpellingServiceTestCase {
   bool success;
   const char* corrected_text;
   std::string language;
-  bool restEndpoint;
 };
 
 // A class derived from the SpellingServiceClient class used by the
@@ -59,7 +56,7 @@ class TestingSpellingServiceClient : public SpellingServiceClient {
                 &test_url_loader_factory_)) {
     SetURLLoaderFactoryForTesting(test_shared_loader_factory_);
   }
-  ~TestingSpellingServiceClient() {}
+  ~TestingSpellingServiceClient() = default;
 
   void SetExpectedTextCheckResult(bool success,
                                   const std::string& sanitized_request_text,
@@ -70,10 +67,10 @@ class TestingSpellingServiceClient : public SpellingServiceClient {
   }
 
   void VerifyResponse(bool success,
-                      const base::string16& request_text,
+                      const std::u16string& request_text,
                       const std::vector<SpellCheckResult>& results) {
     EXPECT_EQ(success_, success);
-    base::string16 text(base::UTF8ToUTF16(sanitized_request_text_));
+    std::u16string text(base::UTF8ToUTF16(sanitized_request_text_));
     for (auto it = results.begin(); it != results.end(); ++it) {
       text.replace(it->location, it->length, it->replacements[0]);
     }
@@ -92,7 +89,7 @@ class TestingSpellingServiceClient : public SpellingServiceClient {
  private:
   bool success_;
   std::string sanitized_request_text_;
-  base::string16 corrected_text_;
+  std::u16string corrected_text_;
   network::TestURLLoaderFactory test_url_loader_factory_;
   scoped_refptr<network::SharedURLLoaderFactory> test_shared_loader_factory_;
 };
@@ -105,20 +102,22 @@ class SpellingServiceClientTest
  public:
   void OnTextCheckComplete(int tag,
                            bool success,
-                           const base::string16& text,
+                           const std::u16string& text,
                            const std::vector<SpellCheckResult>& results) {
     client_.VerifyResponse(success, text, results);
   }
 
  protected:
   bool GetExpectedCountry(const std::string& language, std::string* country) {
-    static const struct {
+    struct Countries {
       const char* language;
       const char* country;
-    } kCountries[] = {
-        {"af", "ZAF"}, {"en", "USA"},
     };
-    for (size_t i = 0; i < base::size(kCountries); ++i) {
+    static const auto kCountries = std::to_array<Countries>({
+        {"af", "ZAF"},
+        {"en", "USA"},
+    });
+    for (size_t i = 0; i < std::size(kCountries); ++i) {
       if (!language.compare(kCountries[i].language)) {
         country->assign(kCountries[i].country);
         return true;
@@ -127,10 +126,9 @@ class SpellingServiceClientTest
     return false;
   }
 
-  content::TestBrowserThreadBundle thread_bundle_;
+  content::BrowserTaskEnvironment task_environment_;
   TestingSpellingServiceClient client_;
   TestingProfile profile_;
-  base::test::ScopedFeatureList scoped_feature_list_;
 };
 
 }  // namespace
@@ -147,17 +145,11 @@ class SpellingServiceClientTest
 // misspelled words with ones suggested by the service so this test can compare
 // the corrected text with the expected results. (If there are not any
 // misspelled words, |corrected_text| should be equal to |request_text|.)
-using Redirects =
-    std::vector<std::pair<net::RedirectInfo, network::ResourceResponseHead>>;
+using Redirects = std::vector<
+    std::pair<net::RedirectInfo, network::mojom::URLResponseHeadPtr>>;
 
 TEST_P(SpellingServiceClientTest, RequestTextCheck) {
   auto test_case = GetParam();
-  bool is_rest = test_case.restEndpoint;
-
-  if (is_rest) {
-    scoped_feature_list_.InitAndEnableFeature(
-        spellcheck::kSpellingServiceRestApi);
-  }
 
   PrefService* pref = profile_.GetPrefs();
   pref->SetBoolean(spellcheck::prefs::kSpellCheckEnable, true);
@@ -166,19 +158,20 @@ TEST_P(SpellingServiceClientTest, RequestTextCheck) {
   client_.test_url_loader_factory()->ClearResponses();
   net::HttpStatusCode http_status = test_case.response_status;
 
-  network::ResourceResponseHead head;
+  auto head = network::mojom::URLResponseHead::New();
   std::string headers(base::StringPrintf(
       "HTTP/1.1 %d %s\nContent-type: application/json\n\n",
       static_cast<int>(http_status), net::GetHttpReasonPhrase(http_status)));
-  head.headers = base::MakeRefCounted<net::HttpResponseHeaders>(
+  head->headers = base::MakeRefCounted<net::HttpResponseHeaders>(
       net::HttpUtil::AssembleRawHeaders(headers));
-  head.mime_type = "application/json";
+  head->mime_type = "application/json";
 
   network::URLLoaderCompletionStatus status;
   status.decoded_body_length = test_case.response_data.size();
   GURL expected_request_url = client_.BuildEndpointUrl(test_case.request_type);
   client_.test_url_loader_factory()->AddResponse(
-      expected_request_url, head, test_case.response_data, status, Redirects(),
+      expected_request_url, std::move(head), test_case.response_data, status,
+      Redirects(),
       network::TestURLLoaderFactory::ResponseProduceFlags::
           kSendHeadersOnNetworkError);
 
@@ -195,182 +188,54 @@ TEST_P(SpellingServiceClientTest, RequestTextCheck) {
                                      test_case.sanitized_request_text,
                                      test_case.corrected_text);
 
-  base::ListValue dictionary;
-  dictionary.AppendString(test_case.language);
-  pref->Set(spellcheck::prefs::kSpellCheckDictionaries, dictionary);
+  base::Value::List dictionary;
+  dictionary.Append(test_case.language);
+  pref->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                std::move(dictionary));
 
   client_.RequestTextCheck(
       &profile_, test_case.request_type,
       base::WideToUTF16(test_case.request_text),
       base::BindOnce(&SpellingServiceClientTest::OnTextCheckComplete,
                      base::Unretained(this), 0));
-  thread_bundle_.RunUntilIdle();
+  task_environment_.RunUntilIdle();
 
   // Verify that the expected endpoint was hit (REST vs RPC).
   ASSERT_EQ(requested_url.path(), expected_request_url.path());
 
   // Verify the request content type was JSON. (The Spelling service returns
   // an internal server error when this content type is not JSON.)
-  std::string request_content_type;
-  ASSERT_TRUE(intercepted_headers.GetHeader(
-      net::HttpRequestHeaders::kContentType, &request_content_type));
-  EXPECT_EQ("application/json", request_content_type);
+  EXPECT_EQ("application/json", intercepted_headers.GetHeader(
+                                    net::HttpRequestHeaders::kContentType));
 
   // Parse the JSON sent to the service, and verify its parameters.
-  std::unique_ptr<base::DictionaryValue> value(
-      static_cast<base::DictionaryValue*>(
-          base::JSONReader::ReadDeprecated(intercepted_body,
-                                           base::JSON_ALLOW_TRAILING_COMMAS)
-              .release()));
-  ASSERT_TRUE(value.get());
+  std::optional<base::Value> value = base::JSONReader::Read(
+      intercepted_body, base::JSON_ALLOW_TRAILING_COMMAS);
+  ASSERT_TRUE(value);
+  ASSERT_TRUE(value->is_dict());
+  const base::Value::Dict& dict = value->GetDict();
 
-  std::string method;
-  std::string version;
-  if (is_rest) {
-    EXPECT_FALSE(value->GetString("method", &method));
-    EXPECT_FALSE(value->GetString("apiVersion", &version));
-  } else {
-    EXPECT_TRUE(value->GetString("method", &method));
-    EXPECT_EQ("spelling.check", method);
-    EXPECT_TRUE(value->GetString("apiVersion", &version));
-    EXPECT_EQ(base::StringPrintf("v%d", test_case.request_type), version);
-  }
-
-  std::string sanitized_text;
-  EXPECT_TRUE(
-      value->GetString(is_rest ? "text" : "params.text", &sanitized_text));
-  EXPECT_EQ(test_case.sanitized_request_text, sanitized_text);
-  std::string language;
-  EXPECT_TRUE(
-      value->GetString(is_rest ? "language" : "params.language", &language));
+  EXPECT_FALSE(dict.FindString("method"));
+  EXPECT_FALSE(dict.FindString("apiVersion"));
+  const std::string* sanitized_text = dict.FindString("text");
+  EXPECT_TRUE(sanitized_text);
+  EXPECT_EQ(test_case.sanitized_request_text, *sanitized_text);
+  const std::string* language = dict.FindString("language");
+  EXPECT_TRUE(language);
   std::string expected_language =
       test_case.language.empty() ? std::string("en") : test_case.language;
-  EXPECT_EQ(expected_language, language);
+  EXPECT_EQ(expected_language, *language);
   std::string expected_country;
-  ASSERT_TRUE(GetExpectedCountry(language, &expected_country));
-  std::string country;
-  EXPECT_TRUE(value->GetString(
-      is_rest ? "originCountry" : "params.originCountry", &country));
-  EXPECT_EQ(expected_country, country);
+  ASSERT_TRUE(GetExpectedCountry(*language, &expected_country));
+  const std::string* country = dict.FindString("originCountry");
+  EXPECT_TRUE(country);
+  EXPECT_EQ(expected_country, *country);
 }
 
 INSTANTIATE_TEST_SUITE_P(
     SpellingService,
     SpellingServiceClientTest,
     testing::Values(
-        // Test cases for the RPC endpoint
-        SpellingServiceTestCase{
-            L"",
-            "",
-            SpellingServiceClient::SUGGEST,
-            net::HttpStatusCode(500),
-            "",
-            false,
-            "",
-            "af",
-            false,
-        },
-        SpellingServiceTestCase{
-            L"chromebook",
-            "chromebook",
-            SpellingServiceClient::SUGGEST,
-            net::HttpStatusCode(200),
-            "{}",
-            true,
-            "chromebook",
-            "af",
-            false,
-        },
-        SpellingServiceTestCase{
-            L"chrombook",
-            "chrombook",
-            SpellingServiceClient::SUGGEST,
-            net::HttpStatusCode(200),
-            "{\n"
-            "  \"result\": {\n"
-            "    \"spellingCheckResponse\": {\n"
-            "      \"misspellings\": [{\n"
-            "        \"charStart\": 0,\n"
-            "        \"charLength\": 9,\n"
-            "        \"suggestions\": [{ \"suggestion\": \"chromebook\" }],\n"
-            "        \"canAutoCorrect\": false\n"
-            "      }]\n"
-            "    }\n"
-            "  }\n"
-            "}",
-            true,
-            "chromebook",
-            "af",
-            false,
-        },
-        SpellingServiceTestCase{
-            L"",
-            "",
-            SpellingServiceClient::SPELLCHECK,
-            net::HttpStatusCode(500),
-            "",
-            false,
-            "",
-            "en",
-            false,
-        },
-        SpellingServiceTestCase{
-            L"I have been to USA.",
-            "I have been to USA.",
-            SpellingServiceClient::SPELLCHECK,
-            net::HttpStatusCode(200),
-            "{}",
-            true,
-            "I have been to USA.",
-            "en",
-            false,
-        },
-        SpellingServiceTestCase{
-            L"I have bean to USA.",
-            "I have bean to USA.",
-            SpellingServiceClient::SPELLCHECK,
-            net::HttpStatusCode(200),
-            "{\n"
-            "  \"result\": {\n"
-            "    \"spellingCheckResponse\": {\n"
-            "      \"misspellings\": [{\n"
-            "        \"charStart\": 7,\n"
-            "        \"charLength\": 4,\n"
-            "        \"suggestions\": [{ \"suggestion\": \"been\" }],\n"
-            "        \"canAutoCorrect\": false\n"
-            "      }]\n"
-            "    }\n"
-            "  }\n"
-            "}",
-            true,
-            "I have been to USA.",
-            "en",
-            false,
-        },
-        SpellingServiceTestCase{
-            L"I\x2019mattheIn'n'Out.",
-            "I'mattheIn'n'Out.",
-            SpellingServiceClient::SPELLCHECK,
-            net::HttpStatusCode(200),
-            "{\n"
-            "  \"result\": {\n"
-            "    \"spellingCheckResponse\": {\n"
-            "      \"misspellings\": [{\n"
-            "        \"charStart\": 0,\n"
-            "        \"charLength\": 16,\n"
-            "        \"suggestions\":"
-            " [{ \"suggestion\": \"I'm at the In'N'Out\" }],\n"
-            "        \"canAutoCorrect\": false\n"
-            "      }]\n"
-            "    }\n"
-            "  }\n"
-            "}",
-            true,
-            "I'm at the In'N'Out.",
-            "en",
-            false,
-        },
-
         // Test cases for the REST endpoint
         SpellingServiceTestCase{
             L"",
@@ -381,7 +246,6 @@ INSTANTIATE_TEST_SUITE_P(
             false,
             "",
             "af",
-            true,
         },
         SpellingServiceTestCase{
             L"chromebook",
@@ -392,7 +256,6 @@ INSTANTIATE_TEST_SUITE_P(
             true,
             "chromebook",
             "af",
-            true,
         },
         SpellingServiceTestCase{
             L"chrombook",
@@ -412,11 +275,17 @@ INSTANTIATE_TEST_SUITE_P(
             true,
             "chromebook",
             "af",
-            true,
         },
-        SpellingServiceTestCase{L"", "", SpellingServiceClient::SPELLCHECK,
-                                net::HttpStatusCode(500), "", false, "", "en",
-                                true},
+        SpellingServiceTestCase{
+            L"",
+            "",
+            SpellingServiceClient::SPELLCHECK,
+            net::HttpStatusCode(500),
+            "",
+            false,
+            "",
+            "en",
+        },
         SpellingServiceTestCase{
             L"I have been to USA.",
             "I have been to USA.",
@@ -426,7 +295,6 @@ INSTANTIATE_TEST_SUITE_P(
             true,
             "I have been to USA.",
             "en",
-            true,
         },
         SpellingServiceTestCase{
             L"I have bean to USA.",
@@ -446,7 +314,6 @@ INSTANTIATE_TEST_SUITE_P(
             true,
             "I have been to USA.",
             "en",
-            true,
         },
         SpellingServiceTestCase{
             L"I\x2019mattheIn'n'Out.",
@@ -467,7 +334,6 @@ INSTANTIATE_TEST_SUITE_P(
             true,
             "I'm at the In'N'Out.",
             "en",
-            true,
         }));
 
 // Verify that SpellingServiceClient::IsAvailable() returns true only when it
@@ -494,20 +360,27 @@ TEST_F(SpellingServiceClientTest, AvailableServices) {
   // SpellingServiceClient::IsAvailable() describes why this function returns
   // false for suggestions.) If there is no language set, then we
   // do not allow any remote.
-  pref->Set(spellcheck::prefs::kSpellCheckDictionaries, base::ListValue());
+  pref->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                base::Value::List());
 
   EXPECT_FALSE(client_.IsAvailable(&profile_, kSuggest));
   EXPECT_FALSE(client_.IsAvailable(&profile_, kSpellcheck));
 
-  static const char* kSupported[] = {
-      "en-AU", "en-CA", "en-GB", "en-US", "da-DK", "es-ES",
-  };
+  constexpr static const auto kSupported = std::to_array<const char*>({
+      "en-AU",
+      "en-CA",
+      "en-GB",
+      "en-US",
+      "da-DK",
+      "es-ES",
+  });
   // If spellcheck is allowed, then suggest is not since spellcheck is a
   // superset of suggest.
-  for (size_t i = 0; i < base::size(kSupported); ++i) {
-    base::ListValue dictionary;
-    dictionary.AppendString(kSupported[i]);
-    pref->Set(spellcheck::prefs::kSpellCheckDictionaries, dictionary);
+  for (size_t i = 0; i < std::size(kSupported); ++i) {
+    base::Value::List dictionary;
+    dictionary.Append(kSupported[i]);
+    pref->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                  std::move(dictionary));
 
     EXPECT_FALSE(client_.IsAvailable(&profile_, kSuggest));
     EXPECT_TRUE(client_.IsAvailable(&profile_, kSpellcheck));
@@ -515,17 +388,18 @@ TEST_F(SpellingServiceClientTest, AvailableServices) {
 
   // This function returns true for suggestions for all and false for
   // spellcheck for unsupported locales.
-  static const char* kUnsupported[] = {
+  constexpr static const auto kUnsupported = std::to_array<const char*>({
       "af-ZA", "bg-BG", "ca-ES", "cs-CZ", "de-DE", "el-GR", "et-EE", "fo-FO",
       "fr-FR", "he-IL", "hi-IN", "hr-HR", "hu-HU", "id-ID", "it-IT", "lt-LT",
       "lv-LV", "nb-NO", "nl-NL", "pl-PL", "pt-BR", "pt-PT", "ro-RO", "ru-RU",
       "sk-SK", "sl-SI", "sh",    "sr",    "sv-SE", "tr-TR", "uk-UA", "vi-VN",
-  };
-  for (size_t i = 0; i < base::size(kUnsupported); ++i) {
+  });
+  for (size_t i = 0; i < std::size(kUnsupported); ++i) {
     SCOPED_TRACE(std::string("Expected language ") + kUnsupported[i]);
-    base::ListValue dictionary;
-    dictionary.AppendString(kUnsupported[i]);
-    pref->Set(spellcheck::prefs::kSpellCheckDictionaries, dictionary);
+    base::Value::List dictionary;
+    dictionary.Append(kUnsupported[i]);
+    pref->SetList(spellcheck::prefs::kSpellCheckDictionaries,
+                  std::move(dictionary));
 
     EXPECT_TRUE(client_.IsAvailable(&profile_, kSuggest));
     EXPECT_FALSE(client_.IsAvailable(&profile_, kSpellcheck));
