@@ -40,8 +40,10 @@
 #include "components/version_info/version_info.h"
 #include "content/public/browser/storage_partition.h"
 #include "google_apis/gaia/gaia_auth_util.h"
+#include "google_apis/gaia/gaia_id.h"
 #include "google_apis/gaia/gaia_urls.h"
 #include "net/base/net_errors.h"
+#include "third_party/abseil-cpp/absl/cleanup/cleanup.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/display/display.h"
 #include "ui/display/screen.h"
@@ -112,10 +114,14 @@ LockScreenReauthHandler::LockScreenReauthHandler(const std::string& email)
 
 LockScreenReauthHandler::~LockScreenReauthHandler() = default;
 
-void LockScreenReauthHandler::HandleInitialize(const base::Value::List& value) {
+void LockScreenReauthHandler::HandleStartOnlineAuth(
+    const base::Value::List& value) {
   AllowJavascript();
   OnReauthDialogReadyForTesting();
-  LoadAuthenticatorParam();
+
+  CHECK_EQ(1u, value.size());
+  const bool force_reauth_gaia_page = value[0].GetBool();
+  LoadAuthenticatorParam(force_reauth_gaia_page);
 }
 
 void LockScreenReauthHandler::HandleAuthenticatorLoaded(
@@ -127,13 +133,27 @@ void LockScreenReauthHandler::HandleAuthenticatorLoaded(
     std::move(waiting_caller_).Run();
   }
 
+  ActivateAutoReload();
+
   // Recreate the client cert usage observer, in order to track only the certs
   // used during the current sign-in attempt.
   extension_provided_client_cert_usage_observer_ =
       std::make_unique<LoginClientCertUsageObserver>();
 }
 
-void LockScreenReauthHandler::LoadAuthenticatorParam() {
+void LockScreenReauthHandler::ActivateAutoReload() {
+  auth_flow_auto_reload_manager_.Activate(base::BindOnce(
+      &LockScreenReauthHandler::LoadAuthenticatorParam,
+      weak_factory_.GetWeakPtr(), /*force_reauth_gaia_page=*/false));
+}
+
+ash::AuthenticationFlowAutoReloadManager&
+LockScreenReauthHandler::GetAutoReloadManager() {
+  return auth_flow_auto_reload_manager_;
+}
+
+void LockScreenReauthHandler::LoadAuthenticatorParam(
+    const bool force_reauth_gaia_page) {
   if (authenticator_state_ == AuthenticatorState::LOADING) {
     VLOG(1) << "Skip loading the Authenticator as it's already being loaded ";
     return;
@@ -153,10 +173,11 @@ void LockScreenReauthHandler::LoadAuthenticatorParam() {
         AccountId::FromUserEmail(gaia::CanonicalizeEmail(context.email)));
   }
 
-  LoadGaia(context);
+  LoadGaia(context, force_reauth_gaia_page);
 }
 
-void LockScreenReauthHandler::LoadGaia(const login::GaiaContext& context) {
+void LockScreenReauthHandler::LoadGaia(const login::GaiaContext& context,
+                                       const bool force_reauth_gaia_page) {
   LOG_ASSERT(Profile::FromWebUI(web_ui()) ==
              ProfileHelper::Get()->GetLockScreenProfile());
   // Start a new session with SigninPartitionManager, generating a unique
@@ -170,15 +191,18 @@ void LockScreenReauthHandler::LoadGaia(const login::GaiaContext& context) {
   signin_partition_manager->StartSigninSession(
       web_ui()->GetWebContents(),
       base::BindOnce(&LockScreenReauthHandler::LoadGaiaWithPartition,
-                     weak_factory_.GetWeakPtr(), context));
+                     weak_factory_.GetWeakPtr(), context,
+                     force_reauth_gaia_page));
 }
 
 void LockScreenReauthHandler::LoadGaiaWithPartition(
     const login::GaiaContext& context,
+    const bool force_reauth_gaia_page,
     const std::string& partition_name) {
   auto callback = base::BindOnce(
       &LockScreenReauthHandler::OnSetCookieForLoadGaiaWithPartition,
-      weak_factory_.GetWeakPtr(), context, partition_name);
+      weak_factory_.GetWeakPtr(), context, force_reauth_gaia_page,
+      partition_name);
   if (context.gaps_cookie.empty()) {
     std::move(callback).Run(net::CookieAccessResult());
     return;
@@ -197,6 +221,7 @@ void LockScreenReauthHandler::LoadGaiaWithPartition(
 
 void LockScreenReauthHandler::OnSetCookieForLoadGaiaWithPartition(
     const login::GaiaContext& context,
+    const bool force_reauth_gaia_page,
     const std::string& partition_name,
     net::CookieAccessResult result) {
   base::Value::Dict params;
@@ -208,7 +233,8 @@ void LockScreenReauthHandler::OnSetCookieForLoadGaiaWithPartition(
   params.Set("gaiaUrl", gaia_urls.gaia_url().spec());
   params.Set("clientId", gaia_urls.oauth2_chrome_client_id());
 
-  bool do_saml_redirect = ShouldDoSamlRedirect(context.email);
+  bool do_saml_redirect =
+      !force_reauth_gaia_page && ShouldDoSamlRedirect(context.email);
   params.Set("doSamlRedirect", do_saml_redirect);
 
   // Path without the leading slash, as expected by authenticator.js.
@@ -242,7 +268,7 @@ void LockScreenReauthHandler::OnSetCookieForLoadGaiaWithPartition(
   DCHECK(!app_locale.empty());
   params.Set("hl", app_locale);
   params.Set("email", context.email);
-  params.Set("gaiaId", context.gaia_id);
+  params.Set("gaiaId", context.gaia_id.ToString());
   params.Set("extractSamlPasswordAttributes",
              login::ExtractSamlPasswordAttributesEnabled());
   params.Set("clientVersion", version_info::GetVersionNumber());
@@ -255,10 +281,10 @@ void LockScreenReauthHandler::OnSetCookieForLoadGaiaWithPartition(
         local_state->GetString(prefs::kUrlParameterToAutofillSAMLUsername));
   }
 
+  // TODO(crbug.com/377862442) Add autoreload url param.
+
   CallJavascript("loadAuthenticator", params);
-  if (features::IsNewLockScreenReauthLayoutEnabled()) {
-    UpdateOrientationAndWidth();
-  }
+  UpdateOrientationAndWidth();
 }
 
 void LockScreenReauthHandler::UpdateOrientationAndWidth() {
@@ -278,12 +304,15 @@ void LockScreenReauthHandler::CallJavascript(const std::string& function,
 
 void LockScreenReauthHandler::HandleCompleteAuthentication(
     const base::Value::List& params) {
+  absl::Cleanup run_callback_on_return = [this] {
+    auth_flow_auto_reload_manager_.Terminate();
+  };
+
   CHECK_EQ(params.size(), 7u);
-  std::string gaia_id, email, password;
   bool using_saml;
-  gaia_id = params[0].GetString();
-  email = params[1].GetString();
-  password = params[2].GetString();
+  GaiaId gaia_id(params[0].GetString());
+  std::string email = params[1].GetString();
+  std::string password = params[2].GetString();
   auto scraped_saml_passwords =
       ::login::ConvertToStringList(params[3].GetList());
   using_saml = params[4].GetBool();
@@ -306,8 +335,7 @@ void LockScreenReauthHandler::HandleCompleteAuthentication(
     auto challenge_response_key_or_error = login::ExtractClientCertificates(
         *extension_provided_client_cert_usage_observer_);
     if (!challenge_response_key_or_error.has_value()) {
-      NOTREACHED_IN_MIGRATION();
-      return;
+      NOTREACHED();
     }
     challenge_response_key = challenge_response_key_or_error.value();
   }
@@ -357,9 +385,7 @@ void LockScreenReauthHandler::FinishAuthentication(
 }
 
 void LockScreenReauthHandler::OnCookieWaitTimeout() {
-  NOTREACHED_IN_MIGRATION()
-      << "Cookie has timed out while attempting to login in.";
-  LockScreenStartReauthDialog::Dismiss();
+  NOTREACHED() << "Cookie has timed out while attempting to login in.";
 }
 
 void LockScreenReauthHandler::OnReauthDialogReadyForTesting() {
@@ -484,14 +510,14 @@ void LockScreenReauthHandler::HandleGetDeviceId(
   ResolveJavascriptCallback(callback_id, GetDeviceId(known_user));
 }
 
-void LockScreenReauthHandler::ReloadGaia() {
+void LockScreenReauthHandler::ReloadGaiaAuthenticator() {
   CallJavascriptFunction(std::string(kMainElement) + "reloadAuthenticator");
 }
 
 void LockScreenReauthHandler::RegisterMessages() {
   web_ui()->RegisterMessageCallback(
-      "initialize",
-      base::BindRepeating(&LockScreenReauthHandler::HandleInitialize,
+      "startOnlineAuth",
+      base::BindRepeating(&LockScreenReauthHandler::HandleStartOnlineAuth,
                           weak_factory_.GetWeakPtr()));
 
   web_ui()->RegisterMessageCallback(
