@@ -26,6 +26,7 @@ class PasswordSelectionScreenView {
 
   virtual void ShowProgress() = 0;
   virtual void ShowPasswordChoice() = 0;
+  virtual void ShowBackButton() = 0;
 
   // Gets a WeakPtr to the instance.
   virtual base::WeakPtr<PasswordSelectionScreenView> AsWeakPtr() = 0;
@@ -50,6 +51,7 @@ class PasswordSelectionScreenHandler final : public PasswordSelectionScreenView,
   void Show() override;
   void ShowProgress() override;
   void ShowPasswordChoice() override;
+  void ShowBackButton() override;
   base::WeakPtr<PasswordSelectionScreenView> AsWeakPtr() override;
 
   // BaseScreenHandler:
