@@ -6,6 +6,7 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <string>
 
 #include "base/base64.h"
@@ -28,16 +29,15 @@
 #include "mojo/public/cpp/bindings/remote.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 using ::testing::_;
 
 namespace ash {
 
-class FakeParentAccessUIHandlerDelegate : public ParentAccessUIHandlerDelegate {
+class FakeParentAccessUiHandlerDelegate : public ParentAccessUiHandlerDelegate {
  public:
-  FakeParentAccessUIHandlerDelegate() = default;
-  ~FakeParentAccessUIHandlerDelegate() = default;
+  FakeParentAccessUiHandlerDelegate() = default;
+  ~FakeParentAccessUiHandlerDelegate() = default;
 
   parent_access_ui::mojom::ParentAccessParamsPtr CloneParentAccessParams()
       override {
@@ -78,35 +78,35 @@ class FakeParentAccessUIHandlerDelegate : public ParentAccessUIHandlerDelegate {
   bool is_disabled_ = false;
 };
 
-class ParentAccessUIHandlerImplBaseTest : public testing::Test {
+class ParentAccessUiHandlerImplBaseTest : public testing::Test {
  public:
-  ParentAccessUIHandlerImplBaseTest() {
+  ParentAccessUiHandlerImplBaseTest() {
     identity_test_env_ = std::make_unique<signin::IdentityTestEnvironment>();
     identity_test_env_->MakePrimaryAccountAvailable(
         "testuser@gmail.com", signin::ConsentLevel::kSync);
   }
 
-  ~ParentAccessUIHandlerImplBaseTest() override = default;
+  ~ParentAccessUiHandlerImplBaseTest() override = default;
 
   void TearDown() override { parent_access_ui_handler_.reset(); }
 
  protected:
   base::test::TaskEnvironment task_environment_;
   std::unique_ptr<signin::IdentityTestEnvironment> identity_test_env_;
-  mojo::Remote<parent_access_ui::mojom::ParentAccessUIHandler>
+  mojo::Remote<parent_access_ui::mojom::ParentAccessUiHandler>
       parent_access_ui_handler_remote_;
-  std::unique_ptr<ParentAccessUIHandlerImpl> parent_access_ui_handler_;
-  FakeParentAccessUIHandlerDelegate delegate_;
+  std::unique_ptr<ParentAccessUiHandlerImpl> parent_access_ui_handler_;
+  FakeParentAccessUiHandlerDelegate delegate_;
 };
 
-class ParentAccessUIHandlerImplTestParameterized
-    : public ParentAccessUIHandlerImplBaseTest,
+class ParentAccessUiHandlerImplTestParameterized
+    : public ParentAccessUiHandlerImplBaseTest,
       public testing::WithParamInterface<
           parent_access_ui::mojom::ParentAccessParams::FlowType> {
  public:
-  ParentAccessUIHandlerImplTestParameterized() {
+  ParentAccessUiHandlerImplTestParameterized() {
     delegate_.set_flow_type(GetTestedFlowType());
-    parent_access_ui_handler_ = std::make_unique<ParentAccessUIHandlerImpl>(
+    parent_access_ui_handler_ = std::make_unique<ParentAccessUiHandlerImpl>(
         parent_access_ui_handler_remote_.BindNewPipeAndPassReceiver(),
         identity_test_env_->identity_manager(), &delegate_);
   }
@@ -130,16 +130,16 @@ class ParentAccessUIHandlerImplTestParameterized
 
 INSTANTIATE_TEST_SUITE_P(
     All,
-    ParentAccessUIHandlerImplTestParameterized,
+    ParentAccessUiHandlerImplTestParameterized,
     testing::Values(
         parent_access_ui::mojom::ParentAccessParams::FlowType::kWebsiteAccess,
         parent_access_ui::mojom::ParentAccessParams::FlowType::
             kExtensionAccess));
 
 // Verifies that the webview URL is properly constructed
-TEST_P(ParentAccessUIHandlerImplTestParameterized, GetParentAccessURL) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, GetParentAccessUrl) {
   base::RunLoop run_loop;
-  parent_access_ui_handler_->GetParentAccessURL(
+  parent_access_ui_handler_->GetParentAccessUrl(
       base::BindLambdaForTesting([&](const std::string& url) -> void {
         GURL webview_url(url);
         ASSERT_TRUE(webview_url.has_query());
@@ -150,8 +150,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, GetParentAccessURL) {
         url::Component key;
         url::Component value;
         std::map<std::string, std::string> query_parts;
-        while (url::ExtractQueryKeyValue(query_str.c_str(), &query, &key,
-                                         &value)) {
+        while (url::ExtractQueryKeyValue(query_str, &query, &key, &value)) {
           query_parts[query_str.substr(key.begin, key.len)] =
               query_str.substr(value.begin, value.len);
         }
@@ -177,13 +176,13 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, GetParentAccessURL) {
 }
 
 // Verify that the access token is successfully fetched.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, GetOAuthTokenSuccess) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, GetOauthTokenSuccess) {
   identity_test_env_->SetAutomaticIssueOfAccessTokens(true);
   base::RunLoop run_loop;
-  parent_access_ui_handler_->GetOAuthToken(base::BindLambdaForTesting(
-      [&](parent_access_ui::mojom::GetOAuthTokenStatus status,
+  parent_access_ui_handler_->GetOauthToken(base::BindLambdaForTesting(
+      [&](parent_access_ui::mojom::GetOauthTokenStatus status,
           const std::string& token) -> void {
-        EXPECT_EQ(parent_access_ui::mojom::GetOAuthTokenStatus::kSuccess,
+        EXPECT_EQ(parent_access_ui::mojom::GetOauthTokenStatus::kSuccess,
                   status);
         run_loop.Quit();
       }));
@@ -191,13 +190,13 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, GetOAuthTokenSuccess) {
 }
 
 // Verifies that access token fetch errors are recorded.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, GetOAuthTokenError) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, GetOauthTokenError) {
   base::HistogramTester histogram_tester;
   base::RunLoop run_loop;
-  parent_access_ui_handler_->GetOAuthToken(base::BindLambdaForTesting(
-      [&](parent_access_ui::mojom::GetOAuthTokenStatus status,
+  parent_access_ui_handler_->GetOauthToken(base::BindLambdaForTesting(
+      [&](parent_access_ui::mojom::GetOauthTokenStatus status,
           const std::string& token) -> void {
-        EXPECT_EQ(parent_access_ui::mojom::GetOAuthTokenStatus::kError, status);
+        EXPECT_EQ(parent_access_ui::mojom::GetOauthTokenStatus::kError, status);
         run_loop.Quit();
       }));
 
@@ -211,27 +210,27 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, GetOAuthTokenError) {
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kOAuthError, 1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kOAuthError, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
           parent_access::kParentAccessWidgetErrorHistogramBase,
           GetTestedFlowType()),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kOAuthError, 1);
+      supervised_user::ParentAccessWidgetError::kOAuthError, 1);
 }
 
 // Verifies that only one access token fetch is possible at a time.
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
-       GetOAuthTokenOnlyOneFetchAtATimeError) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
+       GetOauthTokenOnlyOneFetchAtATimeError) {
   identity_test_env_->SetAutomaticIssueOfAccessTokens(false);
-  parent_access_ui_handler_->GetOAuthToken(base::DoNothing());
+  parent_access_ui_handler_->GetOauthToken(base::DoNothing());
 
   base::RunLoop one_fetch_run_loop;
-  parent_access_ui_handler_->GetOAuthToken(base::BindLambdaForTesting(
-      [&](parent_access_ui::mojom::GetOAuthTokenStatus status,
+  parent_access_ui_handler_->GetOauthToken(base::BindLambdaForTesting(
+      [&](parent_access_ui::mojom::GetOauthTokenStatus status,
           const std::string& token) -> void {
         EXPECT_EQ(
-            parent_access_ui::mojom::GetOAuthTokenStatus::kOnlyOneFetchAtATime,
+            parent_access_ui::mojom::GetOauthTokenStatus::kOnlyOneFetchAtATime,
             status);
         one_fetch_run_loop.Quit();
       }));
@@ -247,7 +246,7 @@ MATCHER_P(EqualsProto,
 }
 
 // Verifies that the parent approvals sequence is handled correctly.
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
        OnParentVerifiedAndApproved) {
   base::HistogramTester histogram_tester;
   // Construct the ParentAccessCallback
@@ -265,12 +264,12 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
   expire_time->set_nanos(567890);
 
   // Encode the proto in base64.
-  std::string encoded_parent_access_callback;
-  base::Base64Encode(parent_access_callback.SerializeAsString(),
-                     &encoded_parent_access_callback);
+  std::string encoded_parent_access_callback =
+      base::Base64Encode(parent_access_callback.SerializeAsString());
 
-  EXPECT_CALL(delegate_, SetApproved(pat->token(), base::Time::FromDoubleT(
-                                                       expire_time->seconds())))
+  EXPECT_CALL(delegate_,
+              SetApproved(pat->token(), base::Time::FromSecondsSinceUnixEpoch(
+                                            expire_time->seconds())))
       .Times(1);
 
   base::RunLoop run_loop;
@@ -306,7 +305,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
   parent_access_ui_handler_.reset();
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessFlowResultHistogramBase, absl::nullopt),
+          parent_access::kParentAccessFlowResultHistogramBase, std::nullopt),
       ParentAccessStateTracker::FlowResult::kAccessApproved, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
@@ -317,12 +316,11 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
 
 // Verifies that an unparsable parent access callback proto is handled
 // properly.
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
        OnInvalidParentAccessCallback) {
   // Encode the proto in base64.
-  std::string encoded_parent_access_callback;
-  base::Base64Encode("INVALID_SERIALIZED_CALLBACK",
-                     &encoded_parent_access_callback);
+  std::string encoded_parent_access_callback =
+      base::Base64Encode("INVALID_SERIALIZED_CALLBACK");
 
   EXPECT_CALL(delegate_, SetApproved(_, _)).Times(0);
 
@@ -344,7 +342,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
 
 // Verifies that non-base64 encoded data passed as a parent access callback is
 // handled properly.
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
        OnNonBase64ParentAccessCallback) {
   EXPECT_CALL(delegate_, SetApproved(_, _)).Times(0);
 
@@ -365,7 +363,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
 }
 
 // Verifies that parent declining is handled correctly.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, OnParentDeclined) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, OnParentDeclined) {
   base::HistogramTester histogram_tester;
   EXPECT_CALL(delegate_, SetDeclined()).Times(1);
 
@@ -381,7 +379,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnParentDeclined) {
   parent_access_ui_handler_.reset();
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessFlowResultHistogramBase, absl::nullopt),
+          parent_access::kParentAccessFlowResultHistogramBase, std::nullopt),
       ParentAccessStateTracker::FlowResult::kAccessDeclined, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
@@ -391,7 +389,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnParentDeclined) {
 }
 
 // Verifies canceling the UI is handled correctly.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, OnCanceled) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, OnCanceled) {
   base::HistogramTester histogram_tester;
   EXPECT_CALL(delegate_, SetCanceled()).Times(1);
 
@@ -407,7 +405,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnCanceled) {
   parent_access_ui_handler_.reset();
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessFlowResultHistogramBase, absl::nullopt),
+          parent_access::kParentAccessFlowResultHistogramBase, std::nullopt),
       GetInitialStateForFlow(), 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
@@ -417,7 +415,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnCanceled) {
 }
 
 // Verifies errors are handled correctly.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, OnError) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, OnError) {
   base::HistogramTester histogram_tester;
   EXPECT_CALL(delegate_, SetError()).Times(1);
 
@@ -433,7 +431,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnError) {
   parent_access_ui_handler_.reset();
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessFlowResultHistogramBase, absl::nullopt),
+          parent_access::kParentAccessFlowResultHistogramBase, std::nullopt),
       ParentAccessStateTracker::FlowResult::kError, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
@@ -443,7 +441,7 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnError) {
 }
 
 // Verifies that the ConsentDeclined status is ignored.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, ConsentDeclinedParsed) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, ConsentDeclinedParsed) {
   base::HistogramTester histogram_tester;
   // Construct the ParentAccessCallback
   kids::platform::parentaccess::client::proto::ParentAccessCallback
@@ -451,9 +449,8 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, ConsentDeclinedParsed) {
   parent_access_callback.mutable_on_consent_declined();
 
   // Encode the proto in base64.
-  std::string encoded_parent_access_callback;
-  base::Base64Encode(parent_access_callback.SerializeAsString(),
-                     &encoded_parent_access_callback);
+  std::string encoded_parent_access_callback =
+      base::Base64Encode(parent_access_callback.SerializeAsString());
 
   base::RunLoop run_loop;
   parent_access_ui_handler_->OnParentAccessCallbackReceived(
@@ -472,17 +469,17 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, ConsentDeclinedParsed) {
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kUnknownCallback, 1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kUnknownCallback, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
           parent_access::kParentAccessWidgetErrorHistogramBase,
           GetTestedFlowType()),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kUnknownCallback, 1);
+      supervised_user::ParentAccessWidgetError::kUnknownCallback, 1);
 }
 
 // Verifies that the OnPageSizeChanged status is ignored.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, OnPageSizeChangedIgnored) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, OnPageSizeChangedIgnored) {
   base::HistogramTester histogram_tester;
   // Construct the ParentAccessCallback
   kids::platform::parentaccess::client::proto::ParentAccessCallback
@@ -490,9 +487,8 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnPageSizeChangedIgnored) {
   parent_access_callback.mutable_on_page_size_changed();
 
   // Encode the proto in base64.
-  std::string encoded_parent_access_callback;
-  base::Base64Encode(parent_access_callback.SerializeAsString(),
-                     &encoded_parent_access_callback);
+  std::string encoded_parent_access_callback =
+      base::Base64Encode(parent_access_callback.SerializeAsString());
 
   base::RunLoop run_loop;
   parent_access_ui_handler_->OnParentAccessCallbackReceived(
@@ -511,17 +507,17 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, OnPageSizeChangedIgnored) {
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kUnknownCallback, 1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kUnknownCallback, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
           parent_access::kParentAccessWidgetErrorHistogramBase,
           GetTestedFlowType()),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kUnknownCallback, 1);
+      supervised_user::ParentAccessWidgetError::kUnknownCallback, 1);
 }
 
 // Verifies that the OnCommunicationEstablished status is ignored.
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
        OnCommunicationEstablishedIgnored) {
   base::HistogramTester histogram_tester;
 
@@ -531,9 +527,8 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
   parent_access_callback.mutable_on_communication_established();
 
   // Encode the proto in base64.
-  std::string encoded_parent_access_callback;
-  base::Base64Encode(parent_access_callback.SerializeAsString(),
-                     &encoded_parent_access_callback);
+  std::string encoded_parent_access_callback =
+      base::Base64Encode(parent_access_callback.SerializeAsString());
 
   base::RunLoop run_loop;
   parent_access_ui_handler_->OnParentAccessCallbackReceived(
@@ -552,19 +547,19 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kUnknownCallback, 1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kUnknownCallback, 1);
 }
 
 // Verifies metric is recorded for no delegate error.
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
        NoDelegateErrorMetricRecorded) {
   base::HistogramTester histogram_tester;
 
-  // Construct a ParentAccessUIHandler without a delegate.
-  mojo::Remote<parent_access_ui::mojom::ParentAccessUIHandler> remote;
+  // Construct a ParentAccessUiHandler without a delegate.
+  mojo::Remote<parent_access_ui::mojom::ParentAccessUiHandler> remote;
   auto parent_access_ui_handler_no_delegate =
-      std::make_unique<ParentAccessUIHandlerImpl>(
+      std::make_unique<ParentAccessUiHandlerImpl>(
           remote.BindNewPipeAndPassReceiver(),
           identity_test_env_->identity_manager(), nullptr);
 
@@ -579,12 +574,11 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kDelegateNotAvailable,
-      1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kDelegateNotAvailable, 1);
 }
 
-TEST_P(ParentAccessUIHandlerImplTestParameterized,
+TEST_P(ParentAccessUiHandlerImplTestParameterized,
        DecodingErrorMetricRecorded) {
   base::HistogramTester histogram_tester;
   base::RunLoop run_loop;
@@ -600,23 +594,22 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized,
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kDecodingError, 1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kDecodingError, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
           parent_access::kParentAccessWidgetErrorHistogramBase,
           GetTestedFlowType()),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kDecodingError, 1);
+      supervised_user::ParentAccessWidgetError::kDecodingError, 1);
 }
 
 // Verifies metric is recorded when received callback cannot be parsed to proto.
-TEST_P(ParentAccessUIHandlerImplTestParameterized, ParsingErrorMetricRecorded) {
+TEST_P(ParentAccessUiHandlerImplTestParameterized, ParsingErrorMetricRecorded) {
   base::HistogramTester histogram_tester;
 
   // Receive non-parseable callback.
   base::RunLoop run_loop;
-  std::string encoded_not_a_callback;
-  base::Base64Encode("not_a_callback", &encoded_not_a_callback);
+  std::string encoded_not_a_callback = base::Base64Encode("not_a_callback");
   parent_access_ui_handler_->OnParentAccessCallbackReceived(
       encoded_not_a_callback,
       base::BindLambdaForTesting(
@@ -627,17 +620,17 @@ TEST_P(ParentAccessUIHandlerImplTestParameterized, ParsingErrorMetricRecorded) {
   // Expect metric to be recorded.
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessWidgetErrorHistogramBase, absl::nullopt),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kParsingError, 1);
+          parent_access::kParentAccessWidgetErrorHistogramBase, std::nullopt),
+      supervised_user::ParentAccessWidgetError::kParsingError, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
           parent_access::kParentAccessWidgetErrorHistogramBase,
           GetTestedFlowType()),
-      ParentAccessUIHandlerImpl::ParentAccessWidgetError::kParsingError, 1);
+      supervised_user::ParentAccessWidgetError::kParsingError, 1);
 }
 
 class ExtensionApprovalsDisabledTest
-    : public ParentAccessUIHandlerImplBaseTest {
+    : public ParentAccessUiHandlerImplBaseTest {
  public:
   ExtensionApprovalsDisabledTest() {
     // Only test extensions flow because disabled state is not allowed for web
@@ -645,7 +638,7 @@ class ExtensionApprovalsDisabledTest
     delegate_.set_flow_type(parent_access_ui::mojom::ParentAccessParams::
                                 FlowType::kExtensionAccess);
     delegate_.set_is_disabled(true);
-    parent_access_ui_handler_ = std::make_unique<ParentAccessUIHandlerImpl>(
+    parent_access_ui_handler_ = std::make_unique<ParentAccessUiHandlerImpl>(
         parent_access_ui_handler_remote_.BindNewPipeAndPassReceiver(),
         identity_test_env_->identity_manager(), &delegate_);
   }
@@ -668,7 +661,7 @@ TEST_F(ExtensionApprovalsDisabledTest, OnDisabled) {
   parent_access_ui_handler_.reset();
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
-          parent_access::kParentAccessFlowResultHistogramBase, absl::nullopt),
+          parent_access::kParentAccessFlowResultHistogramBase, std::nullopt),
       ParentAccessStateTracker::FlowResult::kRequestsDisabled, 1);
   histogram_tester.ExpectUniqueSample(
       parent_access::GetHistogramTitleForFlowType(
