@@ -4,11 +4,12 @@
 
 #include "chrome/browser/ui/webui/ash/settings/pages/crostini/crostini_section.h"
 
-#include "ash/components/arc/arc_prefs.h"
+#include <array>
+
 #include "ash/constants/ash_features.h"
 #include "ash/webui/settings/public/constants/routes.mojom-forward.h"
+#include "base/containers/span.h"
 #include "base/feature_list.h"
-#include "base/no_destructor.h"
 #include "base/strings/utf_string_conversions.h"
 #include "chrome/browser/ash/bruschetta/bruschetta_util.h"
 #include "chrome/browser/ash/crostini/crostini_disk.h"
@@ -26,6 +27,7 @@
 #include "chrome/browser/ui/webui/ash/settings/search/search_tag_registry.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/generated_resources.h"
+#include "chromeos/ash/experiences/arc/arc_prefs.h"
 #include "components/policy/core/common/management/management_service.h"
 #include "components/prefs/pref_service.h"
 #include "content/public/browser/web_ui.h"
@@ -48,7 +50,6 @@ using ::chromeos::settings::mojom::kCrostiniDevelopAndroidAppsSubpagePath;
 using ::chromeos::settings::mojom::kCrostiniExtraContainersSubpagePath;
 using ::chromeos::settings::mojom::kCrostiniManageSharedFoldersSubpagePath;
 using ::chromeos::settings::mojom::kCrostiniPortForwardingSubpagePath;
-using ::chromeos::settings::mojom::kCrostiniSectionPath;
 using ::chromeos::settings::mojom::kCrostiniUsbPreferencesSubpagePath;
 using ::chromeos::settings::mojom::Section;
 using ::chromeos::settings::mojom::Setting;
@@ -57,8 +58,8 @@ using ::chromeos::settings::mojom::Subpage;
 
 namespace {
 
-const std::vector<SearchConcept>& GetCrostiniOptedInSearchConcepts() {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniOptedInSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI,
        mojom::kCrostiniDetailsSubpagePath,
        mojom::SearchResultIcon::kDeveloperTags,
@@ -107,34 +108,32 @@ const std::vector<SearchConcept>& GetCrostiniOptedInSearchConcepts() {
        {IDS_OS_SETTINGS_TAG_CROSTINI_MIC_ACCESS_ALT1,
         SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
-const std::vector<SearchConcept>& GetCrostiniOptedOutSearchConcepts(
-    mojom::Section section,
-    const char* section_path) {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniOptedOutSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI,
-       section_path,
+       mojom::kAboutChromeOsSectionPath,
        mojom::SearchResultIcon::kDeveloperTags,
        mojom::SearchResultDefaultRank::kMedium,
        mojom::SearchResultType::kSection,
-       {.section = section},
+       {.section = mojom::Section::kAboutChromeOs},
        {IDS_OS_SETTINGS_TAG_CROSTINI_ALT1, IDS_OS_SETTINGS_TAG_CROSTINI_ALT2,
         SearchConcept::kAltTagEnd}},
       {IDS_OS_SETTINGS_TAG_CROSTINI_SETUP,
-       section_path,
+       mojom::kAboutChromeOsSectionPath,
        mojom::SearchResultIcon::kDeveloperTags,
        mojom::SearchResultDefaultRank::kMedium,
        mojom::SearchResultType::kSetting,
        {.setting = mojom::Setting::kSetUpCrostini},
        {IDS_OS_SETTINGS_TAG_CROSTINI_SETUP_ALT1, SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
-const std::vector<SearchConcept>& GetCrostiniExportImportSearchConcepts() {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniExportImportSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI_BACKUP_RESTORE,
        mojom::kCrostiniBackupAndRestoreSubpagePath,
        mojom::SearchResultIcon::kPenguin,
@@ -144,11 +143,11 @@ const std::vector<SearchConcept>& GetCrostiniExportImportSearchConcepts() {
        {IDS_OS_SETTINGS_TAG_CROSTINI_BACKUP_RESTORE_ALT1,
         SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
-const std::vector<SearchConcept>& GetCrostiniAdbSideloadingSearchConcepts() {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniAdbSideloadingSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI_ANDROID_APPS_ADB,
        mojom::kCrostiniDevelopAndroidAppsSubpagePath,
        mojom::SearchResultIcon::kDeveloperTags,
@@ -166,11 +165,11 @@ const std::vector<SearchConcept>& GetCrostiniAdbSideloadingSearchConcepts() {
        {IDS_OS_SETTINGS_TAG_CROSTINI_ANDROID_APPS_ALT1,
         SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
-const std::vector<SearchConcept>& GetCrostiniPortForwardingSearchConcepts() {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniPortForwardingSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI_PORT_FORWARDING,
        mojom::kCrostiniPortForwardingSubpagePath,
        mojom::SearchResultIcon::kPenguin,
@@ -180,11 +179,11 @@ const std::vector<SearchConcept>& GetCrostiniPortForwardingSearchConcepts() {
        {IDS_OS_SETTINGS_TAG_CROSTINI_PORT_FORWARDING_ALT1,
         SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
-const std::vector<SearchConcept>& GetCrostiniContainerUpgradeSearchConcepts() {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniContainerUpgradeSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI_CONTAINER_UPGRADE,
        mojom::kCrostiniDetailsSubpagePath,
        mojom::SearchResultIcon::kPenguin,
@@ -194,11 +193,11 @@ const std::vector<SearchConcept>& GetCrostiniContainerUpgradeSearchConcepts() {
        {IDS_OS_SETTINGS_TAG_CROSTINI_CONTAINER_UPGRADE_ALT1,
         SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
-const std::vector<SearchConcept>& GetCrostiniDiskResizingSearchConcepts() {
-  static const base::NoDestructor<std::vector<SearchConcept>> tags({
+base::span<const SearchConcept> GetCrostiniDiskResizingSearchConcepts() {
+  static constexpr auto tags = std::to_array<SearchConcept>({
       {IDS_OS_SETTINGS_TAG_CROSTINI_DISK_RESIZE,
        mojom::kCrostiniDetailsSubpagePath,
        mojom::SearchResultIcon::kPenguin,
@@ -208,7 +207,7 @@ const std::vector<SearchConcept>& GetCrostiniDiskResizingSearchConcepts() {
        {IDS_OS_SETTINGS_TAG_CROSTINI_DISK_RESIZE_ALT1,
         SearchConcept::kAltTagEnd}},
   });
-  return *tags;
+  return tags;
 }
 
 bool IsProfileManaged(Profile* profile) {
@@ -262,18 +261,15 @@ bool CrostiniSection::ShouldShowBruschetta(Profile* profile) {
 }
 
 void CrostiniSection::AddLoadTimeData(content::WebUIDataSource* html_source) {
-  const bool kIsRevampEnabled =
-      ash::features::IsOsSettingsRevampWayfindingEnabled();
-
   webui::LocalizedString kLocalizedStrings[] = {
       {"bruschettaPageLabel", IDS_SETTINGS_BRUSCHETTA_LABEL},
       {"bruschettaEnable", IDS_SETTINGS_TURN_ON},
+      {"bruschettaMicDialogShutdownButton",
+       IDS_SETTINGS_BRUSCHETTA_MIC_DIALOG_SHUTDOWN_BUTTON},
       {"bruschettaRemoveButton", IDS_SETTINGS_BRUSCHETTA_REMOVE_BUTTON},
       {"crostiniPageTitle", IDS_SETTINGS_CROSTINI_TITLE},
       {"crostiniPageLabel", IDS_SETTINGS_CROSTINI_LABEL},
-      {"crostiniEnable", kIsRevampEnabled
-                             ? IDS_OS_SETTINGS_REVAMP_CROSTINI_SET_UP
-                             : IDS_SETTINGS_TURN_ON},
+      {"crostiniEnable", IDS_OS_SETTINGS_CROSTINI_SET_UP},
       {"crostiniSharedPathsInstructionsAdd",
        IDS_SETTINGS_CROSTINI_SHARED_PATHS_INSTRUCTIONS_ADD},
       {"crostiniSharedPathsRemoveFailureDialogMessage",
@@ -500,16 +496,20 @@ void CrostiniSection::AddLoadTimeData(content::WebUIDataSource* html_source) {
   html_source->AddString("bruschettaRemove",
                          l10n_util::GetStringFUTF16(
                              IDS_SETTINGS_BRUSCHETTA_REMOVE, bruschetta_name));
+  html_source->AddString(
+      "bruschettaMicTitle",
+      l10n_util::GetStringFUTF16(IDS_SETTINGS_BRUSCHETTA_MIC_TITLE,
+                                 bruschetta_name));
+  html_source->AddString(
+      "bruschettaMicDialogLabel",
+      l10n_util::GetStringFUTF16(IDS_SETTINGS_BRUSCHETTA_MIC_DIALOG_LABEL,
+                                 bruschetta_name));
 
   html_source->AddString(
       "crostiniSubtext",
-      kIsRevampEnabled
-          ? l10n_util::GetStringFUTF16(
-                IDS_OS_SETTINGS_REVAMP_CROSTINI_SUBTEXT,
-                GetHelpUrlWithBoard(chrome::kLinuxAppsLearnMoreURL))
-          : l10n_util::GetStringFUTF16(
-                IDS_SETTINGS_CROSTINI_SUBTEXT, ui::GetChromeOSDeviceName(),
-                GetHelpUrlWithBoard(chrome::kLinuxAppsLearnMoreURL)));
+      l10n_util::GetStringFUTF16(
+          IDS_OS_SETTINGS_CROSTINI_SUBTEXT,
+          GetHelpUrlWithBoard(chrome::kLinuxAppsLearnMoreURL)));
   html_source->AddString(
       "crostiniSubtextNotSupported",
       l10n_util::GetStringFUTF16(
@@ -573,9 +573,7 @@ int CrostiniSection::GetSectionNameMessageId() const {
 }
 
 mojom::Section CrostiniSection::GetSection() const {
-  return ash::features::IsOsSettingsRevampWayfindingEnabled()
-             ? mojom::Section::kAboutChromeOs
-             : mojom::Section::kCrostini;
+  return mojom::Section::kAboutChromeOs;
 }
 
 mojom::SearchResultIcon CrostiniSection::GetSectionIcon() const {
@@ -583,9 +581,7 @@ mojom::SearchResultIcon CrostiniSection::GetSectionIcon() const {
 }
 
 const char* CrostiniSection::GetSectionPath() const {
-  return ash::features::IsOsSettingsRevampWayfindingEnabled()
-             ? mojom::kAboutChromeOsSectionPath
-             : mojom::kCrostiniSectionPath;
+  return mojom::kAboutChromeOsSectionPath;
 }
 
 bool CrostiniSection::LogMetric(mojom::Setting setting,
@@ -608,6 +604,9 @@ void CrostiniSection::RegisterHierarchy(HierarchyGenerator* generator) const {
       mojom::Setting::kCrostiniDiskResize,
       mojom::Setting::kCrostiniMicAccess,
       mojom::Setting::kUninstallCrostini,
+      mojom::Setting::kBruschettaMicAccess,
+      mojom::Setting::kGuestUsbNotification,
+      mojom::Setting::kGuestUsbPersistentPassthrough,
   };
   RegisterNestedSettingBulk(mojom::Subpage::kCrostiniDetails,
                             kCrostiniDetailsSettings, generator);
@@ -712,8 +711,7 @@ void CrostiniSection::UpdateSearchTags() {
   SearchTagRegistry::ScopedTagUpdater updater = registry()->StartUpdate();
 
   updater.RemoveSearchTags(GetCrostiniOptedInSearchConcepts());
-  updater.RemoveSearchTags(
-      GetCrostiniOptedOutSearchConcepts(GetSection(), GetSectionPath()));
+  updater.RemoveSearchTags(GetCrostiniOptedOutSearchConcepts());
   updater.RemoveSearchTags(GetCrostiniExportImportSearchConcepts());
   updater.RemoveSearchTags(GetCrostiniAdbSideloadingSearchConcepts());
   updater.RemoveSearchTags(GetCrostiniPortForwardingSearchConcepts());
@@ -722,8 +720,7 @@ void CrostiniSection::UpdateSearchTags() {
 
   if (!crostini::CrostiniFeatures::Get()->IsAllowedNow(profile_) ||
       !pref_service_->GetBoolean(crostini::prefs::kCrostiniEnabled)) {
-    updater.AddSearchTags(
-        GetCrostiniOptedOutSearchConcepts(GetSection(), GetSectionPath()));
+    updater.AddSearchTags(GetCrostiniOptedOutSearchConcepts());
     return;
   }
 
