@@ -2,6 +2,8 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "chrome/browser/ui/webui/browser_command/browser_command_handler.h"
+
 #include <memory>
 
 #include "base/functional/bind.h"
@@ -10,16 +12,15 @@
 #include "base/test/metrics/histogram_tester.h"
 #include "chrome/browser/command_updater_impl.h"
 #include "chrome/browser/ui/chrome_pages.h"
-#include "chrome/browser/ui/webui/browser_command/browser_command_handler.h"
-#include "chrome/browser/user_education/user_education_service.h"
+#include "chrome/browser/user_education/tutorial_identifiers.h"
 #include "chrome/common/webui_url_constants.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/safe_browsing/core/common/safe_browsing_prefs.h"
+#include "components/saved_tab_groups/public/features.h"
 #include "components/sync_preferences/testing_pref_service_syncable.h"
-#include "components/user_education/common/help_bubble_factory_registry.h"
-#include "components/user_education/common/tutorial_identifier.h"
-#include "components/user_education/common/tutorial_registry.h"
-#include "components/user_education/common/tutorial_service.h"
+#include "components/user_education/common/help_bubble/help_bubble_factory_registry.h"
+#include "components/user_education/common/tutorial/tutorial_identifier.h"
+#include "components/user_education/common/tutorial/tutorial_registry.h"
 #include "content/public/test/browser_task_environment.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -47,6 +48,11 @@ std::vector<Command> supported_commands = {
     Command::kOpenNTPAndStartCustomizeChromeTutorial,
     Command::kStartPasswordManagerTutorial,
     Command::kStartSavedTabGroupTutorial,
+    Command::kOpenAISettings,
+    Command::kOpenSafetyCheckFromWhatsNew,
+    Command::kOpenPaymentsSettings,
+    Command::KOpenHistorySearchSettings,
+    Command::kShowCustomizeChromeToolbar,
 };
 
 const ui::ElementContext kTestContext1(1);
@@ -79,6 +85,16 @@ class TestCommandHandler : public BrowserCommandHandler {
     // cannot be executed in a unittest.
   }
 
+  void OpenAISettings() override {
+    // The functionality of opening the AI settings is removed, as it
+    // cannot be executed in a unittest.
+  }
+
+  void ShowCustomizeChromeToolbar() override {
+    // The functionality of opening the AI settings is removed, as it
+    // cannot be executed in a unittest.
+  }
+
   bool TutorialServiceExists() override { return tutorial_service_exists_; }
 
   CommandUpdater* GetCommandUpdater() override {
@@ -102,10 +118,6 @@ class TestCommandHandler : public BrowserCommandHandler {
     tab_groups_feature_supported_ = is_supported;
   }
 
-  void SetBrowserSupportsCustomizeChromeSidePanel(bool is_supported) {
-    customize_chrome_side_panel_feature_supported_ = is_supported;
-  }
-
   void SetDefaultSearchProviderToGoogle(bool is_google) {
     default_search_provider_is_google_ = is_google;
   }
@@ -114,15 +126,15 @@ class TestCommandHandler : public BrowserCommandHandler {
     saved_tab_groups_feature_supported_ = is_supported;
   }
 
+  void SetActiveTabSupportsCustomizeChrome(bool is_supported) {
+    customize_chrome_supported_ = is_supported;
+  }
+
  protected:
   bool BrowserSupportsTabGroups() override {
     return tab_groups_feature_supported_;
   }
 
-  bool BrowserSupportsCustomizeChromeSidePanel() override {
-    return customize_chrome_side_panel_feature_supported_;
-  }
-
   bool DefaultSearchProviderIsGoogle() override {
     return default_search_provider_is_google_;
   }
@@ -131,14 +143,18 @@ class TestCommandHandler : public BrowserCommandHandler {
     return saved_tab_groups_feature_supported_;
   }
 
+  bool ActiveTabSupportsCustomizeChrome() override {
+    return customize_chrome_supported_;
+  }
+
  private:
-  bool tutorial_service_exists_;
+  bool tutorial_service_exists_ = false;
   std::unique_ptr<CommandUpdater> command_updater_;
 
   bool tab_groups_feature_supported_ = true;
-  bool customize_chrome_side_panel_feature_supported_ = true;
   bool default_search_provider_is_google_ = true;
   bool saved_tab_groups_feature_supported_ = true;
+  bool customize_chrome_supported_ = true;
 };
 
 class TestTutorialService : public user_education::TutorialService {
@@ -153,11 +169,11 @@ class TestTutorialService : public user_education::TutorialService {
     return std::u16string();
   }
 
-  void StartTutorial(
-      user_education::TutorialIdentifier id,
-      ui::ElementContext context,
-      base::OnceClosure completed_callback = base::DoNothing(),
-      base::OnceClosure aborted_callback = base::DoNothing()) override {
+  void StartTutorial(user_education::TutorialIdentifier id,
+                     ui::ElementContext context,
+                     base::OnceClosure completed_callback,
+                     base::OnceClosure aborted_callback,
+                     base::RepeatingClosure restart_callback) override {
     running_id_ = id;
   }
 
@@ -183,7 +199,8 @@ class MockTutorialService : public TestTutorialService {
               (user_education::TutorialIdentifier,
                ui::ElementContext,
                base::OnceClosure,
-               base::OnceClosure));
+               base::OnceClosure,
+               base::RepeatingClosure));
   MOCK_METHOD(void,
               LogStartedFromWhatsNewPage,
               (user_education::TutorialIdentifier, bool));
@@ -205,6 +222,10 @@ class MockCommandHandler : public TestCommandHandler {
   MOCK_METHOD(void, OpenFeedbackForm, ());
 
   MOCK_METHOD(void, OpenPasswordManager, ());
+
+  MOCK_METHOD(void, OpenAISettings, ());
+
+  MOCK_METHOD(void, ShowCustomizeChromeToolbar, ());
 };
 
 class MockCommandUpdater : public CommandUpdaterImpl {
@@ -358,6 +379,21 @@ TEST_F(BrowserCommandHandlerTest, OpenSafetyCheckCommand) {
   EXPECT_TRUE(ExecuteCommand(Command::kOpenSafetyCheck, std::move(info)));
 }
 
+TEST_F(BrowserCommandHandlerTest, OpenSafetyCheckFromWhatsNewCommand) {
+  EXPECT_TRUE(CanExecuteCommand(Command::kOpenSafetyCheckFromWhatsNew));
+  // The OpenSafetyCheck command opens a new settings window with the Safety
+  // Check, and the correct disposition.
+  ClickInfoPtr info = ClickInfo::New();
+  info->middle_button = true;
+  info->meta_key = true;
+  EXPECT_CALL(
+      *command_handler_,
+      NavigateToURL(GURL(chrome::GetSettingsUrl(chrome::kSafetyCheckSubPage)),
+                    DispositionFromClick(*info)));
+  EXPECT_TRUE(
+      ExecuteCommand(Command::kOpenSafetyCheckFromWhatsNew, std::move(info)));
+}
+
 TEST_F(BrowserCommandHandlerTest,
        CanShowSafeBrowsingEnhancedProtectionCommandPromo_NoPolicies) {
   EXPECT_TRUE(
@@ -554,12 +590,6 @@ TEST_F(BrowserCommandHandlerTest,
   // Allow command to be executed.
   command_handler_->SetTutorialServiceExists(true);
 
-  // If the customize chrome side panel is not enabled, dont run the command.
-  command_handler_->SetBrowserSupportsCustomizeChromeSidePanel(false);
-  EXPECT_FALSE(
-      CanExecuteCommand(Command::kOpenNTPAndStartCustomizeChromeTutorial));
-  command_handler_->SetBrowserSupportsCustomizeChromeSidePanel(true);
-
   // If the search provider is not set to Google, dont run the command
   command_handler_->SetDefaultSearchProviderToGoogle(false);
   EXPECT_FALSE(
@@ -622,6 +652,12 @@ TEST_F(BrowserCommandHandlerTest, StartPasswordManagerTutorialCommand) {
 }
 
 TEST_F(BrowserCommandHandlerTest, StartSavedTabGroupTutorialCommand) {
+  // Skip test if Tab Groups Save V2 feature flag is enabled
+  if (tab_groups::IsTabGroupsSaveV2Enabled()) {
+    EXPECT_FALSE(CanExecuteCommand(Command::kStartSavedTabGroupTutorial));
+    GTEST_SKIP();
+  }
+
   // Command cannot be executed if the tutorial service doesn't exist.
   command_handler_->SetTutorialServiceExists(false);
   EXPECT_FALSE(CanExecuteCommand(Command::kStartSavedTabGroupTutorial));
@@ -664,3 +700,62 @@ TEST_F(BrowserCommandHandlerTest, StartSavedTabGroupTutorialCommand) {
   // Manually call tutorial started callback.
   command_handler_->OnTutorialStarted(kSavedTabGroupTutorialId, &service);
 }
+
+TEST_F(BrowserCommandHandlerTest, OpenAISettingsCommand) {
+  // By default, opening the password manager is allowed.
+  EXPECT_TRUE(CanExecuteCommand(Command::kOpenAISettings));
+  ClickInfoPtr info = ClickInfo::New();
+  info->middle_button = true;
+  info->meta_key = true;
+  // The OpenAISettings command opens a new settings window with the
+  // AI settings and the correct disposition.
+  EXPECT_CALL(*command_handler_, OpenAISettings());
+  EXPECT_TRUE(ExecuteCommand(Command::kOpenAISettings, std::move(info)));
+}
+
+TEST_F(BrowserCommandHandlerTest, OpenPaymentsSettingsCommand) {
+  // The OpenPaymentsSettings command opens a new settings window with the
+  // Payments settings sub page, and the correct disposition.
+  EXPECT_TRUE(CanExecuteCommand(Command::kOpenPaymentsSettings));
+  ClickInfoPtr info = ClickInfo::New();
+  info->middle_button = true;
+  info->meta_key = true;
+  EXPECT_CALL(
+      *command_handler_,
+      NavigateToURL(GURL(chrome::GetSettingsUrl(chrome::kPaymentsSubPage)),
+                    DispositionFromClick(*info)));
+  EXPECT_TRUE(ExecuteCommand(Command::kOpenPaymentsSettings, std::move(info)));
+}
+
+TEST_F(BrowserCommandHandlerTest, OpenHistorySearchSettingsCommand) {
+  // By default, opening the History Search subpage is allowed.
+  EXPECT_TRUE(CanExecuteCommand(Command::KOpenHistorySearchSettings));
+  ClickInfoPtr info = ClickInfo::New();
+  info->middle_button = true;
+  info->meta_key = true;
+  // The KOpenHistorySearchSettings command opens a new settings window with the
+  // History Search settings and the correct disposition.
+  EXPECT_CALL(
+      *command_handler_,
+      NavigateToURL(GURL(chrome::GetSettingsUrl(chrome::kHistorySearchSubpage)),
+                    DispositionFromClick(*info)));
+  EXPECT_TRUE(
+      ExecuteCommand(Command::KOpenHistorySearchSettings, std::move(info)));
+}
+
+TEST_F(BrowserCommandHandlerTest, ShowCustomizeChromeToolbarCommand) {
+  // If the active tab does not support customize chrome, dont run the command.
+  command_handler_->SetActiveTabSupportsCustomizeChrome(false);
+  EXPECT_FALSE(CanExecuteCommand(Command::kShowCustomizeChromeToolbar));
+
+  // If the active tab supports customize chrome it should
+  // allow running commands.
+  command_handler_->SetActiveTabSupportsCustomizeChrome(true);
+  EXPECT_TRUE(CanExecuteCommand(Command::kShowCustomizeChromeToolbar));
+
+  // Show customize chrome toolbar command calls show customize chrome toolbar.
+  ClickInfoPtr info = ClickInfo::New();
+  EXPECT_CALL(*command_handler_, ShowCustomizeChromeToolbar());
+  EXPECT_TRUE(
+      ExecuteCommand(Command::kShowCustomizeChromeToolbar, std::move(info)));
+}
