@@ -1,4 +1,4 @@
-// Copyright (c) 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,18 +8,21 @@
 
 #include <memory>
 #include <string>
+#include <utility>
 #include <vector>
 
-#include "base/bind.h"
 #include "base/command_line.h"
+#include "base/functional/bind.h"
 #include "base/lazy_instance.h"
-#include "base/macros.h"
+#include "base/memory/raw_ptr.h"
 #include "base/memory/ref_counted.h"
-#include "base/scoped_observer.h"
-#include "base/single_thread_task_runner.h"
+#include "base/memory/weak_ptr.h"
+#include "base/scoped_observation.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/values.h"
+#include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/download/download_prefs.h"
 #include "chrome/browser/net/net_export_helper.h"
@@ -29,7 +32,8 @@
 #include "chrome/browser/ui/chrome_select_file_policy.h"
 #include "chrome/common/channel_info.h"
 #include "chrome/common/url_constants.h"
-#include "components/grit/components_resources.h"
+#include "components/grit/net_export_resources.h"
+#include "components/grit/net_export_resources_map.h"
 #include "components/net_log/net_export_file_writer.h"
 #include "components/net_log/net_export_ui_constants.h"
 #include "content/public/browser/browser_thread.h"
@@ -42,9 +46,11 @@
 #include "extensions/buildflags/buildflags.h"
 #include "net/log/net_log_capture_mode.h"
 #include "ui/shell_dialogs/select_file_dialog.h"
+#include "ui/shell_dialogs/selected_file_info.h"
+#include "ui/webui/webui_util.h"
 
-#if defined(OS_ANDROID)
-#include "chrome/browser/android/intent_helper.h"
+#if BUILDFLAG(IS_ANDROID)
+#include "components/browser_ui/share/android/intent_helper.h"
 #endif
 
 using content::BrowserThread;
@@ -54,57 +60,47 @@ using content::WebUIMessageHandler;
 namespace {
 
 // May only be accessed on the UI thread
-base::LazyInstance<base::FilePath>::Leaky
-    last_save_dir = LAZY_INSTANCE_INITIALIZER;
+base::LazyInstance<base::FilePath>::Leaky last_save_dir =
+    LAZY_INSTANCE_INITIALIZER;
 
-content::WebUIDataSource* CreateNetExportHTMLSource() {
-  content::WebUIDataSource* source =
-      content::WebUIDataSource::Create(chrome::kChromeUINetExportHost);
-
-  source->SetJsonPath("strings.js");
-  source->AddResourcePath(net_log::kNetExportUIJS, IDR_NET_LOG_NET_EXPORT_JS);
-  source->SetDefaultResource(IDR_NET_LOG_NET_EXPORT_HTML);
-  return source;
-}
-
-void SetIfNotNull(base::DictionaryValue* dict,
-                  const base::StringPiece& path,
-                  std::unique_ptr<base::Value> in_value) {
-  if (in_value) {
-    dict->Set(path, std::move(in_value));
-  }
+void CreateAndAddNetExportHTMLSource(Profile* profile) {
+  content::WebUIDataSource* source = content::WebUIDataSource::CreateAndAdd(
+      profile, chrome::kChromeUINetExportHost);
+  webui::SetupWebUIDataSource(source, kNetExportResources,
+                              IDR_NET_EXPORT_NET_EXPORT_HTML);
 }
 
 // This class receives javascript messages from the renderer.
 // Note that the WebUI infrastructure runs on the UI thread, therefore all of
 // this class's public methods are expected to run on the UI thread.
-class NetExportMessageHandler
+class NetExportMessageHandler final
     : public WebUIMessageHandler,
-      public base::SupportsWeakPtr<NetExportMessageHandler>,
       public ui::SelectFileDialog::Listener,
       public net_log::NetExportFileWriter::StateObserver {
  public:
   NetExportMessageHandler();
+
+  NetExportMessageHandler(const NetExportMessageHandler&) = delete;
+  NetExportMessageHandler& operator=(const NetExportMessageHandler&) = delete;
+
   ~NetExportMessageHandler() override;
 
   // WebUIMessageHandler implementation.
   void RegisterMessages() override;
 
   // Messages
-  void OnEnableNotifyUIWithState(const base::ListValue* list);
-  void OnStartNetLog(const base::ListValue* list);
-  void OnStopNetLog(const base::ListValue* list);
-  void OnSendNetLog(const base::ListValue* list);
-  void OnShowFile(const base::ListValue* list);
+  void OnEnableNotifyUIWithState(const base::Value::List& list);
+  void OnStartNetLog(const base::Value::List& list);
+  void OnStopNetLog(const base::Value::List& list);
+  void OnSendNetLog(const base::Value::List& list);
+  void OnShowFile(const base::Value::List& list);
 
   // ui::SelectFileDialog::Listener implementation.
-  void FileSelected(const base::FilePath& path,
-                    int index,
-                    void* params) override;
-  void FileSelectionCanceled(void* params) override;
+  void FileSelected(const ui::SelectedFileInfo& file, int index) override;
+  void FileSelectionCanceled() override;
 
   // net_log::NetExportFileWriter::StateObserver implementation.
-  void OnNewState(const base::DictionaryValue& state) override;
+  void OnNewState(const base::Value::Dict& state) override;
 
  private:
   // Send NetLog data via email.
@@ -127,20 +123,20 @@ class NetExportMessageHandler
   // UI.
   static bool UsingMobileUI();
 
-  // Calls NetExportView.onExportNetLogInfoChanged JavaScript function in the
+  // Fires net-log-info-changed event to update the JavaScript UI in the
   // renderer.
-  void NotifyUIWithState(std::unique_ptr<base::DictionaryValue> state);
+  void NotifyUIWithState(const base::Value::Dict& state);
 
   // Opens the SelectFileDialog UI with the default path to save a
   // NetLog file.
   void ShowSelectFileDialog(const base::FilePath& default_path);
 
   // Cached pointer to SystemNetworkContextManager's NetExportFileWriter.
-  net_log::NetExportFileWriter* file_writer_;
+  raw_ptr<net_log::NetExportFileWriter> file_writer_;
 
-  ScopedObserver<net_log::NetExportFileWriter,
-                 net_log::NetExportFileWriter::StateObserver>
-      state_observer_manager_;
+  base::ScopedObservation<net_log::NetExportFileWriter,
+                          net_log::NetExportFileWriter::StateObserver>
+      state_observation_manager_{this};
 
   // The capture mode and file size bound that the user chose in the UI when
   // logging started is cached here and is read after a file path is chosen in
@@ -152,24 +148,22 @@ class NetExportMessageHandler
   scoped_refptr<ui::SelectFileDialog> select_file_dialog_;
 
   base::WeakPtrFactory<NetExportMessageHandler> weak_ptr_factory_{this};
-
-  DISALLOW_COPY_AND_ASSIGN(NetExportMessageHandler);
 };
 
 NetExportMessageHandler::NetExportMessageHandler()
     : file_writer_(g_browser_process->system_network_context_manager()
-                       ->GetNetExportFileWriter()),
-      state_observer_manager_(this) {
+                       ->GetNetExportFileWriter()) {
   file_writer_->Initialize();
 }
 
 NetExportMessageHandler::~NetExportMessageHandler() {
   // There may be a pending file dialog, it needs to be told that the user
   // has gone away so that it doesn't try to call back.
-  if (select_file_dialog_)
+  if (select_file_dialog_) {
     select_file_dialog_->ListenerDestroyed();
+  }
 
-  file_writer_->StopNetLog(nullptr);
+  file_writer_->StopNetLog();
 }
 
 void NetExportMessageHandler::RegisterMessages() {
@@ -201,19 +195,18 @@ void NetExportMessageHandler::RegisterMessages() {
 // After this function, NotifyUIWithState() will be called on all |file_writer_|
 // state changes.
 void NetExportMessageHandler::OnEnableNotifyUIWithState(
-    const base::ListValue* list) {
+    const base::Value::List& list) {
+  AllowJavascript();
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (!state_observer_manager_.IsObservingSources()) {
-    state_observer_manager_.Add(file_writer_);
+  if (!state_observation_manager_.IsObserving()) {
+    state_observation_manager_.Observe(file_writer_.get());
   }
   NotifyUIWithState(file_writer_->GetState());
 }
 
-void NetExportMessageHandler::OnStartNetLog(const base::ListValue* list) {
+void NetExportMessageHandler::OnStartNetLog(const base::Value::List& params) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
-  const base::Value::ListStorage& params = list->GetList();
-
   // Determine the capture mode.
   capture_mode_ = net::NetLogCaptureMode::kDefault;
   if (!params.empty() && params[0].is_string()) {
@@ -230,86 +223,87 @@ void NetExportMessageHandler::OnStartNetLog(const base::ListValue* list) {
   if (UsingMobileUI()) {
     StartNetLog(base::FilePath());
   } else {
-    base::FilePath initial_dir = last_save_dir.Pointer()->empty() ?
-        DownloadPrefs::FromBrowserContext(
-            web_ui()->GetWebContents()->GetBrowserContext())->DownloadPath() :
-        *last_save_dir.Pointer();
+    base::FilePath initial_dir =
+        last_save_dir.Pointer()->empty()
+            ? DownloadPrefs::FromBrowserContext(
+                  web_ui()->GetWebContents()->GetBrowserContext())
+                  ->DownloadPath()
+            : *last_save_dir.Pointer();
     base::FilePath initial_path =
         initial_dir.Append(FILE_PATH_LITERAL("chrome-net-export-log.json"));
     ShowSelectFileDialog(initial_path);
   }
 }
 
-void NetExportMessageHandler::OnStopNetLog(const base::ListValue* list) {
+void NetExportMessageHandler::OnStopNetLog(const base::Value::List& list) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
 
-  std::unique_ptr<base::DictionaryValue> ui_thread_polled_data(
-      new base::DictionaryValue());
+  base::Value::Dict ui_thread_polled_data;
 
   Profile* profile = Profile::FromWebUI(web_ui());
-  SetIfNotNull(ui_thread_polled_data.get(), "prerenderInfo",
-               chrome_browser_net::GetPrerenderInfo(profile));
-  SetIfNotNull(ui_thread_polled_data.get(), "extensionInfo",
-               chrome_browser_net::GetExtensionInfo(profile));
-#if defined(OS_WIN)
-  SetIfNotNull(ui_thread_polled_data.get(), "serviceProviders",
-               chrome_browser_net::GetWindowsServiceProviders());
+  ui_thread_polled_data.Set("prerenderInfo",
+                            chrome_browser_net::GetPrerenderInfo(profile));
+  ui_thread_polled_data.Set("extensionInfo",
+                            chrome_browser_net::GetExtensionInfo(profile));
+#if BUILDFLAG(IS_WIN)
+  ui_thread_polled_data.Set("serviceProviders",
+                            chrome_browser_net::GetWindowsServiceProviders());
 #endif
 
   file_writer_->StopNetLog(std::move(ui_thread_polled_data));
 }
 
-void NetExportMessageHandler::OnSendNetLog(const base::ListValue* list) {
+void NetExportMessageHandler::OnSendNetLog(const base::Value::List& list) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   file_writer_->GetFilePathToCompletedLog(
-      base::Bind(&NetExportMessageHandler::SendEmail));
+      base::BindOnce(&NetExportMessageHandler::SendEmail));
 }
 
-void NetExportMessageHandler::OnShowFile(const base::ListValue* list) {
+void NetExportMessageHandler::OnShowFile(const base::Value::List& list) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   file_writer_->GetFilePathToCompletedLog(
-      base::Bind(&NetExportMessageHandler::ShowFileInShell, AsWeakPtr()));
+      base::BindOnce(&NetExportMessageHandler::ShowFileInShell,
+                     weak_ptr_factory_.GetWeakPtr()));
 }
 
-void NetExportMessageHandler::FileSelected(const base::FilePath& path,
-                                           int index,
-                                           void* params) {
+void NetExportMessageHandler::FileSelected(const ui::SelectedFileInfo& file,
+                                           int index) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   DCHECK(select_file_dialog_);
-  *last_save_dir.Pointer() = path.DirName();
+  *last_save_dir.Pointer() = file.path().DirName();
 
-  StartNetLog(path);
+  StartNetLog(file.path());
 
   // IMPORTANT: resetting the dialog may lead to the deletion of |path|, so keep
   // this line last.
   select_file_dialog_ = nullptr;
 }
 
-void NetExportMessageHandler::FileSelectionCanceled(void* params) {
+void NetExportMessageHandler::FileSelectionCanceled() {
   DCHECK(select_file_dialog_);
   select_file_dialog_ = nullptr;
 }
 
-void NetExportMessageHandler::OnNewState(const base::DictionaryValue& state) {
-  NotifyUIWithState(state.CreateDeepCopy());
+void NetExportMessageHandler::OnNewState(const base::Value::Dict& state) {
+  NotifyUIWithState(state);
 }
 
 // static
 void NetExportMessageHandler::SendEmail(const base::FilePath& file_to_send) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-#if defined(OS_ANDROID)
-  if (file_to_send.empty())
+#if BUILDFLAG(IS_ANDROID)
+  if (file_to_send.empty()) {
     return;
+  }
   std::string email;
   std::string subject = "net_internals_log";
   std::string title = "Issue number: ";
   std::string body =
       "Please add some informative text about the network issues.";
   base::FilePath::StringType file_to_attach(file_to_send.value());
-  chrome::android::SendEmail(
-      base::UTF8ToUTF16(email), base::UTF8ToUTF16(subject),
-      base::UTF8ToUTF16(body), base::UTF8ToUTF16(title),
-      base::UTF8ToUTF16(file_to_attach));
+  browser_ui::SendEmail(base::UTF8ToUTF16(email), base::UTF8ToUTF16(subject),
+                        base::UTF8ToUTF16(body), base::UTF8ToUTF16(title),
+                        base::UTF8ToUTF16(file_to_attach));
 #endif
 }
 
@@ -319,16 +313,17 @@ void NetExportMessageHandler::StartNetLog(const base::FilePath& path) {
   file_writer_->StartNetLog(
       path, capture_mode_, max_log_file_size_,
       base::CommandLine::ForCurrentProcess()->GetCommandLineString(),
-      chrome::GetChannelName(),
-      content::BrowserContext::GetDefaultStoragePartition(
-          Profile::FromWebUI(web_ui()))
+      chrome::GetChannelName(chrome::WithExtendedStable(true)),
+      Profile::FromWebUI(web_ui())
+          ->GetDefaultStoragePartition()
           ->GetNetworkContext());
 }
 
 void NetExportMessageHandler::ShowFileInShell(const base::FilePath& path) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
-  if (path.empty())
+  if (path.empty()) {
     return;
+  }
 
   // (The |profile| parameter is relevant for Chrome OS)
   Profile* profile = Profile::FromWebUI(web_ui());
@@ -338,7 +333,7 @@ void NetExportMessageHandler::ShowFileInShell(const base::FilePath& path) {
 
 // static
 bool NetExportMessageHandler::UsingMobileUI() {
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
   return true;
 #else
   return false;
@@ -346,30 +341,30 @@ bool NetExportMessageHandler::UsingMobileUI() {
 }
 
 void NetExportMessageHandler::NotifyUIWithState(
-    std::unique_ptr<base::DictionaryValue> state) {
+    const base::Value::Dict& state) {
   DCHECK_CURRENTLY_ON(BrowserThread::UI);
   DCHECK(web_ui());
-  web_ui()->CallJavascriptFunctionUnsafe(net_log::kOnExportNetLogInfoChanged,
-                                         *state);
+  FireWebUIListener(net_log::kNetLogInfoChangedEvent, state);
 }
 
 void NetExportMessageHandler::ShowSelectFileDialog(
     const base::FilePath& default_path) {
   // User may have clicked more than once before the save dialog appears.
   // This prevents creating more than one save dialog.
-  if (select_file_dialog_)
+  if (select_file_dialog_) {
     return;
+  }
 
   WebContents* webcontents = web_ui()->GetWebContents();
 
   select_file_dialog_ = ui::SelectFileDialog::Create(
       this, std::make_unique<ChromeSelectFilePolicy>(webcontents));
-  ui::SelectFileDialog::FileTypeInfo file_type_info;
-  file_type_info.extensions = {{FILE_PATH_LITERAL("json")}};
+  ui::SelectFileDialog::FileTypeInfo file_type_info{
+      {FILE_PATH_LITERAL("json")}};
   gfx::NativeWindow owning_window = webcontents->GetTopLevelNativeWindow();
   select_file_dialog_->SelectFile(
-      ui::SelectFileDialog::SELECT_SAVEAS_FILE, base::string16(), default_path,
-      &file_type_info, 0, base::FilePath::StringType(), owning_window, nullptr);
+      ui::SelectFileDialog::SELECT_SAVEAS_FILE, std::u16string(), default_path,
+      &file_type_info, 0, base::FilePath::StringType(), owning_window);
 }
 
 }  // namespace
@@ -378,6 +373,5 @@ NetExportUI::NetExportUI(content::WebUI* web_ui) : WebUIController(web_ui) {
   web_ui->AddMessageHandler(std::make_unique<NetExportMessageHandler>());
 
   // Set up the chrome://net-export/ source.
-  Profile* profile = Profile::FromWebUI(web_ui);
-  content::WebUIDataSource::Add(profile, CreateNetExportHTMLSource());
+  CreateAndAddNetExportHTMLSource(Profile::FromWebUI(web_ui));
 }
