@@ -4,16 +4,19 @@
 
 #include "chrome/browser/ui/webui/new_tab_page/untrusted_source.h"
 
+#include <optional>
 #include <string>
+#include <string_view>
 #include <utility>
 
 #include "base/base64.h"
+#include "base/containers/contains.h"
 #include "base/files/file_util.h"
 #include "base/i18n/rtl.h"
 #include "base/memory/ref_counted_memory.h"
 #include "base/memory/scoped_refptr.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/strings/string_piece.h"
+#include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/task/task_traits.h"
@@ -23,17 +26,19 @@
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/search/ntp_user_data_logger.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/grit/new_tab_page_resources.h"
+#include "chrome/grit/new_tab_page_untrusted_resources.h"
+#include "components/policy/content/policy_blocklist_service.h"
 #include "components/search/ntp_features.h"
 #include "content/public/common/url_constants.h"
 #include "net/base/url_util.h"
 #include "services/network/public/mojom/content_security_policy.mojom.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "third_party/re2/src/re2/re2.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/base/template_expressions.h"
 #include "url/url_util.h"
 
+using URLBlocklistState = policy::URLBlocklist::URLBlocklistState;
+
 namespace {
 
 constexpr int kMaxUriDecodeLen = 2048;
@@ -43,17 +48,14 @@ std::string FormatTemplate(int resource_id,
   ui::ResourceBundle& bundle = ui::ResourceBundle::GetSharedInstance();
   scoped_refptr<base::RefCountedMemory> bytes =
       bundle.LoadDataResourceBytes(resource_id);
-  base::StringPiece string_piece(reinterpret_cast<const char*>(bytes->front()),
-                                 bytes->size());
   return ui::ReplaceTemplateExpressions(
-      string_piece, replacements,
+      base::as_string_view(*bytes), replacements,
       /* skip_unexpected_placeholder_check= */ true);
 }
 
-std::string ReadBackgroundImageData(const base::FilePath& profile_path) {
+std::string ReadBackgroundImageData(const base::FilePath& path) {
   std::string data_string;
-  base::ReadFileToString(profile_path.AppendASCII("background.jpg"),
-                         &data_string);
+  base::ReadFileToString(path, &data_string);
   return data_string;
 }
 
@@ -86,7 +88,7 @@ std::string UntrustedSource::GetContentSecurityPolicy(
     case network::mojom::CSPDirectiveName::ChildSrc:
       return "child-src https:;";
     case network::mojom::CSPDirectiveName::DefaultSrc:
-      // TODO(https://crbug.com/1085325): Audit and tighten CSP.
+      // TODO(crbug.com/40693567): Audit and tighten CSP.
       return std::string();
     case network::mojom::CSPDirectiveName::FrameAncestors:
       return base::StringPrintf("frame-ancestors %s",
@@ -110,20 +112,18 @@ void UntrustedSource::StartDataRequest(
     const GURL& url,
     const content::WebContents::Getter& wc_getter,
     content::URLDataSource::GotDataCallback callback) {
-  const std::string path = url.has_path() ? url.path().substr(1) : "";
   GURL url_param = GURL(url.query());
+  if (url_param.is_valid() && IsURLBlockedByPolicy(url_param)) {
+    std::move(callback).Run(base::MakeRefCounted<base::RefCountedString>());
+    return;
+  }
+  const std::string path = url.has_path() ? url.path().substr(1) : "";
   if (path == "one-google-bar" && one_google_bar_service_) {
     std::string query_params;
     net::GetValueForKeyInQuery(url, "paramsencoded", &query_params);
     base::Base64Decode(query_params, &query_params);
-    bool wait_for_refresh =
-        one_google_bar_service_->SetAdditionalQueryParams(query_params);
+    one_google_bar_service_->SetAdditionalQueryParams(query_params);
     one_google_bar_callbacks_.push_back(std::move(callback));
-    if (one_google_bar_service_->one_google_bar_data().has_value() &&
-        !wait_for_refresh &&
-        base::FeatureList::IsEnabled(ntp_features::kCacheOneGoogleBar)) {
-      OnOneGoogleBarDataUpdated();
-    }
     if (one_google_bar_callbacks_.size() == 1) {
       one_google_bar_load_start_time_ = base::TimeTicks::Now();
       one_google_bar_service_->Refresh();
@@ -153,17 +153,16 @@ void UntrustedSource::StartDataRequest(
   if (path == "custom_background_image") {
     // Parse all query parameters to hash map and decode values.
     std::unordered_map<std::string, std::string> params;
-    url::Component query(0, url.query().length());
+    std::string_view query_piece = url.query_piece();
+    url::Component query(0, url.query_piece().length());
     url::Component key, value;
-    while (
-        url::ExtractQueryKeyValue(url.query().c_str(), &query, &key, &value)) {
+    while (url::ExtractQueryKeyValue(query_piece, &query, &key, &value)) {
       url::RawCanonOutputW<kMaxUriDecodeLen> output;
-      url::DecodeURLEscapeSequences(
-          url.query().c_str() + value.begin, value.len,
-          url::DecodeURLMode::kUTF8OrIsomorphic, &output);
-      params.insert(
-          {url.query().substr(key.begin, key.len),
-           base::UTF16ToUTF8(std::u16string(output.data(), output.length()))});
+      url::DecodeURLEscapeSequences(query_piece.substr(value.begin, value.len),
+                                    url::DecodeURLMode::kUTF8OrIsomorphic,
+                                    &output);
+      params.insert({std::string(query_piece.substr(key.begin, key.len)),
+                     base::UTF16ToUTF8(output.view())});
     }
     // Extract desired values.
     ServeBackgroundImage(
@@ -183,10 +182,11 @@ void UntrustedSource::StartDataRequest(
         IDR_NEW_TAB_PAGE_UNTRUSTED_BACKGROUND_IMAGE_JS));
     return;
   }
-  if (path == "background.jpg") {
+  if (base::EndsWith(path, "background.jpg")) {
     base::ThreadPool::PostTaskAndReplyWithResult(
         FROM_HERE, {base::TaskPriority::USER_VISIBLE, base::MayBlock()},
-        base::BindOnce(&ReadBackgroundImageData, profile_->GetPath()),
+        base::BindOnce(&ReadBackgroundImageData,
+                       profile_->GetPath().AppendASCII(path)),
         base::BindOnce(&ServeBackgroundImageData, std::move(callback)));
     return;
   }
@@ -194,7 +194,7 @@ void UntrustedSource::StartDataRequest(
 }
 
 std::string UntrustedSource::GetMimeType(const GURL& url) {
-  const base::StringPiece stripped_path = url.path_piece();
+  const std::string_view stripped_path = url.path_piece();
   if (base::EndsWith(stripped_path, ".js",
                      base::CompareCase::INSENSITIVE_ASCII)) {
     return "application/javascript";
@@ -230,18 +230,18 @@ bool UntrustedSource::ShouldServiceRequest(
   return path == "one-google-bar" || path == "one_google_bar.js" ||
          path == "image" || path == "background_image" ||
          path == "custom_background_image" || path == "background_image.js" ||
-         path == "background.jpg";
+         base::Contains(path, "background.jpg");
 }
 
 void UntrustedSource::OnOneGoogleBarDataUpdated() {
-  absl::optional<OneGoogleBarData> data =
+  std::optional<OneGoogleBarData> data =
       one_google_bar_service_->one_google_bar_data();
 
   if (one_google_bar_load_start_time_.has_value()) {
     NTPUserDataLogger::LogOneGoogleBarFetchDuration(
         /*success=*/data.has_value(),
         /*duration=*/base::TimeTicks::Now() - *one_google_bar_load_start_time_);
-    one_google_bar_load_start_time_ = absl::nullopt;
+    one_google_bar_load_start_time_ = std::nullopt;
   }
 
   std::string html;
@@ -249,6 +249,11 @@ void UntrustedSource::OnOneGoogleBarDataUpdated() {
     ui::TemplateReplacements replacements;
     replacements["textdirection"] = base::i18n::IsRTL() ? "rtl" : "ltr";
     replacements["barHtml"] = data->bar_html;
+    replacements["varsHeadScript"] = base::StringPrintf(
+        "let abp = %s;", base::FeatureList::IsEnabled(
+                             ntp_features::kNtpOneGoogleBarAsyncBarParts)
+                             ? "true"
+                             : "false");
     replacements["inHeadScript"] = data->in_head_script;
     replacements["inHeadStyle"] = data->in_head_style;
     replacements["afterBarScript"] = data->after_bar_script;
@@ -282,16 +287,15 @@ void UntrustedSource::ServeBackgroundImage(
     const std::string& position_y,
     const std::string& scrim_display,
     content::URLDataSource::GotDataCallback callback) {
-  if (!url.is_valid() || !(url.SchemeIs(url::kHttpsScheme) ||
-                           url.SchemeIs(content::kChromeUIUntrustedScheme))) {
+  if (!IsURLAllowed(url)) {
     std::move(callback).Run(base::MakeRefCounted<base::RefCountedString>());
     return;
   }
   ui::TemplateReplacements replacements;
   replacements["url"] = url.spec();
-  if (url_2x.is_valid()) {
+  if (IsURLAllowed(url_2x)) {
     replacements["backgroundUrl"] =
-        base::StringPrintf("-webkit-image-set(url(%s) 1x, url(%s) 2x)",
+        base::StringPrintf("image-set(url(%s) 1x, url(%s) 2x)",
                            url.spec().c_str(), url_2x.spec().c_str());
   } else {
     replacements["backgroundUrl"] =
@@ -307,3 +311,25 @@ void UntrustedSource::ServeBackgroundImage(
       base::MakeRefCounted<base::RefCountedString>(FormatTemplate(
           IDR_NEW_TAB_PAGE_UNTRUSTED_BACKGROUND_IMAGE_HTML, replacements)));
 }
+
+bool UntrustedSource::IsURLAllowed(const GURL& url) {
+  if (!url.is_valid() ||
+      !(url.SchemeIs(url::kHttpsScheme) ||
+        url.SchemeIs(content::kChromeUIUntrustedScheme)) ||
+      IsURLBlockedByPolicy(url)) {
+    LOG(WARNING) << "URL is not allowed.";
+    return false;
+  }
+  return true;
+}
+
+bool UntrustedSource::IsURLBlockedByPolicy(const GURL& url) {
+  PolicyBlocklistService* service =
+      PolicyBlocklistFactory::GetForBrowserContext(profile_);
+  URLBlocklistState blocklist_state = service->GetURLBlocklistState(url);
+  if (blocklist_state == URLBlocklistState::URL_IN_BLOCKLIST) {
+    LOG(WARNING) << "URL is blocked by a policy.";
+    return true;
+  }
+  return false;
+}
