@@ -1,25 +1,25 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/ui/webui/signin/signin_error_handler.h"
 
-#include "base/bind.h"
-#include "chrome/browser/profiles/profile_manager.h"
+#include "base/functional/bind.h"
 #include "chrome/browser/profiles/profile_window.h"
 #include "chrome/browser/signin/signin_ui_util.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_window.h"
-#include "chrome/browser/ui/user_manager.h"
+#include "chrome/browser/ui/profiles/profile_picker.h"
 #include "chrome/browser/ui/webui/signin/signin_utils.h"
 #include "content/public/browser/web_ui.h"
 #include "url/gurl.h"
 
-SigninErrorHandler::SigninErrorHandler(Browser* browser, bool is_system_profile)
-    : browser_(browser), is_system_profile_(is_system_profile) {
+SigninErrorHandler::SigninErrorHandler(Browser* browser,
+                                       bool from_profile_picker)
+    : browser_(browser), from_profile_picker_(from_profile_picker) {
   // |browser_| must not be null when this dialog is presented from the
-  // user manager.
-  DCHECK(browser_ || is_system_profile_);
+  // profile picker.
+  DCHECK(browser_ || from_profile_picker_);
   BrowserList::AddObserver(this);
 }
 
@@ -28,8 +28,9 @@ SigninErrorHandler::~SigninErrorHandler() {
 }
 
 void SigninErrorHandler::OnBrowserRemoved(Browser* browser) {
-  if (browser_ == browser)
+  if (browser_ == browser) {
     browser_ = nullptr;
+  }
 }
 
 void SigninErrorHandler::RegisterMessages() {
@@ -40,7 +41,7 @@ void SigninErrorHandler::RegisterMessages() {
       "switchToExistingProfile",
       base::BindRepeating(&SigninErrorHandler::HandleSwitchToExistingProfile,
                           base::Unretained(this)));
-  if (!is_system_profile_) {
+  if (!from_profile_picker_) {
     web_ui()->RegisterMessageCallback(
         "learnMore", base::BindRepeating(&SigninErrorHandler::HandleLearnMore,
                                          base::Unretained(this)));
@@ -52,9 +53,10 @@ void SigninErrorHandler::RegisterMessages() {
 }
 
 void SigninErrorHandler::HandleSwitchToExistingProfile(
-    const base::ListValue* args) {
-  if (duplicate_profile_path_.empty())
+    const base::Value::List& args) {
+  if (duplicate_profile_path_.empty()) {
     return;
+  }
 
   // CloseDialog will eventually destroy this object, so nothing should access
   // its members after this call. However, closing the dialog may steal focus
@@ -65,43 +67,35 @@ void SigninErrorHandler::HandleSwitchToExistingProfile(
 
   // Switch to the existing duplicate profile. Do not create a new window when
   // any existing ones can be reused.
-  profiles::SwitchToProfile(path_switching_to, false,
-                            ProfileManager::CreateCallback());
+  profiles::SwitchToProfile(path_switching_to, false);
 }
 
-void SigninErrorHandler::HandleConfirm(const base::ListValue* args) {
+void SigninErrorHandler::HandleConfirm(const base::Value::List& args) {
   CloseDialog();
 }
 
-void SigninErrorHandler::HandleLearnMore(const base::ListValue* args) {
-  // "Learn more" only shown when is_system_profile_=false
-  DCHECK(!is_system_profile_);
-  if (!browser_)
+void SigninErrorHandler::HandleLearnMore(const base::Value::List& args) {
+  // "Learn more" only shown when from_profile_picker_=false
+  DCHECK(!from_profile_picker_);
+  if (!browser_) {
     return;
+  }
   CloseDialog();
   signin_ui_util::ShowSigninErrorLearnMorePage(browser_->profile());
 }
 
 void SigninErrorHandler::HandleInitializedWithSize(
-    const base::ListValue* args) {
+    const base::Value::List& args) {
   AllowJavascript();
-  if (duplicate_profile_path_.empty())
+  if (duplicate_profile_path_.empty()) {
     FireWebUIListener("switch-button-unavailable");
+  }
 
   signin::SetInitializedModalHeight(browser_, web_ui(), args);
-
-  // After the dialog is shown, some platforms might have an element focused.
-  // To be consistent, clear the focused element on all platforms.
-  // TODO(anthonyvd): Figure out why this is needed on Mac and not other
-  // platforms and if there's a way to start unfocused while avoiding this
-  // workaround.
-  FireWebUIListener("clear-focus");
 }
 
 void SigninErrorHandler::CloseDialog() {
-  if (is_system_profile_) {
-    CloseUserManagerProfileDialog();
-  } else if (browser_){
+  if (browser_) {
     CloseBrowserModalSigninDialog();
   }
 }
@@ -109,7 +103,3 @@ void SigninErrorHandler::CloseDialog() {
 void SigninErrorHandler::CloseBrowserModalSigninDialog() {
   browser_->signin_view_controller()->CloseModalSignin();
 }
-
-void SigninErrorHandler::CloseUserManagerProfileDialog() {
-  UserManagerProfileDialog::HideDialog();
-}
